// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/BiDirIIOPInit.h>

#include <Hello_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif


int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);

    //
    // Obtain the ORB's property set
    //
    OB::Properties_var props = oborb -> properties();

    //
    // We must use a threaded concurrency model with the bi-directional
    // plug-in
    //
    props -> setProperty("ooc.orb.oa.conc_model", "threaded");

    //
    // Define our peer id
    //
    props -> setProperty("ooc.bidir.peer", "HelloServer");

    //
    // Initialize the bi-directional plug-in for IIOP and create
    // the RootPOAManager (the POAManager for the RootPOA). This
    // must be done prior to resolving the RootPOA.
    //
    PortableServer::POAManager_var manager =
        OBBiDir::IIOP::createPOAManager("RootPOAManager", orb, argc, argv,
                                        props);

    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(poaObj);

    //
    // Create implementation object and activate it on the RootPOA
    //
    Hello_impl* helloImpl = new Hello_impl(orb, root);
    PortableServer::ServantBase_var servant = helloImpl;
    Hello_var hello = helloImpl -> _this();

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(hello);

    const char* refFile = "Hello.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "': "
             << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        orb = CORBA::ORB_init(argc, argv);
        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
