// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OCI_BIDIR_IMPL_H
#define OCI_BIDIR_IMPL_H

#include <OB/OCI_BiDir.h>
#include <OB/BiDirIOP.h>

#include <ActiveTransport.h>
#include <TransportManager_fwd.h>

namespace OCI
{

namespace BiDir
{

//
// Declare classes
//
class AccFactoryInfo_impl;
class AccFactory_impl;
class AcceptorInfo_impl;
class Acceptor_impl;
class CallbackAcceptor_impl;
class CallbackTransportInfo_impl;
class CallbackTransport_impl;
class Connector_impl;
class ConnectorInfo_impl;
class ConFactory_impl;
class ConFactoryInfo_impl;
class SelfTransport_impl;
class SelfTransportInfo_impl;
class Transport_impl;
class TransportInfo_impl;

class AccFactoryInfo_impl : public AccFactoryInfo
{
    //
    // Hide copy-constructor and assignment operator
    //
    AccFactoryInfo_impl(const AccFactoryInfo_impl&);
    void operator=(const AccFactoryInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // AccFactory information delegate
    //
    OCI::AccFactoryInfo_var delegate_;

public:
    AccFactoryInfo_impl(OCI::ProtocolId, OCI::ProfileId,
                        OCI::AccFactoryInfo_ptr);
    virtual ~AccFactoryInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);
};

class AccFactory_impl : public OCI::AccFactory
{
    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The acceptor factory delegate
    //
    OCI::AccFactory_var delegate_;

    //
    // AccFactory information
    //
    AccFactoryInfo_impl* info_;

public:
    AccFactory_impl(TransportManager_ptr, OCI::AccFactory_ptr);
    virtual ~AccFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::AccFactoryInfo_ptr get_info()
        throw(CORBA::SystemException);

    virtual OCI::Acceptor_ptr create_acceptor(const OCI::ParamSeq&)
        throw(OCI::InvalidParam,
              CORBA::SystemException);
};

class AcceptorInfo_impl : public AcceptorInfo, public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    AcceptorInfo_impl(const AcceptorInfo_impl&);
    void operator=(const AcceptorInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // Acceptor information delegate
    //
    OCI::AcceptorInfo_var delegate_;

    //
    // All accept callback objects
    //
    OB_STL::list< OCI::AcceptCB_var > acceptCBs_;

public:

    AcceptorInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                      OCI::AcceptorInfo_ptr);
    ~AcceptorInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_accept_cb(OCI::AcceptCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_accept_cb(OCI::AcceptCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callAcceptCB(OCI::TransportInfo_ptr);
};

class Acceptor_impl : public OCI::Acceptor
{
    //
    // Hide copy-constructor and assignment operator
    //
    Acceptor_impl(const Acceptor_impl&);
    void operator=(const Acceptor_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The Acceptor delegate
    //
    OCI::Acceptor_var delegate_;

    //
    // Acceptor information
    //
    AcceptorInfo_impl* info_;

public:
    Acceptor_impl(TransportManager_ptr, OCI::Acceptor_ptr);
    ~Acceptor_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void listen()
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr accept(CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr connect_self()
        throw(CORBA::SystemException);

    virtual void add_profiles(const OCI::ProfileInfo&, OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::ProfileInfoSeq* get_local_profiles(const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class CallbackAcceptor_impl : public OCI::Acceptor
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackAcceptor_impl(const CallbackAcceptor_impl&);
    void operator=(const CallbackAcceptor_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // Acceptor information
    //
    AcceptorInfo_impl* info_;

    //
    // Transports waiting to be accepted
    //
    OB_STL::list< OCI::Transport_var > transports_;
    JTCMonitor transportsMonitor_;

public:
    CallbackAcceptor_impl(TransportManager_ptr);
    ~CallbackAcceptor_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void listen()
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr accept(CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr connect_self()
        throw(CORBA::SystemException);

    virtual void add_profiles(const OCI::ProfileInfo&, OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::ProfileInfoSeq* get_local_profiles(const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_connect(OCI::Transport_ptr);
};

class CallbackTransportInfo_impl : public TransportInfo,
                                   public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackTransportInfo_impl(const CallbackTransportInfo_impl&);
    void operator=(const CallbackTransportInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // Connector information
    //
    OCI::ConnectorInfo_var connectorInfo_;

    //
    // Acceptor information
    //
    OCI::AcceptorInfo_var acceptorInfo_;

    //
    // The Transport information delegate
    //
    OCI::TransportInfo_var delegate_;

    //
    // All close callback objects
    //
    OB_STL::list<OCI::CloseCB_var> closeCBs_;

public:

    CallbackTransportInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                               const char*, OCI::Connector_ptr,
                               OCI::TransportInfo_ptr);
    CallbackTransportInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                               const char*, OCI::Acceptor_ptr,
                               OCI::TransportInfo_ptr);
    ~CallbackTransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callCloseCB(OCI::TransportInfo_ptr);
};

class CallbackTransport_impl : public OCI::Transport, public ShutdownStatus
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackTransport_impl(const CallbackTransport_impl&);
    void operator=(const CallbackTransport_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The active transport
    //
    ActiveTransport_var transport_;

    //
    // Transport information
    //
    CallbackTransportInfo_impl* info_;

    //
    // Have we been shutdown?
    //
    bool shutdown_;

public:

    CallbackTransport_impl(OCI::ProtocolId, OCI::ProfileId,
                           ActiveTransport_ptr, OCI::Connector_ptr);
    CallbackTransport_impl(OCI::ProtocolId, OCI::ProfileId,
                           ActiveTransport_ptr, OCI::Acceptor_ptr);
    ~CallbackTransport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ShutdownStatus
    //
    bool isShutdown() { return shutdown_; }
};

class ConFactoryInfo_impl : public ConFactoryInfo, public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConFactoryInfo_impl(const ConFactoryInfo_impl&);
    void operator=(const ConFactoryInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // Acceptor information delegate
    //
    OCI::ConFactoryInfo_var delegate_;

    //
    // All connect callback objects
    //
    OB_STL::list<OCI::ConnectCB_var> connectCBs_;

public:

    ConFactoryInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                        OCI::ConFactoryInfo_ptr);
    ~ConFactoryInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_connect_cb(OCI::ConnectCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_connect_cb(OCI::ConnectCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    OCI::ConnectCBSeq* _OB_getConnectCBSeq();
};

class ConFactory_impl : public OCI::ConFactory
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConFactory_impl(const ConFactory_impl&);
    void operator=(const ConFactory_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The ConFactory delegate
    //
    OCI::ConFactory_var delegate_;

    //
    // ConFactory information
    //
    ConFactoryInfo_impl* info_;

    bool compareBodies(const BiDirIOP::ProfileBody&,
                       const BiDirIOP::ProfileBody&);

public:

    ConFactory_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                    OCI::ConFactory_ptr);
    ~ConFactory_impl();

    TransportManager_ptr transportManager();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorSeq* create_connectors(const OCI::IOR&,
                                                 const CORBA::PolicyList&)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equivalent(const OCI::IOR&, const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual CORBA::ULong hash(const OCI::IOR&, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::ConFactoryInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class ConnectorInfo_impl : public ConnectorInfo, public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectorInfo_impl(const ConnectorInfo_impl&);
    void operator=(const ConnectorInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // The ConnectorInfo delegate
    //
    OCI::ConnectorInfo_var delegate_;

    //
    // All connect callback objects
    //
    OB_STL::list< ConnectCB_var > connectCBs_;

public:

    ConnectorInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                       const char*, const OCI::ConnectCBSeq&,
                       OCI::ConnectorInfo_ptr);
    ~ConnectorInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_connect_cb(OCI::ConnectCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_connect_cb(OCI::ConnectCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callConnectCB(OCI::TransportInfo_ptr);
};

class Connector_impl : public OCI::Connector
{
    //
    // Hide copy-constructor and assignment operator
    //
    Connector_impl(const Connector_impl&);
    void operator=(const Connector_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The profile body
    //
    BiDirIOP::ProfileBody profile_;

    //
    // The Connector delegate
    //
    OCI::Connector_var delegate_;

    //
    // Connector information
    //
    ConnectorInfo_impl* info_;

public:

    Connector_impl(TransportManager_ptr, const BiDirIOP::ProfileBody&,
                   const OCI::ConnectCBSeq&, OCI::Connector_ptr);
    ~Connector_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr connect()
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr connect_timeout(CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::ProfileInfoSeq* get_usable_profiles(const OCI::IOR&,
                                                     const CORBA::PolicyList&)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(OCI::Connector_ptr)
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class SelfTransportInfo_impl : public TransportInfo
{
    //
    // Hide copy-constructor and assignment operator
    //
    SelfTransportInfo_impl(const SelfTransportInfo_impl&);
    void operator=(const SelfTransportInfo_impl&);

public:

    SelfTransportInfo_impl();
    ~SelfTransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);
};

class SelfTransport_impl : public OCI::Transport, public JTCMonitor
{
    //
    // Hide copy-constructor and assignment operator
    //
    SelfTransport_impl(const SelfTransport_impl&);
    void operator=(const SelfTransport_impl&);

    //
    // The transport info
    //
    SelfTransportInfo_impl* info_;

    //
    // Have we been closed?
    //
    bool closed_;

public:

    SelfTransport_impl();
    ~SelfTransport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class TransportInfo_impl : public TransportInfo, public JTCRecursiveMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    TransportInfo_impl(const TransportInfo_impl&);
    void operator=(const TransportInfo_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // Connector information
    //
    OCI::ConnectorInfo_var connectorInfo_;

    //
    // Acceptor information
    //
    OCI::AcceptorInfo_var acceptorInfo_;

    //
    // The Transport information delegate
    //
    OCI::TransportInfo_var delegate_;

    //
    // All close callback objects
    //
    OB_STL::list<OCI::CloseCB_var> closeCBs_;

public:

    TransportInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                       const char*, OCI::Connector_ptr,
                       OCI::TransportInfo_ptr);
    TransportInfo_impl(OCI::ProtocolId, OCI::ProfileId, const char*,
                       const char*, OCI::Acceptor_ptr, OCI::TransportInfo_ptr);
    ~TransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(OCI::CloseCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callCloseCB(OCI::TransportInfo_ptr);
};

class Transport_impl : public OCI::Transport
{
    //
    // Hide copy-constructor and assignment operator
    //
    Transport_impl(const Transport_impl&);
    void operator=(const Transport_impl&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    OCI::ProfileId tag_;

    //
    // The active transport
    //
    ActiveTransport_var transport_;

    //
    // Transport information
    //
    TransportInfo_impl* info_;

public:

    Transport_impl(OCI::ProtocolId, OCI::ProfileId, ActiveTransport_ptr,
                   OCI::Connector_ptr);
    Transport_impl(OCI::ProtocolId, OCI::ProfileId, ActiveTransport_ptr,
                   OCI::Acceptor_ptr);
    ~Transport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException);

    virtual OCI::ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(OCI::Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(OCI::Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);
};

} // End of namespace BiDir

} // End of namespace OCI

#endif
