// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OCI_BIDIR_TRANSPORT_MANAGER_H
#define OCI_BIDIR_TRANSPORT_MANAGER_H

#include <OB/OCI_fwd.h>
#include <OB/IOP.h>
#include <OB/Hashtable.h>
#include <TransportManager_fwd.h>
#include <ActiveTransport.h>

namespace OCI
{

namespace BiDir
{

class CallbackAcceptor_impl;

class TransportManager : public OB::RefCount, public TransportObserver
{
    //
    // Hide copy-constructor and assignment operator
    //
    TransportManager(const TransportManager&);
    void operator=(const TransportManager&);

    //
    // The protocol id
    //
    OCI::ProtocolId id_;

    //
    // The profile tag
    //
    IOP::ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // A map of remote peer id to ActiveTransport
    //
    typedef OB::Hashtable<CORBA::String_var, ActiveTransport_var,
        OB::StringHasher, OB::LockStrategyNull> TransportMap;
    TransportMap transportMap_;
    JTCMonitor transportMapMonitor_;

    //
    // The callback acceptor
    //
    CallbackAcceptor_impl* callbackAcceptor_;
    JTCMonitor callbackAcceptorMonitor_;

public:
    TransportManager(OCI::ProtocolId, IOP::ProfileId, const char*);
    virtual ~TransportManager();

    static inline TransportManager_ptr _duplicate(TransportManager_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline TransportManager_ptr _nil()
    { return 0; }

    OCI::ProtocolId id() const { return id_; }
    IOP::ProfileId tag() const { return tag_; }
    const char* peer() const { return localPeer_.in(); }

    OCI::Transport_ptr addIncomingTransport(OCI::Acceptor_ptr,
                                            OCI::Transport_ptr);

    OCI::Transport_ptr connect(const char*, OCI::Connector_ptr,
                               OCI::Connector_ptr);
    OCI::Transport_ptr connectTimeout(const char*, OCI::Connector_ptr,
                                      OCI::Connector_ptr, CORBA::ULong);

    void setCallbackAcceptor(CallbackAcceptor_impl*);

    //
    // TransportObserver methods
    //
    virtual void callbackConnection(ActiveTransport_ptr);
    virtual void shutdown(ActiveTransport_ptr);
};

} // End of namespace BiDir

} // End of namespace OCI

namespace CORBA
{

inline void
release(OCI::BiDir::TransportManager_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(OCI::BiDir::TransportManager_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
