// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
#ifndef EVENT_CHANNEL_FACTORY_IMPL_H
#define EVENT_CHANNEL_FACTORY_IMPL_H

//
// Forward declarations
//

class EC_EventChannel_impl;
class Reaper;

//
// The event channel factory implementation
//
class EventChannelFactory_impl
    : virtual public PortableServer::RefCountServantBase,
      public POA_OBEventChannelFactory::EventChannelFactory,
      public JTCMonitor
{
    //
    // No copy constructor/assignment operator
    //
    EventChannelFactory_impl(const EventChannelFactory_impl&);
    void operator=(const EventChannelFactory_impl&);

public: // For some compilers this must be public

    typedef OB::Hashtable<CORBA::String_var, EC_EventChannel_impl*,
	OB::StringHasher, OB::LockStrategySynchronized> EventChannelMap;

private:

    CORBA::ORB_var orb_; // The ORB
    PortableServer::POA_var poa_; // The factory POA

    OB::EventLogger_var logger_; // The logger
    Reaper* reaper_; // The proxy reaper

    //
    // Mapping from CORBA::String_var to EC_EventChannel_impl*
    //
    EventChannelMap* channels_;

    void reap();
    
public:

    EventChannelFactory_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
    			     OB::EventLogger_ptr, Reaper*);
    ~EventChannelFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // Create EventChannel object
    //
    virtual CosEventChannelAdmin::EventChannel_ptr
    create_channel(const char*)
	throw(OBEventChannelFactory::ChannelAlreadyExists,
	      CORBA::SystemException);

    //
    // Get EventChannel object by id
    //
    virtual CosEventChannelAdmin::EventChannel_ptr
    get_channel_by_id(const char*)
	throw(OBEventChannelFactory::ChannelNotAvailable,
	      CORBA::SystemException);

    //
    // Get EventChannel object ids
    //
    virtual OBEventChannelFactory::ChannelIdSeq* get_channels()
	throw(CORBA::SystemException);

    //
    // Shutdown the event service.
    //
    virtual void shutdown()
	throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr _default_POA();
};

#endif
