 // **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <EventLogger.h>
#include <EventUtil.h>
#include <PipeSegment.h>
#include <QoS.h>

#include <ProxyBase.h>

#include <stdlib.h> // for atoi, ...

#ifdef WIN32
#   include <sys/timeb.h>
#endif

#include <time.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// ProxyBase protected members
// ----------------------------------------------------------------------

void
ProxyBase::resolveQoSParams(QoSParameters& params)
{
    OBCORBA::ORB_var orb = OBCORBA::ORB::_narrow(orb_);
    OB::Properties_var properties = orb -> properties();
    params.retryTimeout =
        atoi(properties -> getProperty(EventQoS::RETRY_TIMEOUT));
    params.retryMultiplier =
        atof(properties -> getProperty(EventQoS::RETRY_MULTIPLIER));
    params.maxRetries =
        atoi(properties -> getProperty(EventQoS::MAX_RETRIES));
    params.maxEvents =
        atoi(properties -> getProperty(EventQoS::MAX_EVENTS));
    params.pullInterval =
        atoi(properties -> getProperty(EventQoS::PULL_INTERVAL));
}

void
ProxyBase::init(ProxyConstructorInfo& info)
{
    orb_ = info.orb;
    poa_ = info.poa;
    logger_ = info.logger;
    id_ = info.id;
    parentPipe_ = info.pipe.in();

    name_ = info.base;
    name_ += '-';
    name_ += id_;

    if(logger_ -> traceLifecycle() >= 2)
    {
        String_var desc = EventUtil::getConnectionAddrDescription(orb_);
        String_var msg = string_dup("Create ");
	msg += name_;
        msg += " from ";
        msg += desc;
        logger_ -> trace("event", msg);
    }
}

void
ProxyBase::disconnectImpl()
{
    if(state_ == Destroyed)
	return;

    state_ = Destroyed;

    //
    // Destroy the supplier event pipe
    //
    if(myPipe_)
	myPipe_ -> destroy();

    disconnectPeer();
    deactivateObject();
}

void
ProxyBase::discardRequestsImpl()
{
    if(state_ == Destroyed || state_ == Discarding)
	return;

    state_ = Discarding;

    //
    // Destroy the supplier event pipe
    //
    if(myPipe_)
	myPipe_ -> destroy();
}

// ----------------------------------------------------------------------
// ProxyBase constructor/destructor
// ----------------------------------------------------------------------

ProxyBase::ProxyBase()
    : lastRequestTime_(time(0)),
      state_(Disconnected)
{
}

ProxyBase::~ProxyBase()
{
    if(logger_ -> traceLifecycle() >= 2)
    {
        String_var msg = string_dup("Destroy ");
	msg += name_;
	logger_ -> trace("event", msg);
    }
}

// ----------------------------------------------------------------------
// ProxyBase public members
// ----------------------------------------------------------------------

void
ProxyBase::disconnect()
{
    JTCSynchronized sync(*this);

    disconnectImpl();
}

void
ProxyBase::discardRequests()
{
    JTCSynchronized sync(*this);

    discardRequestsImpl();
}

bool
ProxyBase::destroyed()
{
    JTCSynchronized sync(*this);

    return state_ == Destroyed;
}

void
ProxyBase::stampRequest()
{
    lastRequestTime_ = time(0);
}

PortableServer::POA_ptr
ProxyBase::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}
