// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <Broadcaster_impl.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace chat;

//
// Constructor/Destructor
//

Broadcaster_impl::Broadcaster_impl(BroadcasterAdmin_impl* admin,
				   CORBA::ULong id)
    : admin_(admin), id_(id)
{
    // Only initializers used
}

Broadcaster_impl::~Broadcaster_impl()
{
    // Empty 
}

//
// Operations from the .idl file
//

void
Broadcaster_impl::setNickName(const char* nick)
    throw(NickExists, CORBA::SystemException)
{
    assert(admin_ != 0);
    admin_ -> setNickName(id_, nick);
}


void
Broadcaster_impl::say(const char* text)
    throw(CORBA::SystemException)
{
    assert(admin_ != 0);
    admin_ -> say(text);
}


void
Broadcaster_impl::unsubscribe()
    throw(CORBA::SystemException)
{
    assert(admin_ != 0);
    try
    {
        //
        // Clean up internal memory
        //
        admin_ -> unregister(id_);

        //
        // Use _default_POA (= rootPOA) for deactivation as object
        // was created with _this() operation
        //
        PortableServer::POA_var poa = _default_POA();
        PortableServer::ObjectId_var oid = poa -> servant_to_id(this);
        poa -> deactivate_object(oid);

        this -> _remove_ref();
    }
    catch(const PortableServer::POA::ServantNotActive& ex)
    {
        cerr << "\nexception in unsubscribe() : " << ex << endl << flush;
    }
    catch(const PortableServer::POA::WrongPolicy& ex)
    {
        cerr << "\nexception in unsubscribe() : " << ex << endl << flush;
    }
    catch(const CORBA::SystemException& ex)
    {
        cerr << "\nexception in unsubscribe() : " << ex << endl << flush;
    }
    catch(...) {
        cerr << "\nunknown exception at object deactivation in unsubscribe()\n"
	     << flush;
    }
}

void
Broadcaster_impl::
getReceiverByNick(const char* nick,
		  ReceiverDesc_out desc)
    throw(UnknownNick, CORBA::SystemException)
{
    //
    // Delegate operation to administrative object
    //
    admin_ -> getReceiverByNick(nick, desc);
}

StringSeq*
Broadcaster_impl::getReceiverNames()
    throw(CORBA::SystemException)
{
    return admin_ -> getReceiverNames();
}
