// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_REF_COUNTED_H
#define OB_REF_COUNTED_H

#include <OB/Object_fwd.h>
#include <OB/Policy_fwd.h>
#include <OB/RefCounted_fwd.h>

#include <OB/Any.h>
#include <OB/TypeCodeConst.h>
#include <OB/OCITypes.h>

//
// This files contains immutable, reference-counted wrappers for
// several non reference-counted datatypes. (Do not port this to Java
// - it's unnecessary there.)
//

namespace CORBA
{

//
// PolicyList
//
class OBUnique_PolicyList;
typedef OB::ObjSeq< Policy, OBUnique_PolicyList > PolicyList;

} // End of namespace CORBA

namespace OB
{

//
// A reference counted, immutable IOR
//
class RefCountIOR : public RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    RefCountIOR(const RefCountIOR&);
    void operator=(const RefCountIOR&);

public:

    const IOP::IOR value; // The immutable IOR

    RefCountIOR(const IOP::IOR& v) : value(v) { }

    static inline RefCountIOR_ptr _duplicate(RefCountIOR_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline RefCountIOR_ptr _nil()
    { return 0; }
};

} // End of namespace OB

namespace CORBA
{

//
// release and is_nil
//
inline void
release(OB::RefCountIOR_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::RefCountIOR_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

namespace OB
{

//
// A reference counted, immutable PolicyList
//
class RefCountPolicyList : public RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    RefCountPolicyList(const RefCountPolicyList&);
    void operator=(const RefCountPolicyList&);

    //
    // Get various policy values from a policy list
    //
    static CORBA::Short getRetry(const CORBA::PolicyList&);
    static CORBA::Long getConnectTimeout(const CORBA::PolicyList&);
    static CORBA::Long getRequestTimeout(const CORBA::PolicyList&);
    static CORBA::Short getSyncScope(const CORBA::PolicyList&);
    static CORBA::Short getLocationTransparency(const CORBA::PolicyList&);
    static bool getInterceptor(const CORBA::PolicyList&);

public:

    //
    // The immutable PolicyList
    //
    const CORBA::PolicyList value;

    //
    // The immutable value of the retry policy
    //
    const CORBA::Short retry;

    //
    // The immutable value of the connect timeout policy
    //
    const CORBA::Long connectTimeout; 

    //
    // The immutable value of the request timeout policy
    //
    const CORBA::Long requestTimeout; 

    //
    // The immutable value of the sync scope policy
    //
    const CORBA::Short syncScope;

    //
    // The immutable value of the location transparency policy
    //
    const CORBA::Short locationTransparency;

    //
    // The immutable value of the InterceptorPolicy, or true if there
    // is no such policy
    //
    const bool interceptor;

    RefCountPolicyList(const CORBA::PolicyList&);

    static inline RefCountPolicyList_ptr _duplicate(RefCountPolicyList_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline RefCountPolicyList_ptr _nil()
    { return 0; }
};

} // End of namespace OB

namespace CORBA
{

//
// release and is_nil
//
inline void
release(OB::RefCountPolicyList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::RefCountPolicyList_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

namespace OB
{

//
// A reference counted, immutable ProfileInfo
//
class RefCountProfileInfo : public RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    RefCountProfileInfo(const RefCountProfileInfo&);
    void operator=(const RefCountProfileInfo&);

public:

    const OCI::ProfileInfo value; // The immutable ProfileInfo

    RefCountProfileInfo(const OCI::ProfileInfo& v) : value(v) { }

    static inline RefCountProfileInfo_ptr _duplicate(RefCountProfileInfo_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline RefCountProfileInfo_ptr _nil()
    { return 0; }
};

} // End of namespace OB

namespace CORBA
{

//
// release and is_nil
//
inline void
release(OB::RefCountProfileInfo_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::RefCountProfileInfo_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
