// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <ActiveStateFactory_impl.h>
#include <InternalServer_impl.h>
#include <ServerDatabase.h>

using namespace IMR;

// ----------------------------------------------------------------------
// ActiveStateFactory_impl private member functions
// ----------------------------------------------------------------------

ActiveState_ptr
ActiveStateFactory_impl::createActiveStateReference(CORBA::Long id)
{
    //
    // Create the ID for the record.
    //
    CORBA::String_var sid = CORBA::string_dup("");
    sid += id;
    PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(sid);

    //
    // Create the object-reference.
    //
    CORBA::Object_var obj = activeStatePOA_ ->
	create_reference_with_id(oid, "IDL:ooc.com/IMR/ActiveState:1.0");
    return ActiveState::_narrow(obj);
}

// ----------------------------------------------------------------------
// ActiveStateFactory_impl constructor and destructor
// ----------------------------------------------------------------------

ActiveStateFactory_impl::ActiveStateFactory_impl(
    ServerDatabase* db,
    OADDatabase* oaddb,
    CORBA::ORB_ptr orb,
    PortableServer::POA_ptr fwdPOA,
    PortableServer::POA_ptr poa,
    IMROAD::Logger_ptr logger,
    ServerStarter* serverStarter)
    : db_(db),
      poa_(PortableServer::POA::_duplicate(poa)),
      server_(new InternalServer_impl(db, oaddb, orb, fwdPOA, logger,
      				      serverStarter))
{
    CORBA::PolicyList pl(6);
    pl.length(6);
    pl[0] = poa_ -> create_lifespan_policy(PortableServer::PERSISTENT);
    pl[1] = poa_ -> create_id_assignment_policy(PortableServer::USER_ID);
    pl[2] = poa_ -> create_request_processing_policy(
	PortableServer::USE_DEFAULT_SERVANT);
    pl[3] = poa_ -> create_id_uniqueness_policy(
	PortableServer::MULTIPLE_ID);
    pl[4] = poa_ -> create_servant_retention_policy(
	PortableServer::NON_RETAIN);
    pl[5] = poa_ -> create_implicit_activation_policy(
	PortableServer::NO_IMPLICIT_ACTIVATION);
    PortableServer::POAManager_var manager = poa_ -> the_POAManager();
    activeStatePOA_ = poa_ -> create_POA("activeState", manager, pl);
    activeStatePOA_ -> set_servant(server_);
}

ActiveStateFactory_impl::~ActiveStateFactory_impl()
{
    delete server_;
}

// ----------------------------------------------------------------------
// ActiveStateFactory_impl public member functions
// ----------------------------------------------------------------------

//
// Implementation for IMR::ActiveStateFactory
//
ActiveState_ptr
ActiveStateFactory_impl::get_active_state(const char* server)
    throw(NoSuchServer, CORBA::SystemException)
{
    ServerRecord_var rec = db_ -> queryRecordByServer(server);
    if(CORBA::is_nil(rec))
	throw NoSuchServer(server);

    return createActiveStateReference(rec -> record().id);
}

PortableServer::POA_ptr
ActiveStateFactory_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}
