// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef INTERNAL_SERVER_IMPL_H
#define INTERNAL_SERVER_IMPL_H

#include <IMRLogger_fwd.h>
#include <ServerRecord_fwd.h>
#include <IMRInternal_skel.h>
#include <IMRDB.h>

namespace IMR
{

//
// Forward declarations
//
class OADDatabase;
class ServerDatabase;
class ServerStarter;

class InternalServer_impl : public POA_IMR::InternalServer
{
    ServerDatabase* db_;
    OADDatabase* oaddb_;

    PortableServer::Current_var current_;

    OB::RefCountIOR_var imrIOR_;

    IMROAD::Logger_var logger_;

    CORBA::ULong count_;

    ServerStarter* serverStarter_;

    ServerRecord_ptr getRecord();

    void forkFailed();

public:

    InternalServer_impl(ServerDatabase*, OADDatabase*, CORBA::ORB_ptr,
			PortableServer::POA_ptr, IMROAD::Logger_ptr, 
			ServerStarter*);
    ~InternalServer_impl();

    //
    // Standard IDL to C++ Mapping
    //
    
    //
    // IDL:ooc.com/IMR/ActiveState/startup:1.0
    //
    virtual void startup(const char* id,
			 const IOP::IOR& ior,
			 OAD::ProcessEndpointManager_out endpoint)
        throw(IMR::NoSuchOAD,
	      IMR::OADNotRunning,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/set_status:1.0
    //
    virtual void set_status(const char*, IMR::ServerStatus status)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/poa_create:1.0
    //
    virtual void poa_create(const IMR::POAName& poa,
    			    IMR::POAStatus state,
			    const IOP::IOR& ior,
			    IOP::IOR_out imrIOR)
        throw(IMR::NoSuchPOA,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/poa_destroy:1.0
    //
    virtual void poa_destroy(const IMR::POAName& poa)
        throw(IMR::NoSuchPOA,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/ActiveState/poa_status_update:1.0
    //
    virtual void poa_status_update(const IMR::POANameSeq& poas, 
    			   	   IMR::POAStatus state)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/kill:1.0
    //
    virtual void kill()
	throw(NoSuchOAD, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/fork:1.0
    //
    virtual void fork()
	throw(NoSuchOAD, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/ior_for:1.0
    //
    virtual IOP::IOR* ior_for(const IMR::POAName& poa)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/wait:1.0
    //
    virtual void wait(IMR::ServerStatus, const IMR::POAName&, IMR::POAStatus)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/timeout:1.0
    //
    virtual CORBA::Boolean timeout()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/InternalServer/status:1.0
    //
    virtual void status(const IMR::POAName&, IMR::ServerStatus&,
    			IMR::POAStatus&)
        throw(CORBA::SystemException);
};

}; // End namespace IMR

#endif
