// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef LOCATION_FORWARDER_H
#define LOCATION_FORWARDER_H

#include <OB/IMRTypes.h>
#include <OB/IMRState_fwd.h>

#include <IMRLogger_fwd.h>

namespace IMR
{

//
// LocationForwarded implementation. This
// PortableServer::ServantLocator implementation intercepts requests
// and redirects them to the appropriate running server.
//
class LocationForwarder : public POA_PortableServer::ServantLocator
{
    IMROAD::Logger_var logger_; // The message logger

    IMR::ActiveStateFactory_var factory_; // The ActiveStateFactory.

    OB::ObjectFactory_var objectFactory_;

    //
    // Get the IOR for the named server and poa
    //
    IOP::IOR*
    getIOR(const char*, const IMR::POAName&);

public:

    LocationForwarder(CORBA::ORB_ptr, IMROAD::Logger_ptr,
    		      IMR::ActiveStateFactory_ptr);

    ~LocationForwarder();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // The provided ObjectId to this method is the object-key of the
    // target servant.
    //
    virtual PortableServer::Servant
    preinvoke(const PortableServer::ObjectId&,
	      PortableServer::POA_ptr,
	      const char*,
	      PortableServer::ServantLocator::Cookie&)
        throw(PortableServer::ForwardRequest,
              CORBA::SystemException);

    virtual void
    postinvoke(const PortableServer::ObjectId&,
	       PortableServer::POA_ptr,
	       const char*,
	       PortableServer::ServantLocator::Cookie,
	       PortableServer::Servant)
        throw(CORBA::SystemException);
};

} // End namespace IMR

#endif
