// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/RefCounted.h>

#include <BootManager_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// BootLocatorHolder constructor/destructor
// ----------------------------------------------------------------------

BootLocatorHolder::BootLocatorHolder()
{
}

BootLocatorHolder::~BootLocatorHolder()
{
}

// ----------------------------------------------------------------------
// BootLocatorHolder public member implementation
// ----------------------------------------------------------------------

void
BootLocatorHolder::setBootLocator(BootLocator_ptr bootLocator)
{
    OB_SYNCHRONIZED

    bootLocator_ = BootLocator::_duplicate(bootLocator);
}

BootLocator_ptr
BootLocatorHolder::getBootLocator()
{
    OB_SYNCHRONIZED

    return BootLocator::_duplicate(bootLocator_);
}

// ----------------------------------------------------------------------
// BootManager_impl constructor/destructor
// ----------------------------------------------------------------------

BootManager_impl::BootManager_impl()
    : bindings_(new BootHashTable(17))
{
}

BootManager_impl::~BootManager_impl()
{
    delete bindings_;
}

// ----------------------------------------------------------------------
// BootManager_impl Standard IDL to C++ Mapping
// ----------------------------------------------------------------------

void
BootManager_impl::add_binding(const PortableServer::ObjectId& oid,
			      Object_ptr obj)
    throw(OB::BootManager::AlreadyExists, SystemException)
{
    //
    // If binding id is not already mapped add the binding.
    //
#ifdef HAVE_JTC
    JTCSynchronized sync(*bindings_);
#endif
    if(bindings_ -> containsKey(oid))
        throw AlreadyExists();

    bindings_ -> put(oid, Object::_duplicate(obj));
}

void
BootManager_impl::remove_binding(const PortableServer::ObjectId& oid)
    throw(OB::BootManager::NotFound, SystemException)
{
    //
    // If binding id is mapped remove the binding
    //
#ifdef HAVE_JTC
    JTCSynchronized sync(*bindings_);
#endif
    if(!bindings_ -> containsKey(oid))
        throw NotFound();

    bindings_ -> remove(oid);
}

void
BootManager_impl::set_locator(BootLocator_ptr locator)
    throw(SystemException)
{
    //
    // Set the BootLocator
    //
    locator_.setBootLocator(locator);
}

// ----------------------------------------------------------------------
// BootManager_impl Internal Methods
// ----------------------------------------------------------------------

RefCountIOR_ptr
BootManager_impl::_OB_locate(const OCI::ObjectKey& key)
{
    //
    // Convert the OCI::ObjectKey to a PortableServer::ObjectId
    //
    PortableServer::ObjectId oid(key.length());
    oid.length(key.length());
    memcpy(oid.get_buffer(), key.get_buffer(), key.length());

    //
    // First check the internal hash table and then the 
    // registered BootLocator (if there is one) to find the
    // binding for the requested ObjectId.
    //
    Object_var obj;
    if(!bindings_ -> get(oid, obj))
    {
        BootLocator_var bootLocator = locator_.getBootLocator();
        if(!is_nil(bootLocator))
        {
            try
	    {
	        bool add;
	        bootLocator -> locate(oid, obj, add);
	        if(add)
    		    bindings_ -> put(oid, Object::_duplicate(obj));
	    }
	    catch(const NotFound&)
	    {
	    }
	}
    }

    if(is_nil(obj))
        return RefCountIOR::_nil();

    return obj -> _OB_IOR();
}
