// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_BOOT_MANAGER_IMPL_H
#define OB_BOOT_MANAGER_IMPL_H

#include <OB/NativeTypes.h>
#include <OB/Except.h>
#include <OB/Object.h>
#include <OB/Hashtable.h>
#include <OB/BootManager.h>
#include <OB/ObjectIdHasher.h>

namespace OB
{

//
// Holder class for BootLocator object reference
//
class BootLocatorHolder OB_COLON_MUTEX
{
    BootLocator_var bootLocator_;

public:

    BootLocatorHolder();
    ~BootLocatorHolder();

    //
    // Set the Boot Locator
    //
    void setBootLocator(BootLocator_ptr);

    //
    // Get the Boot Locator
    //
    BootLocator_ptr getBootLocator();
};

//
// Implementation class for Boot Manager
//
class BootManager_impl : public BootManager
{
    //
    // Hide copy constructor/assignment operator
    //
    BootManager_impl(const BootManager_impl&);
    void operator=(const BootManager_impl&);

    //
    // Set of known bindings
    //
    typedef OB::Hashtable< PortableServer::ObjectId,
    			   CORBA::Object_var,
    			   OBPortableServer::ObjectIdHasher,
			   OB_LOCK_STRATEGY_SYNCHRONIZED > BootHashTable;
    BootHashTable* bindings_; 

    //
    // The Boot Locator Holder
    //
    BootLocatorHolder locator_; 

public:

    BootManager_impl();
    ~BootManager_impl();

    //
    // Standard IDL to C++ Mapping
    //

    virtual void add_binding(const PortableServer::ObjectId&,
    		             CORBA::Object_ptr)
	throw(OB::BootManager::AlreadyExists, CORBA::SystemException);

    virtual void remove_binding(const PortableServer::ObjectId&)
        throw(OB::BootManager::NotFound, CORBA::SystemException);

    virtual void set_locator(BootLocator_ptr)
        throw(CORBA::SystemException);

    //
    // Internal Methods
    //

    RefCountIOR_ptr _OB_locate(const OCI::ObjectKey&);
};

} // End of namespace OB

#endif
