// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CODE_SET_DATABASE_H
#define OB_CODE_SET_DATABASE_H

#include <OB/TypeCode.h>
#include <OB/CONV_FRAME.h>

namespace OB
{

//
// Forward declaration
//
class CodeConverterBase;

class CodeSetDatabase OB_COLON_MUTEX
{
public:

    //
    // The code set registry IDs for the supported code sets
    //
    OB_IMPORT static const CORBA::ULong ISOLATIN1;
    OB_IMPORT static const CORBA::ULong ISOLATIN2;
    OB_IMPORT static const CORBA::ULong ISOLATIN3;
    OB_IMPORT static const CORBA::ULong ISOLATIN4;
    OB_IMPORT static const CORBA::ULong ISOLATIN5;
    OB_IMPORT static const CORBA::ULong ISOLATIN7;
    OB_IMPORT static const CORBA::ULong ISOLATIN9;
    OB_IMPORT static const CORBA::ULong PCS;
    OB_IMPORT static const CORBA::ULong UTF8;
    OB_IMPORT static const CORBA::ULong UTF16;
    OB_IMPORT static const CORBA::ULong UCS2;

    //
    // The data required for the conversion
    //
    struct CodeSetInfo
    {
        const char* description;
	CORBA::ULong rgy_value;
	CORBA::ULong char_values_size;
	const CORBA::UShort* char_values;
	CORBA::UShort max_bytes;
    };

    //
    // The character mapping data
    //
    struct CharMapInfo
    {
	const char* name;
	CORBA::ULong lower_bound;
	CORBA::ULong upper_bound;
//	CORBA::ULong map_values_size;
	const CORBA::UShort* map_values;
    };

private:

    typedef FixSeq< CodeConverterBase*, int> CodeConverterSeq;
    CodeConverterSeq codeConverters_;

    //
    // The CodeSetDatabase singleton
    //
#ifdef HAVE_JTC
    static JTCMutex instanceMutex_;
#endif
    static OB_IMPORT CodeSetDatabase* instance_;
    friend class CodeSetDatabaseDestroyer;

    //
    // The array with code set infos and the size of that array
    //
    static const CodeSetInfo codeSetInfoArray_[];
    static const CORBA::ULong codeSetInfoArraySize_;

    //
    // The array with character mapping infos and the size of that array
    //
    static const CharMapInfo charMapInfoArray_[];
    static const CORBA::ULong charMapInfoArraySize_;

    CodeSetDatabase();
    ~CodeSetDatabase();

public:

    //
    // Get singleton instance
    //
    static CodeSetDatabase* instance();

    //
    // Get CodeSetInfo for a particular registry ID
    //
    const CodeSetInfo* getCodeSetInfo(CORBA::ULong);

    //
    // Determine transmission codeset (TCS) from client/server code sets
    //
    CORBA::ULong determineTCS(const CONV_FRAME::CodeSetComponent&,
			      const CONV_FRAME::CodeSetComponent&,
			      CORBA::ULong);

    //
    // Check code set compatibility
    //
    bool isCompatible(CORBA::ULong, CORBA::ULong);

    //
    // Check for particular code set id in CodeSetComponent
    //
    bool checkCodeSetId(const CONV_FRAME::CodeSetComponent&, CORBA::ULong);

    //
    // Convert code set name to OSF code set ID and vice versa
    //
    CORBA::ULong nameToId(const char*);
    const char* idToName(CORBA::ULong);

    //
    // Get Unicode character map for a particular code set
    //
    const CharMapInfo* getCharMapInfo(const CodeSetInfo*);

    //
    // Get converter
    //
    CodeConverterBase* getConverter(CORBA::ULong, CORBA::ULong);
};

} // End of namespace OB

#endif
