// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CODE_SET_UTIL_H
#define OB_CODE_SET_UTIL_H

#include <OB/PIManager_fwd.h>

#include <OB/Any.h>
#include <OB/TypeCodeConst.h>
#include <OB/OCITypes.h>

#include <CodeConverter.h>

namespace IOP
{

//
// Forward declarations
//
struct IOR;
struct TaggedComponent;
struct ServiceContext;

} // End of namespace IOP

namespace CONV_FRAME
{

//
// Forward declarations
//
struct CodeSetComponentInfo;
struct CodeSetComponent;
struct CodeSetContext;

}; // End of namespace CONV_FRAME

namespace OCI
{

struct ProfileInfo;

}; // End of namespace OCI

namespace OB
{

//
// Forward declarations
//
struct CodeConverters;

class CodeSetUtil
{
    friend class CodeSetDatabase;

    //
    // The current native codesets
    //
    static CORBA::ULong native_cs_;
    static CORBA::ULong native_wcs_;

    //
    // The default wchar codeset (should be 0 according to specification)
    //
    static CORBA::ULong default_wcs_;

    //
    // The supported codesets in the preferred order
    //
    static FixSeq< CORBA::ULong, int > supportedCharCodeSets_;
    static FixSeq< CORBA::ULong, int > supportedWcharCodeSets_;

public:

    //
    // Get and set native codesets
    //
    static CORBA::ULong native_cs();
    static void native_cs(CORBA::ULong);
    static CORBA::ULong native_wcs();
    static void native_wcs(CORBA::ULong);

    //
    // Set default wchar codeset
    //
    static void default_wcs(CORBA::ULong);

    //
    // Add codeset
    //
    static void addCharCodeSet(CORBA::ULong);
    static void addWcharCodeSet(CORBA::ULong);

    //
    // Create codeset component info
    //
    static CONV_FRAME::CodeSetComponentInfo* createCodeSetComponentInfo();

    //
    // Create codeset component
    //
    static CONV_FRAME::CodeSetComponent*
        createCharCodeSetComponent(CORBA::ULong);
    static CONV_FRAME::CodeSetComponent*
        createWcharCodeSetComponent(CORBA::ULong);

    //
    // Get code converters from ProfileInfo and/or IOR
    //
    static void getCodeConverters(const OCI::ProfileInfo&,
				  OB::CodeConverters&);

    //
    // Check for code set information in a tagged component
    //
    static bool checkForCodeSetInfo(const IOP::TaggedComponent&,
                                    CONV_FRAME::CodeSetComponentInfo&);

    //
    // Extract code set context from service context
    //
    static void extractCodeSetContext(const IOP::ServiceContext&,
				      CONV_FRAME::CodeSetContext&);

    //
    // Extract codeset information
    //
    static bool
    getCodeSetInfoFromComponents(const OCI::ProfileInfo&,
				 CONV_FRAME::CodeSetComponentInfo&);

    //
    // Convert between UTF-8 and Unicode
    //
    static void unicodeToUtf8(CORBA::Octet*&, CORBA::WChar)
	throw(CORBA::DATA_CONVERSION);
    static CORBA::WChar utf8ToUnicode(CORBA::Octet*&)
	throw(CORBA::DATA_CONVERSION);

    //
    // Registry the IORInterceptor that installs the code set
    // information
    //
    static void registerIORInterceptor(PIManager_ptr);
};

} // End of namespace OB

#endif
