// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/TypeCode.h>
#include <OB/Stream.h>
#include <OB/ORBInstance.h>

#include <CodecFactory_impl.h>

using namespace CORBA;
using namespace OB;

namespace IOP
{

//
// The CDR codec implementation
//
class CDRCodec_impl : public Codec
{

    ORBInstance_var orbInstance_;

public:

    CDRCodec_impl(ORBInstance_ptr);
    ~CDRCodec_impl();

    virtual OctetSeq* encode(const Any&);

    virtual Any* decode(const OctetSeq&);

    virtual OctetSeq* encode_value(const Any&);

    virtual Any* decode_value(const OctetSeq&, TypeCode_ptr);
};

} // End namespace IOP

// ----------------------------------------------------------------------
// CDRCodec_impl constructor and destructor
// ----------------------------------------------------------------------

IOP::CDRCodec_impl::CDRCodec_impl(ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
}

IOP::CDRCodec_impl::~CDRCodec_impl()
{
}

// ----------------------------------------------------------------------
// CDRCodec_impl_impl constructor and destructor
// ----------------------------------------------------------------------

OctetSeq*
IOP::CDRCodec_impl::encode(const Any& data)
{
    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    out._OB_writeEndian();
    out.write_any(data);
    OctetSeq* result = new OctetSeq;
    result -> length(buf.length());
    memcpy(result -> get_buffer(), buf.data(), buf.length());
    return result;
}

Any*
IOP::CDRCodec_impl::decode(const OctetSeq& data)
{
    OCI::BufferImpl buf((Octet*)data.get_buffer(), data.length());
    InputStreamImpl in(&buf, 0, false);
    in._OB_ORBInstance(orbInstance_);
    in._OB_readEndian();

    return in.read_any();
}

OctetSeq*
IOP::CDRCodec_impl::encode_value(const Any& data)
{
    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    out._OB_writeEndian();
    out.write_any_NoTypeCode(data);
    OctetSeq* result = new OctetSeq;
    result -> length(buf.length());
    memcpy(result -> get_buffer(), buf.data(), buf.length());

    return result;
}

Any*
IOP::CDRCodec_impl::decode_value(const OctetSeq& data, TypeCode_ptr tc)
{
    if(is_nil(tc))
        throw TypeMismatch();

    OCI::BufferImpl buf((Octet*)data.get_buffer(), data.length());
    InputStreamImpl in(&buf, 0, false);
    in._OB_ORBInstance(orbInstance_);
    in._OB_readEndian();

    Any_var any = new Any;
    in.read_any_NoTypeCode(any.inout(), tc);

    return any._retn();
}

// ----------------------------------------------------------------------
// CodecFactory_impl constructor and destructor
// ----------------------------------------------------------------------

IOP::CodecFactory_impl::~CodecFactory_impl()
{
}

// ----------------------------------------------------------------------
// CodecFactory_impl public member implementation
// ----------------------------------------------------------------------

IOP::Codec_ptr
IOP::CodecFactory_impl::create_codec(const IOP::Encoding& encoding)
{
    assert(!is_nil(orbInstance_));

    //
    // TODO: check major/minor version
    //
    if(encoding.format != ENCODING_CDR_ENCAPS)
        throw UnknownEncoding();

    //
    // Double checked locking
    //
    if(is_nil(cdrCodec_))
    {
#ifdef HAVE_JTC
        JTCSynchronized sync(cdrCodecMut_);
#endif
        if(is_nil(cdrCodec_))
            cdrCodec_ = new CDRCodec_impl(orbInstance_);
    }

    return Codec::_duplicate(cdrCodec_);
}

void
IOP::CodecFactory_impl::_OB_setORBInstance(ORBInstance_ptr orbInstance)
{
    orbInstance_ = ORBInstance::_duplicate(orbInstance);
}
