// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Downcall.h>
#include <OB/Upcall.h>
#include <OB/Stream.h>

#include <DowncallUpcallPair.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::DowncallUpcallPair_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::DowncallUpcallPair_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// DowncallUpcallPair constructor/destructor
// ----------------------------------------------------------------------

DowncallUpcallPair::DowncallUpcallPair(Downcall_ptr down)
    : downcall_(Downcall::_duplicate(down))
{
}

// ----------------------------------------------------------------------
// DowncallUpcallPair public member implementation
// ----------------------------------------------------------------------

void 
DowncallUpcallPair::setUpcall(Upcall_ptr up)
{
    upcall_ = Upcall::_duplicate(up);
}

bool
DowncallUpcallPair::waitReply(bool block, Long t)
    throw(SystemException)
{
    OB_SYNCHRONIZED

#ifdef HAVE_JTC
    assert(!downcall_ -> unsent());
    
    if(block)
    {
	while(downcall_ -> pending())
	{
	    try
	    {
		if(t < 0)
		    wait();
		else
		    wait(t);
		
		break;
	    }
	    catch(const JTCInterruptedException&)
	    {
	    }
	}
	
	if(downcall_ -> pending())
	    throw NO_RESPONSE(0, COMPLETED_MAYBE); // Timeout
    }
    else
    {
	if(downcall_ -> pending())
	    return false; // Still in progress
    }
#else
    assert(!downcall_ -> unsent());
    assert(!downcall_ -> pending());
#endif
    return true;
}

void
DowncallUpcallPair::setReplySCL(const IOP::ServiceContextList& replySCL)
{
    OB_SYNCHRONIZED

    downcall_ -> setReplySCL(replySCL);
}

void
DowncallUpcallPair::endReply()
{
    OB_SYNCHRONIZED

    OutputStream_ptr out = upcall_ -> output_nodup();
    OCI::Buffer_var buf = new OCI::Buffer;
    buf -> consume(out -> _OB_buffer_nodup());
    InputStream_var in = new InputStream(buf, 0, false);
    downcall_ -> setNoException(in);

#ifdef HAVE_JTC
    notifyAll();
#endif
}

void
DowncallUpcallPair::endUserException()
{
    OB_SYNCHRONIZED

    OutputStream_ptr out = upcall_ -> output_nodup();
    OCI::Buffer_var buf = new OCI::Buffer;
    buf -> consume(out -> _OB_buffer_nodup());
    InputStream_var in = new InputStream(buf, 0, false);
    downcall_ -> setUserException(in);

#ifdef HAVE_JTC
    notifyAll();
#endif
}

void
DowncallUpcallPair::systemException(const SystemException& ex,
				    const IOP::ServiceContextList& replySCL)
{
    OB_SYNCHRONIZED

    downcall_ -> setSystemException(ex);
    if(replySCL.length() > 0)
        downcall_ -> setReplySCL(replySCL);

#ifdef HAVE_JTC
    notifyAll();
#endif
}

void
DowncallUpcallPair::forward(RefCountIOR_ptr ior,
			    bool perm,
			    const IOP::ServiceContextList& replySCL)
{
    OB_SYNCHRONIZED

    downcall_ -> setLocationForward(ior, perm);
    if(replySCL.length() > 0)
        downcall_ -> setReplySCL(replySCL);

#ifdef HAVE_JTC
    notifyAll();
#endif
}
