// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/MarshalStubImpl.h>
#include <OB/ORBInstance.h>
#include <OB/Logger.h>
#include <OB/IMRState.h>

#include <IMRActiveStateHolder.h>

using namespace OB;

// ----------------------------------------------------------------------
// IMRActiveStateHolder implementation
// ----------------------------------------------------------------------

IMRActiveStateHolder::IMRActiveStateHolder(const char* serverName,
				           const char* serverInstance)
    : serverName_(serverName),
      serverInstance_(serverInstance),
      running_(false)
{
}

void
IMRActiveStateHolder::setActiveState(IMR::ActiveState_ptr activeState)
{
    OB_SYNCHRONIZED
    activeState_ = IMR::ActiveState::_duplicate(activeState);
}

IMR::ActiveState_ptr
IMRActiveStateHolder::getActiveState() const
{
    OB_SYNCHRONIZED
    return IMR::ActiveState::_duplicate(activeState_);
}

void
IMRActiveStateHolder::serverRunning(OB::ORBInstance_ptr orbInstance)
{
    OB_SYNCHRONIZED

    //
    // Has the IMR already been informed by the activation
    // of another POA Manager?
    //
    if(running_)
        return;

    //
    // Tell the IMR that we're running.
    //
    if(!CORBA::is_nil(activeState_))
    {
        Logger_var logger = orbInstance -> getLogger();
        try
        {
            activeState_ -> set_status(serverInstance_, IMR::RUNNING);
        }
        catch(const CORBA::OBJECT_NOT_EXIST&)
        {
            CORBA::String_var msg = CORBA::string_dup(serverName_);
            msg += ": Not registered with IMR";
            logger -> error(msg);
            throw CORBA::INITIALIZE();
        }
        catch(...)
        {
            CORBA::String_var msg = CORBA::string_dup(serverName_);
            msg += ": Cannot contact IMR";
            logger -> error(msg);
            throw CORBA::INITIALIZE();
        }
    }

    running_ = true;
}
