// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OADProcessEndpoint_impl.h>
#include <OBPolicies_impl.h>

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ProcessController_impl constructor and destructor
// ----------------------------------------------------------------------

OAD::ProcessEndpoint_impl::ProcessEndpoint_impl(const char* name,
						const char* id,
    						PortableServer::POA_ptr poa,
						OB::ORBControl_ptr orbControl)
    : name_(name),
      id_(id),
      poa_(PortableServer::POA::_duplicate(poa)),
      orbControl_(OB::ORBControl::_duplicate(orbControl))
{
#ifdef WIN32
    pid_ = GetCurrentProcessId();
#else
    pid_ = getpid();
#endif

    //
    // Create a PolicyList for RETRY_ALWAYS
    //
    pl_.length(1);
    pl_[0] = new ACMTimeoutPolicy_impl(0);
}

OAD::ProcessEndpoint_impl::~ProcessEndpoint_impl()
{
}

// ----------------------------------------------------------------------
// ProcessEndpoint_impl public member implementation
// ----------------------------------------------------------------------

//
// Standard IDL to C++ Mapping
//
void
OAD::ProcessEndpoint_impl::reestablish_link(OAD::ProcessEndpointManager_ptr d)
    throw(CORBA::SystemException)
{
    //
    // Set the policies on this object
    //
    endpointManager_ = OAD::ProcessEndpointManager::_narrow(
        Object_var(d -> _set_policy_overrides(pl_, SET_OVERRIDE)));
    assert(!is_nil(endpointManager_));
    
    ProcessEndpoint_var cb = _this();

    //
    // Establish a new link with the ProcessEndpointManager
    //
    try
    {
        endpointManager_ -> establish_link(name_, id_, pid_, cb);
    }
    catch(const OAD::AlreadyLinked&)
    {
    }
    catch(const CORBA::SystemException& e)
    {
        String_var msg = string_dup("connect_server failed: ");
        String_var s = e._to_string();
        msg += s;
        //logger -> error(msg);
    }
}

void
OAD::ProcessEndpoint_impl::stop()
    throw(CORBA::SystemException)
{
    orbControl_ -> shutdownServer(false);
}

void
OAD::ProcessEndpoint_impl::_OB_establish_link(OAD::ProcessEndpointManager_ptr d)
{
    //
    // Set the policies on this object
    //
    endpointManager_ = OAD::ProcessEndpointManager::_narrow(
        Object_var(d -> _set_policy_overrides(pl_, SET_OVERRIDE)));
    assert(!is_nil(endpointManager_));
    
    //
    // Establish the link
    //
    ProcessEndpoint_var cb = _this();
    endpointManager_ -> establish_link(name_, id_, pid_, cb);

    //
    // Set the retry policy for reestablished connections
    //
    pl_.length(2);
    pl_[1] = new RetryPolicy_impl(RETRY_ALWAYS);
}

PortableServer::POA_ptr
OAD::ProcessEndpoint_impl::_default_POA()
{
   return PortableServer::POA::_duplicate(poa_);
}
