// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBPolicies_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ProtocolPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ProtocolPolicy_impl::ProtocolPolicy_impl(IOP::ProfileId value)
    : value_(value)
{
}

ProtocolPolicy_impl::~ProtocolPolicy_impl()
{
}

// ----------------------------------------------------------------------
// ProtocolPolicy_impl public member implementation
// ----------------------------------------------------------------------

IOP::ProfileId
ProtocolPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ProtocolPolicy_impl::policy_type()
    throw(SystemException)
{
    return PROTOCOL_POLICY_ID;
}

Policy_ptr
ProtocolPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ProtocolPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// ConnectionReusePolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ConnectionReusePolicy_impl::ConnectionReusePolicy_impl(Boolean r)
    : value_(r)
{
}

ConnectionReusePolicy_impl::~ConnectionReusePolicy_impl()
{
}

// ----------------------------------------------------------------------
// ConnectionReusePolicy_impl public member implementation
// ----------------------------------------------------------------------

Boolean
ConnectionReusePolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ConnectionReusePolicy_impl::policy_type()
    throw(SystemException)
{
    return CONNECTION_REUSE_POLICY_ID;
}

Policy_ptr
ConnectionReusePolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ConnectionReusePolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// RetryPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

RetryPolicy_impl::RetryPolicy_impl(CORBA::Short r)
    : value_(r)
{
}

RetryPolicy_impl::~RetryPolicy_impl()
{
}

// ----------------------------------------------------------------------
// RetryPolicy_impl public member implementation
// ----------------------------------------------------------------------

CORBA::Short
RetryPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
RetryPolicy_impl::policy_type()
    throw(SystemException)
{
    return RETRY_POLICY_ID;
}

Policy_ptr
RetryPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
RetryPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

TimeoutPolicy_impl::TimeoutPolicy_impl(ULong t)
{
    try
    {
	if(t > 0x7fffffff)
	    throw IMP_LIMIT();
	value_ = t;
    }
    catch(...)
    {
	_OB_setRef(0);
	throw;
    }
}

TimeoutPolicy_impl::~TimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
TimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
TimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return TIMEOUT_POLICY_ID;
}

Policy_ptr
TimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
TimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// ConnectTimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ConnectTimeoutPolicy_impl::ConnectTimeoutPolicy_impl(ULong t)
{
    if(t > 0x7fffffff)
        throw IMP_LIMIT();
    value_ = t;
}

ConnectTimeoutPolicy_impl::~ConnectTimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
ConnectTimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ConnectTimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return CONNECT_TIMEOUT_POLICY_ID;
}

Policy_ptr
ConnectTimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ConnectTimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// RequestTimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

RequestTimeoutPolicy_impl::RequestTimeoutPolicy_impl(ULong t)
{
    try
    {
	if(t > 0x7fffffff)
	    throw IMP_LIMIT();
	value_ = t;
    }
    catch(...)
    {
	_OB_setRef(0);
	throw;
    }
}

RequestTimeoutPolicy_impl::~RequestTimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
RequestTimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
RequestTimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return REQUEST_TIMEOUT_POLICY_ID;
}

Policy_ptr
RequestTimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
RequestTimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// LocationTransparencyPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

LocationTransparencyPolicy_impl::LocationTransparencyPolicy_impl(Short value)
    : value_(value)
{
}

LocationTransparencyPolicy_impl::~LocationTransparencyPolicy_impl()
{
}

// ----------------------------------------------------------------------
// LocationTransparencyPolicy_impl public member implementation
// ----------------------------------------------------------------------

Short
LocationTransparencyPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
LocationTransparencyPolicy_impl::policy_type()
    throw(SystemException)
{
    return LOCATION_TRANSPARENCY_POLICY_ID;
}

Policy_ptr
LocationTransparencyPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
LocationTransparencyPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// InterceptorPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

InterceptorPolicy_impl::InterceptorPolicy_impl(bool value)
    : value_(value)
{
}

InterceptorPolicy_impl::~InterceptorPolicy_impl()
{
}

// ----------------------------------------------------------------------
// InterceptorPolicy_impl public member implementation
// ----------------------------------------------------------------------

bool
InterceptorPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
InterceptorPolicy_impl::policy_type()
    throw(SystemException)
{
    return INTERCEPTOR_POLICY_ID;
}

Policy_ptr
InterceptorPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
InterceptorPolicy_impl::destroy()
    throw(SystemException)
{
}

// ----------------------------------------------------------------------
// RequestTimeoutPolicy_impl constructor and destructor
// ----------------------------------------------------------------------

ACMTimeoutPolicy_impl::ACMTimeoutPolicy_impl(ULong t)
{
    try
    {
	if(t > 0x7fffffff)
	    throw IMP_LIMIT();
	value_ = t;
    }
    catch(...)
    {
	_OB_setRef(0);
	throw;
    }
}

ACMTimeoutPolicy_impl::~ACMTimeoutPolicy_impl()
{
}

// ----------------------------------------------------------------------
// TimeoutPolicy_impl public member implementation
// ----------------------------------------------------------------------

ULong
ACMTimeoutPolicy_impl::value()
    throw(SystemException)
{
    return value_;
}

PolicyType
ACMTimeoutPolicy_impl::policy_type()
    throw(SystemException)
{
    return ACM_TIMEOUT_POLICY_ID;
}

Policy_ptr
ACMTimeoutPolicy_impl::copy()
    throw(SystemException)
{
    return Policy::_duplicate(this);
}

void
ACMTimeoutPolicy_impl::destroy()
    throw(SystemException)
{
}
