// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POLICIES_IMPL_H
#define OB_POLICIES_IMPL_H

#include <OB/Object.h>
#include <OB/Except.h>
#include <OB/OBPolicies.h>

namespace OB
{

class ProtocolPolicy_impl : public ProtocolPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    ProtocolPolicy_impl(const ProtocolPolicy_impl&);
    void operator=(const ProtocolPolicy_impl&);

    IOP::ProfileId value_;

public:

    ProtocolPolicy_impl(IOP::ProfileId);
    ~ProtocolPolicy_impl();

    virtual IOP::ProfileId value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ConnectionReusePolicy_impl : public ConnectionReusePolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectionReusePolicy_impl(const ConnectionReusePolicy_impl&);
    void operator=(const ConnectionReusePolicy_impl&);

    CORBA::Boolean value_;

public:

    ConnectionReusePolicy_impl(CORBA::Boolean);
    ~ConnectionReusePolicy_impl();

    virtual CORBA::Boolean value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RetryPolicy_impl : public RetryPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    RetryPolicy_impl(const RetryPolicy_impl&);
    void operator=(const RetryPolicy_impl&);

    CORBA::Short value_;

public:

    RetryPolicy_impl(CORBA::Short);
    ~RetryPolicy_impl();

    virtual CORBA::Short value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class TimeoutPolicy_impl : public TimeoutPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    TimeoutPolicy_impl(const TimeoutPolicy_impl&);
    void operator=(const TimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    TimeoutPolicy_impl(CORBA::ULong);
    ~TimeoutPolicy_impl();

    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ConnectTimeoutPolicy_impl : public ConnectTimeoutPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectTimeoutPolicy_impl(const ConnectTimeoutPolicy_impl&);
    void operator=(const ConnectTimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    ConnectTimeoutPolicy_impl(CORBA::ULong);
    ~ConnectTimeoutPolicy_impl();

    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class RequestTimeoutPolicy_impl : public RequestTimeoutPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    RequestTimeoutPolicy_impl(const RequestTimeoutPolicy_impl&);
    void operator=(const RequestTimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    RequestTimeoutPolicy_impl(CORBA::ULong);
    ~RequestTimeoutPolicy_impl();

    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class LocationTransparencyPolicy_impl : public LocationTransparencyPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    LocationTransparencyPolicy_impl(const LocationTransparencyPolicy_impl&);
    void operator=(const LocationTransparencyPolicy_impl&);

    CORBA::Short value_;

public:

    LocationTransparencyPolicy_impl(CORBA::Short);
    ~LocationTransparencyPolicy_impl();

    virtual CORBA::Short value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class InterceptorPolicy_impl : public InterceptorPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    InterceptorPolicy_impl(const InterceptorPolicy_impl&);
    void operator=(const InterceptorPolicy_impl&);

    bool value_;

public:

    InterceptorPolicy_impl(bool);
    ~InterceptorPolicy_impl();

    virtual bool value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

class ACMTimeoutPolicy_impl : public ACMTimeoutPolicy
{
    //
    // Hide copy-constructor and assignment operator
    //
    ACMTimeoutPolicy_impl(const ACMTimeoutPolicy_impl&);
    void operator=(const ACMTimeoutPolicy_impl&);

    CORBA::ULong value_;

public:

    ACMTimeoutPolicy_impl(CORBA::ULong);
    ~ACMTimeoutPolicy_impl();

    virtual CORBA::ULong value()
        throw(CORBA::SystemException);

    virtual CORBA::PolicyType policy_type()
        throw(CORBA::SystemException);
    virtual CORBA::Policy_ptr copy()
        throw(CORBA::SystemException);
    virtual void destroy()
        throw(CORBA::SystemException);
};

} // End of namespace OB

#endif
