// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/ObjectKey.h>
#include <OB/Object.h>
#include <OB/StubImpl.h>
#include <OB/MarshalStubImpl.h>
#include <OB/POAInterface.h>
#include <OB/seq_converter.h>

#include <POALocator.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// POAManager_impl::POANameHasher public member implementations
// ----------------------------------------------------------------------

ULong
OBPortableServer::POANameHasher::hash(const IMR::POAName& k)
{
    ULong result = 0;

    for(ULong i = 0 ; i < k.length() ; i++)
        result ^= StringHasher::hash(k[i]);

    return result;
}

bool
OBPortableServer::POANameHasher::comp(const IMR::POAName& k1,
				      const IMR::POAName& k2)
{
    if(k1.length() == k2.length())
    {
        for(ULong i = 0 ; i < k1.length() ; i++)
        {
            if(strcmp(k1[i], k2[i]) != 0)
                return false;
        }

        return true;
    }

    return false;
}

// ----------------------------------------------------------------------
// POALocator constructor/destructor
// ----------------------------------------------------------------------

OBPortableServer::POALocator::POALocator()
      : poas_(new POAHashTable(63))
{
}

OBPortableServer::POALocator::~POALocator()
{
    delete poas_;
}

// ----------------------------------------------------------------------
// POALocator public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
OBPortableServer::POALocator::locate(const OB::ObjectKeyData& data)
    throw(LocationForward, SystemException)
{
    OB_SYNCHRONIZED

    PortableServer::POA_var poa;
    if(!poas_ -> get(data.poaID, poa))
    {
        //
        // Arrange for POA activation. The algorithm for this is to
        // find the first POA in the `path' that exists. We then call
        // find_POA on each remaining POA in the path to cause the
        // activation.
        //
        IMR::POAName poaID = data.poaID;
        IMR::POAName remaining;
        do
        {
            CORBA::ULong remainingLength = remaining.length();
            remaining.length(remainingLength + 1);
            remaining[remainingLength] = poaID[poaID.length()-1];

            poaID.length(poaID.length() - 1);
            if(poas_ -> get(poaID, poa))
                break;
        }
        while(poaID.length() > 0);
        
        //
        // Now find each POA in the path until we fail to find one of
        // the POA's or we're done.
        //
        if(poaID.length() > 0)
        {
            do
            {
                try
                {
                    poa = poa -> find_POA(remaining[0], true);
		    remove_seq_item(remaining, 0);
                }
                catch(PortableServer::POA::AdapterNonExistent&)
                {
                    poa = PortableServer::POA::_nil();
                }
            }
            while(remaining.length() > 0 && !is_nil(poa));
        }
    }

    return poa._retn();
}

void
OBPortableServer::POALocator::add(PortableServer::POA_ptr poa,
				  const IMR::POAName& id)
{
    OB_SYNCHRONIZED

    assert(!poas_ -> containsKey(id));
    poas_ -> put(id, PortableServer::POA::_duplicate(poa));
}

void
OBPortableServer::POALocator::remove(const IMR::POAName& id)
{
    OB_SYNCHRONIZED

    assert(poas_ -> containsKey(id));
    poas_ -> remove(id);
}

