// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POA_MANAGER_FACTORY_IMPL_H
#define OB_POA_MANAGER_FACTORY_IMPL_H

#include <OB/Server_fwd.h>
#include <OB/POAServantBase_fwd.h>
#include <OB/POAInterface_fwd.h>
#include <OB/OAInterface_fwd.h>
#include <OB/DirectStubImpl_fwd.h>
#include <OB/Properties_fwd.h>

#include <OB/Object.h>
#include <OB/StubImpl.h>
#include <OB/OBPOAManager.h>
#include <OB/Hashtable.h>
#include <OB/IMRTypes.h>
#include <OB/OBPOAManager.h>
#include <OB/ORBControl.h>

namespace OB
{

class IMRActiveStateHolder;

} // End namespace OB

namespace OBPortableServer
{

//
// Forward declarations
//
class POA_impl;
class POALocator;

//
// Implementation of OBPortableServer::POAManagerFactory
//
class POAManagerFactory_impl : public POAManagerFactory 
{
    //
    // The ORB Instance
    //
    OB::ORBInstance_var orbInstance_;

    //
    // Hashtable mapping name -> POAManager
    //
    typedef OB::Hashtable<CORBA::String_var, POAManager_var,
	OB::StringHasher, OB::OB_LOCK_STRATEGY_SYNCHRONIZED> POAManagerTable;
    POAManagerTable* managers_;

    //
    // The server name
    //
    CORBA::String_var serverName_;

    //
    // The server instance id
    //
    CORBA::String_var serverInstance_;

    //
    // Running count for generating unique names
    //
    int count_;

    //
    // Does this process have an IMR?
    //
    bool needIMR_;

    //
    // The IMRActiveStateHolder
    //
    OB::IMRActiveStateHolder* imrHolder_;

    //
    // The POALocator
    //
    POALocator* poaLocator_;
    
    //
    // The OAD::ProcessEndpoint
    //
    PortableServer::ServantBase_var processEndpoint_;

    //
    // Assign a unique name for a POAManager
    //
    char* getUniqueName();

    //
    // Validate a POAManager name
    //
    void validateName(const char*)
        throw(POAManagerAlreadyExists);

public:

    POAManagerFactory_impl(const char*, const char*);
    ~POAManagerFactory_impl();
    
    virtual POAManager_ptr
    create_poa_manager(const char*)
	throw(POAManagerFactory::POAManagerAlreadyExists,
              OCI::InvalidParam,
              CORBA::SystemException);

    virtual POAManager_ptr
    create_poa_manager_with_config(
        const char*,
        const AcceptorConfigSeq&)
        throw(POAManagerAlreadyExists,
              OCI::NoSuchFactory,
              OCI::InvalidParam,
              CORBA::SystemException);

    virtual POAManagerSeq*
    get_poa_managers()
	throw(CORBA::SystemException);

    virtual void
    destroy()
	throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //

    //
    // Deactivate all POAManagers
    //
    void _OB_deactivate()
	throw(CORBA::SystemException);
 
    OB::DirectStubImpl_ptr _OB_createDirectStubImpl(OB::RefCountIOR_ptr,
    						    OB::RefCountPolicyList_ptr)
	throw(CORBA::SystemException, OB::LocationForward);

    void _OB_setORBInstance(OB::ORBInstance_ptr);

    void _OB_initializeIMR(POA_impl*, OB::ORBControl_ptr);

    void _OB_destroy();
};

} // End of namespace OBPortableServer

#endif
