// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/PolicyFactoryManager.h>
#include <OB/PIPolicy.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(PolicyFactoryManager_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(PolicyFactoryManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// PolicyFactoryManager constructor and destructor
// ----------------------------------------------------------------------

PolicyFactoryManager::PolicyFactoryManager()
    : destroy_(false),
      policyFactoryTableExternal_(new PolicyFactoryTable(63)),
      policyFactoryTableInternal_(new PolicyFactoryTable(63))
{
}

PolicyFactoryManager::~PolicyFactoryManager()
{
    delete policyFactoryTableInternal_;
    delete policyFactoryTableExternal_;
}

// ----------------------------------------------------------------------
// PolicyFactoryManager public member implementation
// ----------------------------------------------------------------------

//
// Destroy the PolicyFactoryManager
//
void
PolicyFactoryManager::destroy()
{
    assert(!destroy_);
    destroy_ = true;

    policyFactoryTableInternal_ -> clear();
    policyFactoryTableExternal_ -> clear();
}

void
PolicyFactoryManager::registerPolicyFactory(
    PolicyType type,
    PortableInterceptor::PolicyFactory_ptr factory,
    bool internal)
{
    //
    // TODO: some sensible error
    //
    PolicyFactoryTable* table =
        (internal)
        ? policyFactoryTableInternal_
        : policyFactoryTableExternal_;

    if(table -> containsKey(type))
        throw BAD_PARAM();
    table -> put(type, PortableInterceptor::PolicyFactory::
		 _duplicate(factory));
}

Policy_ptr
PolicyFactoryManager::createPolicy(PolicyType type, const Any& any)
{
    PortableInterceptor::PolicyFactory_var factory;
    if(policyFactoryTableInternal_ -> get(type, factory))
        return factory -> create_policy(type, any);
    if(policyFactoryTableExternal_ -> get(type, factory))
        return factory -> create_policy(type, any);

    throw PolicyError(BAD_POLICY);
}

//
// Remove all policies from in that are contained in the external list
// and place in out
//
void
PolicyFactoryManager::filterPolicyList(PolicyList& in, PolicyList& out)
{
    PolicyFactoryTable::Enumerator keys =
        policyFactoryTableExternal_ -> keys();
    for(ULong policy = 0 ; policy < in.length(); policy++)
    {
        PolicyType type = in[policy] -> policy_type();
        for(ULong i = 0 ; i < keys.length() ; i++)
        {
            if(keys[i] == type)
            {
                CORBA::ULong outLength = out.length();
                out.length(outLength + 1);
                out[outLength] = in[policy];
		for(ULong k = policy + 1 ; k < in.length() ; k++)
		    in[k - 1] = in[k];
		in.length(in.length() - 1);
                --policy;
                break;
            }
        }
    }

/*
    PolicyList orig = in;
    in.length(0);

    for(ULong i = 0 ; i < orig.length() ; i++)
    {
	if(policyFactoryTableExternal_ ->
	   containsKey(orig[i] -> policy_type()))
	{
	    ULong l = out.length();
	    out.length(l + 1);
	    out[l] = orig[i];
        }
	else
	{
	    ULong l = in.length();
	    in.length(l + 1);
	    in[l] = orig[i];
	}
    }

    assert(in.length() + out.length() == orig.length());
*/
}
