// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_SELECT_REACTOR_H
#define OB_SELECT_REACTOR_H

#include <OB/EventHandlerBase_fwd.h>

#include <OB/NativeTypes.h>
#include <OB/Object.h>
#include <OB/Reactor.h>
#include <OB/Net.h>
#include <OB/list.h>

namespace OB
{

//
// The SelectReactor class
//
class SelectReactor : public Reactor OB_COMMA_RECURSIVE_MUTEX
{
    //
    // Hide copy-constructor and assignment operator
    //
    SelectReactor(const SelectReactor&);
    void operator=(const SelectReactor&);

#ifdef HAVE_JTC
    //
    // The read and write end of the interrupt fd
    //
    int intRead_;
    int intWrite_;

    //
    // Is there a pending read of the interrupt file descriptor?
    //
    bool pendingRead_;
#endif

    //
    // Is the reactor in a block select?
    //
    bool inBlockingSelect_;

    //
    // If set to true, interrupt event dispatching immediately
    //
    bool interrupt_;

    //
    // Struct for an event handler info
    //
    struct HandlerInfo
    {
	EventHandlerBase_var handler;
	Handle handle;
	Mask mask;
	TypeMask typeMask;
    };

    //
    // Our event handler info list
    //
    OB_STL::list<HandlerInfo> handlerInfoList_;

    //
    // Set file descriptor sets
    //
    TypeMask setFdSets(fd_set&, fd_set&, int&);

    //
    // Evaluate file descriptor sets
    //
    void evalFdSets(fd_set&, fd_set&);

#ifdef HAVE_JTC
    //
    // Clear the interrupt fd
    //
    void clearInterruptFd();

    //
    // Set the interrupt fd
    //
    void setInterruptFd();
#endif

public:

    SelectReactor();
    virtual ~SelectReactor();

    virtual void register_handler(EventHandlerBase*, Mask, TypeMask, Handle);
    virtual void unregister_handler(EventHandlerBase*);
    virtual bool dispatch(TypeMask);
    virtual void interrupt_dispatch();
    virtual bool dispatch_one_event(CORBA::Long);
    virtual bool event_ready();
};

} // End of namespace OB

#endif
