// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_X11_REACTOR_H
#define OB_X11_REACTOR_H

#include <OB/EventHandlerBase_fwd.h>

#include <OB/NativeTypes.h>
#include <OB/Object.h>
#include <OB/Reactor.h>
#include <OB/list.h>

namespace OB
{

//
// The X11Reactor class
//
class X11Reactor : public Reactor OB_COMMA_RECURSIVE_MUTEX
{
    //
    // Hide copy-constructor and assignment operator
    //
    X11Reactor(const X11Reactor&);
    void operator=(const X11Reactor&);

    //
    // The read and write end of the select interrupt fd
    //
    int intRead_;
    int intWrite_;

    //
    // The input ID for the interrupt input handler
    //
    XtInputId interruptInputId_;

    //
    // If set to true, interrupt event dispatching immediately
    //
    bool interrupt_;

    //
    // The application context
    //
    XtAppContext context_;

    //
    // Struct for an event handler info
    //
    struct HandlerInfo
    {
	EventHandlerBase_var handler;
	Handle handle;
	Mask mask;
	TypeMask typeMask;
	XtInputId readId;
	XtInputId writeId;
    };

    //
    // Our event handler info list
    //
    OB_STL::list<HandlerInfo> handlerInfoList_;

    //
    // Return the TypeMask for the registered event handlers
    //
    TypeMask evalTypeMasks() const;

public:

    X11Reactor(XtAppContext);
    virtual ~X11Reactor();

    virtual void register_handler(EventHandlerBase*, Mask, TypeMask, Handle);
    virtual void unregister_handler(EventHandlerBase*);
    virtual bool dispatch(TypeMask);
    virtual void interrupt_dispatch();
    virtual bool dispatch_one_event(CORBA::Long);
    virtual bool event_ready();
};

} // End of namespace OB

#endif
