// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>
#include <OB/list.h>

#include <Util.h>
#include <GenUtil.h>
#include <GenCPP.h>

#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;
using namespace OB;

#define OBEVersion "1.0.1"

//
// The header
//
const char* IdlCPPGenerator::header_ =
"// **********************************************************************\n"
"//\n"
"// Generated by the ORBacus IDL-to-C++ Translator\n"
"//\n"
"// Copyright (c) 2000\n"
"// Object Oriented Concepts, Inc.\n"
"// Billerica, MA, USA\n"
"//\n"
"// All Rights Reserved\n"
"//\n"
"// **********************************************************************\n";

const char* IdlCPPGenerator::embeddedHeader_ =
"// **********************************************************************\n"
"//\n"
"// Generated by the ORBacus/E IDL-to-C++ Translator\n"
"//\n"
"// Copyright (c) 2000\n"
"// Object Oriented Concepts, Inc.\n"
"// Billerica, MA, USA\n"
"//\n"
"// All Rights Reserved\n"
"//\n"
"// **********************************************************************\n";

// ----------------------------------------------------------------------
// Version check
// ----------------------------------------------------------------------

void
IdlCPPGenerator::checkVersion(OB_STD(ostream&) s)
{
    if(embedded_)
    {
        s << '\n';
        s << "\n#ifndef OBE_INTEGER_VERSION";
        s << "\n#   error No ORBacus/E version defined!";
        s << "\n#endif";
        s << '\n';
        s << "\n#ifndef OBE_NO_VERSION_CHECK";
        s << "\n#   if (OBE_INTEGER_VERSION != 1000100L)";
        s << "\n#       error ORBacus/E version mismatch!";
        s << "\n#   endif";
        s << "\n#endif";
    }
    else
    {
        s << '\n';
        s << "\n#ifndef OB_INTEGER_VERSION";
        s << "\n#   error No ORBacus version defined!";
        s << "\n#endif";
        s << '\n';
        s << "\n#ifndef OB_NO_VERSION_CHECK";
        s << "\n#   if (OB_INTEGER_VERSION != " << OB_INTEGER_VERSION << "L)";
        s << "\n#       error ORBacus version mismatch!";
        s << "\n#   endif";
        s << "\n#endif";
    }
}

// ----------------------------------------------------------------------
// C++ code generator constructor and destructor
// ----------------------------------------------------------------------

IdlCPPGenerator::IdlCPPGenerator(const char* prog,
                                 ORB_ptr orb,
				 Repository_ptr rep,
				 bool all,
				 const IdlStringHash& idHash,
				 const IdlStringHash& fwdHash,
				 const StringSeq& includeSeq,
				 OB_STL::list<CORBA::String_var>& fileNameList,
				 bool noTC,
                                 bool local,
				 bool noVirtual,
                                 bool noSkel,
                                 bool withInterceptorArgs,
				 bool noOrbMediation,
				 bool noLocalCopy,
				 bool embedded, 
				 bool noDirectStubs, 
				 const char* thisHeaderDir,
				 const char* otherHeaderDir,
				 const char* dllImport,
				 const char* hSuff,
				 const char* cSuff,
				 const char* skelSuff)
    : prog_(prog),
      orb_(ORB::_duplicate(orb)),
      repository_(Repository::_duplicate(rep)),
      all_(all),
      idHash_(idHash),
      fwdHash_(fwdHash),
      includeSeq_(includeSeq),
      fileNameList_(fileNameList),
      noTC_(noTC),
      local_(local),
      noVirtual_(noVirtual),
      noSkel_(noSkel),
      withInterceptorArgs_(withInterceptorArgs),
      noOrbMediation_(noOrbMediation),
      noLocalCopy_(noLocalCopy),
      embedded_(embedded),
      noDirectStubs_(noDirectStubs),
      thisHeaderDir_(thisHeaderDir),
      otherHeaderDir_(otherHeaderDir),
      dllImport_(dllImport),
      hSuff_(hSuff),
      cSuff_(cSuff),
      skelSuff_(skelSuff),
      descriptionMap_(255),
      absMap_(1023),
      uniMap_(1023)
{
    if(local_)
	noTC_ = true;
}

IdlCPPGenerator::~IdlCPPGenerator()
{
    DescriptionMap::Enumerator enumerator = descriptionMap_.keys();
    for(ULong i = 0 ; i < enumerator.length() ; i++)
    {
	Container::DescriptionSeq* seq;
	bool b = descriptionMap_.get(enumerator[i], seq);
	assert(b);
	delete seq;
    }
}

// ----------------------------------------------------------------------
// Generate C++ code
// ----------------------------------------------------------------------

int
IdlCPPGenerator::gen(const char* dir, const char* name, bool fwd)
{
    ULong i;

    //
    // Open output files
    //
    String_var outputFileFwdH = dir;
    outputFileFwdH += name;
    outputFileFwdH += "_fwd";
    outputFileFwdH += hSuff_;

    String_var outputFileH = dir;
    outputFileH += name;
    outputFileH += hSuff_;

    String_var outputFileC = dir;
    outputFileC += name;
    outputFileC += cSuff_;

    OB_OFSTREAM outFwdH;
    if(fwd)
    {
	outFwdH.open(outputFileFwdH);
	if(!outFwdH.good())
	{
	    OB_CERR << prog_ << ": can't open \"" << outputFileFwdH
		         << "\": " << strerror(errno) << OB_ENDL;
	    
	    return 1;
	}

        fileNameList_.push_back(outputFileFwdH);
    }
    
    OB_OFSTREAM outH(outputFileH);
    if(!outH.good())
    {
	OB_CERR << prog_ << ": can't open \"" << outputFileH
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }
    
    fileNameList_.push_back(outputFileH);

    OB_OFSTREAM outC(outputFileC);
    if(!outC.good())
    {
	outH.close();

	OB_CERR << prog_ << ": can't open \"" << outputFileC
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }

    fileNameList_.push_back(outputFileC);

    //
    // Print headers
    //
    if(fwd)
    {
	String_var macroFwd = thisHeaderDir_;
	macroFwd += '/';
	macroFwd += name;
	macroFwd += "_fwd";
	macroFwd += hSuff_;
	macroFwd = IdlPathToIdent(macroFwd);
	
	if(embedded_)
	{
	    outFwdH << embeddedHeader_;
	    outFwdH << "\n// Version: " << OBEVersion;
	}
	else
	{
	    outFwdH << header_;
	    outFwdH << "\n// Version: " << Version;
	}
	outFwdH << '\n';
	outFwdH << "\n#ifndef __" << macroFwd << "__";
	outFwdH << "\n#define __" << macroFwd << "__";

	outFwdH << '\n';
	if(embedded_)
	    outFwdH << "\n#include <OBE/Template.h>";
	else
	    outFwdH << "\n#include <OB/Template.h>";
    }

    String_var macro = thisHeaderDir_;
    macro += '/';
    macro += name;
    macro += hSuff_;
    macro = IdlPathToIdent(macro);
    
    if(embedded_)
    {
        outH << embeddedHeader_;
        outH << "\n// Version: " << OBEVersion;
    }
    else
    {
        outH << header_;
        outH << "\n// Version: " << Version;
    }
    outH << '\n';
    outH << "\n#ifndef __" << macro << "__";
    outH << "\n#define __" << macro << "__";

    if(strlen(dllImport_) > 0)
    {
        outH << '\n';
        outH << "\n#ifndef " << dllImport_;
	if(embedded_)
            outH << "\n#   ifdef OBE_DLL";
	else
            outH << "\n#   ifdef OB_DLL";
        outH << "\n#      define " << dllImport_ << " __declspec(dllimport)";
        outH << "\n#   else";
        outH << "\n#      define " << dllImport_ << " /**/";
        outH << "\n#   endif";
        outH << "\n#endif";
        outH << '\n';
    }

    //
    // Don't include (non-existent) header files if --all was specified
    //
    if(!all_)
    {
	for(i = 0 ; i < includeSeq_.length() ; i++)
	{
	    if(i == 0)
		outH << '\n';
	    
	    outH << "\n#include <";
	    if(strlen(otherHeaderDir_) > 0)
		outH << otherHeaderDir_ << '/';
	    outH << includeSeq_[i] << hSuff_ << '>';
	}
    }

    if(fwd)
    {
	if(includeSeq_.length() == 0)
	    outH << '\n';

	outH << "\n#include <";
	if(strlen(thisHeaderDir_) > 0)
	    outH << thisHeaderDir_ << '/';
	outH << name << "_fwd" << hSuff_ << '>';
    }
    
    if(embedded_)
    {
        outC << embeddedHeader_;
        outC << "\n// Version: " << OBEVersion;
    }
    else
    {
        outC << header_;
        outC << "\n// Version: " << Version;
    }
    outC << '\n';
    if(embedded_)
        outC << "\n#include <OBE/CORBAClient.h>";
    else
        outC << "\n#include <OB/CORBAClient.h>";
    outC << "\n#include <";
    if(strlen(thisHeaderDir_) > 0)
	outC << thisHeaderDir_ << '/';
    outC << name << hSuff_ << '>';

    //
    // Generate code to check version
    //
    if(fwd)
	checkVersion(outFwdH);
    checkVersion(outH);
    checkVersion(outC);

    //
    // Generate code
    //
    if(fwd)
    {
	IdlPrettyPrint ppFwdH(outFwdH);
	genCPlusPlusFwdH(repository_, ppFwdH);
	genCPlusPlusFwdHGlobalNamespace(repository_, ppFwdH);
	genCPlusPlusFwdHVar(repository_, ppFwdH);
    }

    
    IdlPrettyPrint ppH(outH);
    if(!fwd)
    {
	genCPlusPlusFwdH(repository_, ppH);
	genCPlusPlusFwdHGlobalNamespace(repository_, ppH);
	genCPlusPlusFwdHVar(repository_, ppH);
    }
    if(!local_)
    {
	if(embedded_)
	{
	    genCPlusPlusFwdHMarshalStubImpl(repository_, ppH);
	}
	else
	{
	    genCPlusPlusFwdHStubImpl(repository_, ppH);
	    genCPlusPlusFwdHStubImplGlobalNamespace(repository_, ppH);
	    genCPlusPlusFwdHStubImplVar(repository_, ppH);
	}	    
    }
    genCPlusPlusH(repository_, ppH);
    if(!local_)
    {
	if(!embedded_)
	    genCPlusPlusHStubImpl(repository_, ppH);
        genCPlusPlusHMarshalStubImpl(repository_, ppH);
    }
    genCPlusPlusHOBV(repository_, ppH);
    genCPlusPlusHGlobalNamespace(repository_, ppH);

    IdlPrettyPrint ppC(outC);
    genCPlusPlusCConstants(repository_, ppC);
    genCPlusPlusC(repository_, ppC);
    if(!local_)
    {
	if(!embedded_)
	    genCPlusPlusCStubImpl(repository_, ppC);
        genCPlusPlusCMarshalStubImpl(repository_, ppC);
    }
    genCPlusPlusCOBV(repository_, ppC);
    
    //
    // That's it
    //
    if(fwd)
    {
	outFwdH << '\n';
	outFwdH << "\n#endif";
	outFwdH << '\n';
    }

    outH << '\n';
    outH << "\n#endif";
    outH << '\n';

    outC << '\n';

    return 0;
}

int
IdlCPPGenerator::genSkel(const char* dir, const char* name)
{
    if(local_)
        return 0;
    
    assert(!noSkel_);
    
    ULong i;

    //
    // Open output files
    //
    String_var outputFileSkelH = dir;
    outputFileSkelH += name;
    String_var outputFileSkelC = dir;
    outputFileSkelC += name;
    
    outputFileSkelH += skelSuff_;
    outputFileSkelC += skelSuff_;

    outputFileSkelH += hSuff_;
    outputFileSkelC += cSuff_;
   
    OB_OFSTREAM outSkelH(outputFileSkelH);
    if(!outSkelH.good())
    {
	OB_CERR << prog_ << ": can't open \"" << outputFileSkelH
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }
    
    fileNameList_.push_back(outputFileSkelH);

    OB_OFSTREAM outSkelC(outputFileSkelC);
    if(!outSkelC.good())
    {
	outSkelH.close();

	OB_CERR << prog_ << ": can't open \"" << outputFileSkelC
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }

    fileNameList_.push_back(outputFileSkelC);

    //
    // Print headers
    //
    String_var macroSkel = thisHeaderDir_;
    macroSkel += '/';
    macroSkel += name;
    macroSkel += skelSuff_;
    macroSkel += hSuff_;
    macroSkel = IdlPathToIdent(macroSkel);

    if(embedded_)
    {
        outSkelH << embeddedHeader_;
        outSkelH << "\n// Version: " << OBEVersion;
    }
    else
    {
        outSkelH << header_;
        outSkelH << "\n// Version: " << Version;
    }
    outSkelH << '\n';
    outSkelH << "\n#ifndef __" << macroSkel << "__";
    outSkelH << "\n#define __" << macroSkel << "__";
    outSkelH << '\n';
    outSkelH << "\n#include <";
    if(strlen(thisHeaderDir_) > 0)
	outSkelH << thisHeaderDir_ << '/';
    outSkelH << name << hSuff_ << '>';
    
    //
    // Don't include (non-existent) header files if --all was specified
    //
    if(!all_)
    {
        for(i = 0 ; i < includeSeq_.length() ; i++)
        {
            if(i == 0)
                outSkelH << '\n';
            
            outSkelH << "\n#include <";
            if(strlen(otherHeaderDir_) > 0)
                outSkelH << otherHeaderDir_ << '/';
            outSkelH << includeSeq_[i] << skelSuff_ << hSuff_ << '>';
        }
    }

    if(strlen(dllImport_) > 0)
    {
        outSkelH << '\n';
        outSkelH << "\n#ifndef " << dllImport_;
        outSkelH << "\n#   ifdef OB_DLL";
        outSkelH << "\n#      define " << dllImport_ <<
            " __declspec(dllimport)";
        outSkelH << "\n#   else";
        outSkelH << "\n#      define " << dllImport_ << " /**/";
        outSkelH << "\n#   endif";
        outSkelH << "\n#endif";
        outSkelH << '\n';
    }

    if(embedded_)
    {
        outSkelC << embeddedHeader_;
        outSkelC << "\n// Version: " << OBEVersion;
    }
    else
    {
        outSkelC << header_;
        outSkelC << "\n// Version: " << Version;
    }
    outSkelC << '\n';
    if(embedded_)
        outSkelC << "\n#include <OBE/CORBAServer.h>";
    else
        outSkelC << "\n#include <OB/CORBAServer.h>";
    outSkelC << "\n#include <";
    if(strlen(thisHeaderDir_) > 0)
	outSkelC << thisHeaderDir_ << '/';
    outSkelC << name << skelSuff_ << hSuff_ << '>';
    
    //
    // Generate code to check version
    //
    checkVersion(outSkelH);
    checkVersion(outSkelC);

    //
    // Generate code
    //
    IdlPrettyPrint ppSkelH(outSkelH);
    IdlPrettyPrint ppSkelC(outSkelC);

    genCPlusPlusSkelH(repository_, ppSkelH);
    if(!noDirectStubs_)
        genCPlusPlusSkelHDirectStubImpl(repository_, ppSkelH);

    genCPlusPlusSkelC(repository_, ppSkelC);
    if(!noDirectStubs_)
        genCPlusPlusSkelCDirectStubImpl(repository_, ppSkelC);
    
    //
    // That's it
    //
    outSkelH << '\n';
    outSkelH << "\n#endif";
    outSkelH << '\n';
    outSkelC << '\n';

    return 0;
}

int
IdlCPPGenerator::genTie(const char* dir, const char* name)
{
    if(local_)
	return 0;

    assert(!noSkel_);

    //
    // Open output files
    //
    String_var outputFileSkelH = dir;
    outputFileSkelH += name;
    String_var outputFileTieH = dir;
    outputFileTieH += name;
    
    outputFileSkelH += skelSuff_;
    outputFileTieH += skelSuff_;
    outputFileTieH += "_tie";

    outputFileSkelH += hSuff_;
    outputFileTieH += hSuff_;
    
    OB_OFSTREAM outTieH(outputFileTieH);
    if(!outTieH.good())
    {
	OB_CERR << prog_ << ": can't open \"" << outputFileTieH
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }
    
    fileNameList_.push_back(outputFileTieH);

    //
    // Print headers
    //
    String_var macroTie = thisHeaderDir_;
    macroTie += '/';
    macroTie += name;
    macroTie += "_tie";
    macroTie += hSuff_;
    macroTie = IdlPathToIdent(macroTie);

    if(embedded_)
    {
        outTieH << embeddedHeader_;
        outTieH << "\n// Version: " << OBEVersion;
    }
    else
    {
        outTieH << header_;
        outTieH << "\n// Version: " << Version;
    }
    outTieH << '\n';
    outTieH << "\n#ifndef __" << macroTie << "__";
    outTieH << "\n#define __" << macroTie << "__";
    outTieH << '\n';
    outTieH << "\n#include <";
    if(strlen(thisHeaderDir_) > 0)
	outTieH << thisHeaderDir_ << '/';
    outTieH << name << skelSuff_ << hSuff_ << '>';
    
    //
    // Generate code to check version
    //
    checkVersion(outTieH);

    //
    // Generate code
    //
    IdlPrettyPrint ppTieH(outTieH);

    genCPlusPlusTieH(repository_, ppTieH);
    
    //
    // That's it
    //
    outTieH << '\n';
    outTieH << "\n#endif";
    outTieH << '\n';

    return 0;
}

int
IdlCPPGenerator::genImpl(const char* dir, const char* name, bool all)
{
    assert(!noSkel_);

    ULong i;

    //
    // Open output files
    //
    String_var outputFileSkelH = dir;
    outputFileSkelH += name;
    String_var outputFileImplH = dir;
    outputFileImplH += name;
    String_var outputFileImplC = dir;
    outputFileImplC += name;
    
    outputFileSkelH += skelSuff_;
    outputFileImplH += "_impl";
    outputFileImplC += "_impl";

    outputFileSkelH += hSuff_;
    outputFileImplH += hSuff_;
    outputFileImplC += cSuff_;

    //
    // Do NOT overwrite existing files!
    //
    struct stat st;
    if(stat(outputFileImplH, &st) == 0 ||
       stat(outputFileImplC, &st) == 0)
    {
        OB_CERR << prog_ << ": will not overwrite \"" << outputFileImplH
                     << "\" or \"" << outputFileImplC << "\"" << OB_ENDL;

        return 1;
    }

    OB_OFSTREAM outImplH(outputFileImplH);
    if(!outImplH.good())
    {
	OB_CERR << prog_ << ": can't open \"" << outputFileImplH
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }
    
    fileNameList_.push_back(outputFileImplH);

    OB_OFSTREAM outImplC(outputFileImplC);
    if(!outImplC.good())
    {
	outImplH.close();

	OB_CERR << prog_ << ": can't open \"" << outputFileImplC
	             << "\": " << strerror(errno) << OB_ENDL;
	
	return 1;
    }

    fileNameList_.push_back(outputFileImplC);

    //
    // Print headers
    //
    String_var macroImpl = thisHeaderDir_;
    macroImpl += '/';
    macroImpl += name;
    macroImpl += "_impl";
    macroImpl += hSuff_;
    macroImpl = IdlPathToIdent(macroImpl);

    outImplH << "#ifndef __" << macroImpl << "__";
    outImplH << "\n#define __" << macroImpl << "__";

    //
    // If implementation inheritance is being used, then include
    // the implementation header files
    //
    if(!all)
    {
        for(i = 0 ; i < includeSeq_.length() ; i++)
        {
            if(i == 0)
                outImplH << '\n';
            
            outImplH << "\n#include <";
            if(strlen(otherHeaderDir_) > 0)
                outImplH << otherHeaderDir_ << '/';
            outImplH << includeSeq_[i] << "_impl" << hSuff_ << '>';
        }
    }

    if(local_)
    {
        outImplH << '\n';
        outImplH << "\n#include <";
        if(strlen(thisHeaderDir_) > 0)
            outImplH << thisHeaderDir_ << '/';
        outImplH << name << hSuff_ << '>';
    }
    else
    {
        outImplH << '\n';
        outImplH << "\n#include <";
        if(strlen(thisHeaderDir_) > 0)
            outImplH << thisHeaderDir_ << '/';
        outImplH << name << skelSuff_ << hSuff_ << '>';
    }

    if(embedded_)
        outImplC << "#include <OBE/CORBA.h>";
    else
        outImplC << "#include <OB/CORBA.h>";
    outImplC << "\n#include <";
    if(strlen(thisHeaderDir_) > 0)
	outImplC << thisHeaderDir_ << '/';
    outImplC << name << "_impl" << hSuff_ << '>';

    //
    // Generate code
    //
    IdlPrettyPrint ppImplH(outImplH);
    IdlPrettyPrint ppImplC(outImplC);

    genCPlusPlusImplH(repository_, all, ppImplH);
    genCPlusPlusImplC(repository_, all, ppImplC);
    
    //
    // That's it
    //
    outImplH << '\n';
    outImplH << "\n#endif";
    outImplH << '\n';
    outImplC << '\n';

    return 0;
}
