// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef HAVE_STRINGS_H
#   include <strings.h>
#endif

#ifndef HAVE_STRCASECMP
#   define strcasecmp stricmp
#endif

#include <Util.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Convert strings with escape sequences
// ----------------------------------------------------------------------

void
IdlAddEscapes(char*& str, bool addQuotes)
{
    char* s = str;
    String_var result = string_dup("");

    if(addQuotes)
	result += '"';

    while(*s)
    {
	switch(*s)
	{
	case '\n':
	    result += "\\n";
	    break;

	case '\t':
	    result += "\\t";
	    break;

	case '\v':
	    result += "\\v";
	    break;

	case '\b':
	    result += "\\b";
	    break;

	case '\r':
	    result += "\\r";
	    break;

	case '\f':
	    result += "\\f";
	    break;

	case '\a':
	    result += "\\a";
	    break;

	case '\\':
	    result += "\\\\";
	    break;
	    
 	case '\?':
 	    result += "\\?";
 	    break;
	    
	case '\'':
	    result += "\\'";
	    break;

	case '"':
	    result += "\\\"";
	    break;

	default:
	    if(*s >= 32 && *s < 127)
		result += *s;
	    else
	    {
		Char c = *s;
		char str[5];
		sprintf(str, "\\%.3o", (unsigned char)c);
		result += str;
	    }
	    break;
	}
	
	s++;
    }

    if(addQuotes)
	result += '"';

    string_free(str);
    str = result._retn();
}

char*
IdlAddEscapes(const wchar_t* str, bool addQuotes)
{
    const wchar_t* s = str;
    String_var result = string_dup("");

    if(addQuotes)
	result += '"';

    while(*s)
    {
	switch(*s)
	{
	case L'\n':
	    result += "\\n";
	    break;

	case L'\t':
	    result += "\\t";
	    break;

	case L'\v':
	    result += "\\v";
	    break;

	case L'\b':
	    result += "\\b";
	    break;

	case L'\r':
	    result += "\\r";
	    break;

	case L'\f':
	    result += "\\f";
	    break;

	case L'\a':
	    result += "\\a";
	    break;

	case L'\\':
	    result += "\\\\";
	    break;
	    
 	case L'\?':
 	    result += "\\?";
 	    break;
	    
	case L'\'':
	    result += "\\'";
	    break;

	case L'"':
	    result += "\\\"";
	    break;

	default:
            if(*s >= 32 && *s < 127)
                result += (Char)*s;
            else if(*s < 256)
            {
		char str[5];
		sprintf(str, "\\%.3o", (unsigned char)*s);
                result += str;
            }
            else
            {
		char str[10];
		sprintf(str, "\\u%lx", (ULong)*s);
		result += str;
            }
	    break;
	}
	
	s++;
    }

    if(addQuotes)
	result += '"';

    return result._retn();
}

bool
IdlRemoveEscapes(char*& str, bool removeQuotes)
{
    char* s = str;
    String_var result = string_dup("");
    bool b = true;

    if(removeQuotes && *s == '"')
	s++;

    while(*s)
    {
	if(removeQuotes && *(s + 1) == 0 && *s == '"')
	    break;

	if(*s != '\\')
	{
	    result += *s;
	    s++;
	}
	else
	{
	    switch(*(s + 1))
	    {
	    case 'n':
		result += '\n';
		s += 2;
		break;

	    case 't':
		result += '\t';
		s += 2;
		break;

	    case 'v':
		result += '\v';
		s += 2;
		break;

	    case 'b':
		result += '\b';
		s += 2;
		break;

	    case 'r':
		result += '\r';
		s += 2;
		break;

	    case 'f':
		result += '\f';
		s += 2;
		break;

	    case 'a':
		result += '\a';
		s += 2;
		break;

	    case '\\':
		result += '\\';
		s += 2;
		break;

	    case '?':
		result += '\?';
		s += 2;
		break;

	    case '\'':
		result += '\'';
		s += 2;
		break;

	    case '"':
		result += '\"';
		s += 2;
		break;

	    case 'x':
	    case 'u':
	    {
		if((*(s + 2) >= '0' && *(s + 2) <= '9') ||
		   (*(s + 2) >= 'a' && *(s + 2) <= 'f') ||
		   (*(s + 2) >= 'A' && *(s + 2) <= 'F'))
		{
		    char* s2;
		    Char c = (Char)strtol(s + 2, &s2, 16);

 		    if(c == 0)
			b = false;
		    else
			result += c;

		    s = s2;
		}
		else
		{
		    result += '\\';
		    s++;
		}

		break;
	    }
	    
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    {
		char* s2;
		Char c = (Char)strtol(s + 1, &s2, 8);

 		if(c == 0)
		    b = false;
		else			
		    result += c;

		s = s2;

		break;
	    }

	    default:
		result += '\\';
		s++;
		break;
	    }
	}
    }

    string_free(str);
    str = result._retn();

    return b;
}

bool
IdlRemoveEscapes(const char* str, wchar_t*& wstr, bool removeQuotes)
{
    const char* s = str;
    WString_var result = wstring_dup(L"");
    bool b = true;

    if(removeQuotes && *s == '"')
	s++;

    while(*s)
    {
	if(removeQuotes && *(s + 1) == 0 && *s == '"')
	    break;

	if(*s != '\\')
	{
	    result += (WChar)*s;
	    s++;
	}
	else
	{
	    switch(*(s + 1))
	    {
	    case 'n':
		result += L'\n';
		s += 2;
		break;

	    case 't':
		result += L'\t';
		s += 2;
		break;

	    case 'v':
		result += L'\v';
		s += 2;
		break;

	    case 'b':
		result += L'\b';
		s += 2;
		break;

	    case 'r':
		result += L'\r';
		s += 2;
		break;

	    case 'f':
		result += L'\f';
		s += 2;
		break;

	    case 'a':
		result += L'\a';
		s += 2;
		break;

	    case '\\':
		result += L'\\';
		s += 2;
		break;

	    case '?':
		result += L'\?';
		s += 2;
		break;

	    case '\'':
		result += L'\'';
		s += 2;
		break;

	    case '"':
		result += L'\"';
		s += 2;
		break;

	    case 'x':
	    case 'u':
	    {
		if((*(s + 2) >= '0' && *(s + 2) <= '9') ||
		   (*(s + 2) >= 'a' && *(s + 2) <= 'f') ||
		   (*(s + 2) >= 'A' && *(s + 2) <= 'F'))
		{
		    char* s2;
		    WChar c = (WChar)strtol(s + 2, &s2, 16);

 		    if(c == 0)
			b = false;
		    else
			result += c;

		    s = s2;
		}
		else
		{
		    result += L'\\';
		    s++;
		}

		break;
	    }
	    
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    {
		char* s2;
		WChar c = (WChar)strtol(s + 1, &s2, 8);
		
 		if(c == 0)
		    b = false;
		else
		    result += c;

		s = s2;

		break;
	    }

	    default:
		result += L'\\';
		s++;
		break;
	    }
	}
    }

    wstr = result._retn();

    return b;
}

// ----------------------------------------------------------------------
// Compare two strings. The return value has the usual strcmp()-like
// semantics. The global variable IdlCaseSensitive determines whether
// case is considered during the comparison.
// ----------------------------------------------------------------------

bool IdlCaseSensitive;

int
IdlStringCompare(const char* s1, const char* s2)
{
    if(IdlCaseSensitive)
        return strcmp(s1, s2);
    else
        return strcasecmp(s1, s2);
}
