// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <stdlib.h>

#include <TestCodeSets.h>

using namespace CORBA;

int
run(ORB_ptr orb, int argc, char* argv[])
{
    try
    {
	Object_var obj = orb -> string_to_object("relfile:TestCodeSets.ref");
	if(is_nil(obj))
	{
	    OB_CERR << argv[0]
		    << ": cannot read IOR from TestCodeSets.ref"
		    << OB_ENDL;
	    return EXIT_FAILURE;
	}
	
	TestCodeSets_var p = TestCodeSets::_narrow(obj);
	assert(!is_nil(p));

	{
	    WChar wc, wcRet;

	    wc = L'a';
	    wcRet = p -> testWChar(wc);
	    assert(wc == wcRet);
	}

	{
	    WString_var ws, wsRet;

	    ws = wstring_dup(L"Hello, World!");
	    wsRet = p -> testWString(ws);
	    assert(OB::wcscmp(ws, wsRet) == 0);
	}

	p -> deactivate();
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	return 1;
    }

    return 0;
}

int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        //
        // Create ORB
        //
        orb = ORB_init(argc, argv);

	//
	// Run the test
	//
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
        OB_CERR << ex << OB_ENDL;
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
