// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf_2_3.h>
#include <TestMacro.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;

int
ClientRun(ORB_ptr orb, bool local, int argc, char* argv[])
{
    ULong i;

    //
    // Get TestIntf
    //
    const char* refFile = "TestIntf.ref";
    OB_IFSTREAM in; // Must use open(name), not ifstream in(name) (VC++ bug)
    in.open(refFile);
    if(in.fail())
    {
	OB_CERR << argv[0] << ": can't open `" << refFile << "': "
		<< strerror(errno) << OB_ENDL;
	return EXIT_FAILURE;
    }

    //
    // Get the stringified IORs
    //
    String_var impl[3];
    for(i = 0 ; i < 3 ; i++)
	in >> impl[i];

    in.close(); // The explicit close is necessary to satisfy the Windows NT
                // file locking

    //
    // Run tests
    //

    OB_COUT << "Testing string_to_object()... " << OB_FLUSH;
    Object_var obj[3];
    for(i = 0 ; i < 3 ; i++)
    {
	obj[i] = orb -> string_to_object(impl[i]);
	TEST(!is_nil(obj[i]));
    }
    OB_COUT << "OK!" << OB_ENDL;

    OB_COUT << "Testing _narrow()... " << OB_FLUSH;
    TestIntf_var ti[3];
    for(i = 0 ; i < 3 ; i++)
    {
	ti[i] = TestIntf::_narrow(obj[i]);
	TEST(!is_nil(ti[i]));
    }
    TestIntf_2_3_NoLongDouble_var ti_2_3_nld[3];
    for(i = 0 ; i < 3 ; i++)
	ti_2_3_nld[i] = TestIntf_2_3_NoLongDouble::_narrow(obj[i]);
    TestIntf_2_3_var ti_2_3[3];
    for(i = 0 ; i < 3 ; i++)
	ti_2_3[i] = TestIntf_2_3::_narrow(obj[i]);
    OB_COUT << "OK!" << OB_ENDL;

    const char* objDesc[] = 
    {
	"SSI",
	"DSI",
	"Tie"
    };

    extern void TestStub(TestIntf_ptr);
    OB_COUT << "Testing the SII w/ 2.0 types:" << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestStub(ti[i]);
	OB_COUT << "OK!" << OB_ENDL;
    }

    if(!is_nil(ti_2_3_nld[0]))
    {
	extern void TestStub_2_3_NoLongDouble(TestIntf_2_3_NoLongDouble_ptr);
	OB_COUT << "Testing the SII w/ 2.3 types w/o long double type:"
		<< OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestStub_2_3_NoLongDouble(ti_2_3_nld[i]);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    if(!is_nil(ti_2_3[0]))
    {
	extern void TestStub_2_3(TestIntf_2_3_ptr);
	OB_COUT << "Testing the SII w/ long double type:" << OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestStub_2_3(ti_2_3[i]);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    extern void TestStubException(TestIntf_ptr, bool);
    OB_COUT << "Testing the SII and exceptions w/ 2.0 types:" << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestStubException(ti[i], !local);
	OB_COUT << "OK!" << OB_ENDL;
    }

    if(!is_nil(ti_2_3_nld[0]))
    {
	extern void TestStubException_2_3_NoLongDouble(
	    TestIntf_2_3_NoLongDouble_ptr, bool);
	OB_COUT
	    << "Testing SII and exceptions w/ 2.3 types w/o long double type:"
	    << OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestStubException_2_3_NoLongDouble(ti_2_3_nld[i], !local);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    if(!is_nil(ti_2_3[0]))
    {
	extern void TestStubException_2_3(TestIntf_2_3_ptr, bool);
	OB_COUT << "Testing SII and exceptions w/ long double type:"
		<< OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestStubException_2_3(ti_2_3[i], !local);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    extern void TestStubContext(TestIntf_ptr, ORB_ptr);
    OB_COUT << "Testing the SII w/ contexts:" << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestStubContext(ti[i], orb);
	OB_COUT << "OK!" << OB_ENDL;
    }

    extern void TestDII(TestIntf_ptr, ORB_ptr);
    OB_COUT << "Testing the DII w/ 2.0 types:" << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestDII(ti[i], orb);
	OB_COUT << "OK!" << OB_ENDL;
    }

    if(!is_nil(ti_2_3_nld[0]))
    {
	extern void TestDII_2_3_NoLongDouble(TestIntf_2_3_NoLongDouble_ptr,
					     ORB_ptr);
	OB_COUT << "Testing the DII w/ 2.3 types w/o long double type:"
		<< OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestDII_2_3_NoLongDouble(ti_2_3_nld[i], orb);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    if(!is_nil(ti_2_3[0]))
    {
	extern void TestDII_2_3(TestIntf_2_3_ptr);
	OB_COUT << "Testing the DII w/ long double type:" << OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    OB_COUT << "\tWith the " << objDesc[i]
		    << " implementation... " << OB_FLUSH;
	    TestDII_2_3(ti_2_3[i]);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    extern void TestDIIException(TestIntf_ptr, bool);
    OB_COUT << "Testing the DII and exceptions w/ 2.0 types:"
		 << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestDIIException(ti[i], !local);
	OB_COUT << "OK!" << OB_ENDL;
    }

    /* No 2.3 DII exception tests yet */

    extern void TestDIIContext(TestIntf_ptr, ORB_ptr);
    OB_COUT << "Testing the DII w/ contexts:" << OB_ENDL;
    for(i = 0 ; i < 3 ; i++)
    {
	OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		<< OB_FLUSH;
	TestDIIContext(ti[i], orb);
	OB_COUT << "OK!" << OB_ENDL;
    }

    if(!local || ti[0] -> concurrentRequestExecution())
    {
	extern void TestStubTimeout(TestIntf_ptr, ORB_ptr);
        OB_COUT << "Testing the SII w/ timeouts:" << OB_ENDL;
	for(i = 0 ; i < 3 ; i++)
	{
	    //
	    // Only the marshal stubs work
	    // TODO: There should be a policy to select marshal stubs
	    //
	    if(local && i != 1)
		continue;

	    OB_COUT << "\tWith the " << objDesc[i] << " implementation... "
		    << OB_FLUSH;
	    TestStubTimeout(ti[i], orb);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }

    //
    // Deactivate the server
    //
    ULong version = ti[0] -> version();
    if(version >= 3999900L)
    {
	ti[0] -> deactivate();
    }
    else
    {
	try
	{
	    ti[0] -> deactivate();
	}
	catch(const TRANSIENT&)
	{
	}
    }

    return EXIT_SUCCESS;
}
