// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf_2_3_skel_tie.h>
#include <Test_impl.h>
#include <TestDSI_impl.h>
#include <TestMacro.h>

#include <errno.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;

static TestIntf_2_3_impl* i = 0;
static TestIntf_2_3_DSI_impl* iDSI = 0;
static POA_TestIntf_2_3_tie< TestIntf_2_3_impl >* iTie = 0;
static const char* refFile = "TestIntf.ref";

int
ServerRun(ORB_ptr orb, bool nonBlocking, int argc, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);
	    
    //
    // Activate the POA manager
    //
    PortableServer::POAManager_var manager = poa -> the_POAManager();
    manager -> activate();
    
    //
    // Create implementation objects
    //
    i = new TestIntf_2_3_impl(orb);
    TestIntf_2_3_var p = i -> _this();
    String_var impl = orb -> object_to_string(p);
    
    iDSI = new TestIntf_2_3_DSI_impl(p, orb);
    PortableServer::ObjectId_var id = poa -> activate_object(iDSI);
    Object_var pDSI =
	poa -> create_reference_with_id(id, "IDL:TestIntf_2_3:1.0");
    String_var implDSI = orb -> object_to_string(pDSI);
    
    iTie = new POA_TestIntf_2_3_tie< TestIntf_2_3_impl >(i, poa, false);
    Object_var pTie = iTie -> _this();
    String_var implTie = orb -> object_to_string(pTie);

    //
    // Save references. This must be done after POA manager activation,
    // otherwise there is a potential for a race condition between the
    // client sending request and the server not being ready yet.
    //
    OB_OFSTREAM out(refFile);
    if(out.fail())
    {
	OB_CERR << argv[0] << ": can't open `" << refFile << "': "
		     << strerror(errno) << OB_ENDL;
	return EXIT_FAILURE;
    }
    
    out << impl << OB_ENDL;
    out << implDSI << OB_ENDL;
    out << implTie << OB_ENDL;

    out.close();

    if(!nonBlocking)
    {
	//
	// Give up control to the ORB
	//
        orb -> run();

        //
        // Clean up
        //
        void ServerCleanup();
        ServerCleanup();
    }

    return EXIT_SUCCESS;
}

void
ServerCleanup()
{
    delete iDSI;
    delete iTie;
    delete i;

#ifdef HAVE__UNLINK
    _unlink(refFile);
#else
    unlink(refFile);
#endif
}
