// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf.h>
#include <TestMacro.h>

using namespace CORBA;

void
TestDIIContext(TestIntf_ptr ti, ORB_ptr orb)
{
    Context_var ctx;

    {
	orb -> get_default_context(ctx.out());

	Request_var request;

	request = ti -> _request("opContext");
	request -> contexts() -> add("A*");
	request -> contexts() -> add("C*");
	request -> contexts() -> add("X");
	request -> contexts() -> add("Z");
	request -> ctx(ctx);
	request -> add_in_arg() <<= (const char*)"*";
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> invoke();
	Exception* ex = request -> env() -> exception();
	BAD_CONTEXT* bex;
	TEST(bex = BAD_CONTEXT::_downcast(ex));
	TEST(bex);
    }

    {
	orb -> get_default_context(ctx.out());

	Any any;
	ULong i;

	any <<= (const char*)"A1";
	ctx -> set_one_value("A", any);

	any <<= (const char*)"A2";
	ctx -> set_one_value("AAA", any);

	any <<= (const char*)"B1";
	ctx -> set_one_value("B", any);

	any <<= (const char*)"B2";
	ctx -> set_one_value("BBB", any);

	any <<= (const char*)"X1";
	ctx -> set_one_value("X", any);

	any <<= (const char*)"X2";
	ctx -> set_one_value("XXX", any);

	any <<= (const char*)"Y1";
	ctx -> set_one_value("Y", any);

	any <<= (const char*)"Y2";
	ctx -> set_one_value("YYY", any);

	Request_var request;
	const TestIntf::StringSequence* seq;

	request = ti -> _request("opContext");
	request -> contexts() -> add("A*");
	request -> contexts() -> add("C*");
	request -> contexts() -> add("X");
	request -> contexts() -> add("Z");
	request -> ctx(ctx);
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> add_in_arg() <<= (const char*)"*";
	request -> invoke();
	TEST(request -> return_value() >>= seq);

	TEST(seq -> length() == 3 * 2);
	for(i = 0 ; i < seq -> length() ; i += 2)
        {
	    if(strcmp((*seq)[i], "A") == 0)
	        TEST(strcmp((*seq)[i + 1], "A1") == 0);

	    if(strcmp((*seq)[i], "AAA") == 0)
	        TEST(strcmp((*seq)[i + 1], "A2") == 0);

	    if(strcmp((*seq)[i], "X") == 0)
	        TEST(strcmp((*seq)[i + 1], "X1") == 0);
	}

	request = ti -> _request("opContext");
	request -> contexts() -> add("A*");
	request -> contexts() -> add("C*");
	request -> contexts() -> add("X");
	request -> contexts() -> add("Z");
	request -> ctx(ctx);
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> add_in_arg() <<= (const char*)"A*";
	request -> invoke();
	TEST(request -> return_value() >>= seq);

	TEST(seq -> length() == 2 * 2);
	for(i = 0 ; i < seq -> length() ; i += 2)
        {
	    if(strcmp((*seq)[i], "A") == 0)
	        TEST(strcmp((*seq)[i + 1], "A1") == 0);

	    if(strcmp((*seq)[i], "AAA") == 0)
	        TEST(strcmp((*seq)[i + 1], "A2") == 0);
	}

	request = ti -> _request("opContext");
	request -> contexts() -> add("A*");
	request -> contexts() -> add("C*");
	request -> contexts() -> add("X");
	request -> contexts() -> add("Z");
	request -> ctx(ctx);
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> add_in_arg() <<= (const char*)"AA*";
	request -> invoke();
	TEST(request -> return_value() >>= seq);

	TEST(seq -> length() == 1 * 2);
	TEST(strcmp((*seq)[0], "AAA") == 0 &&
	     strcmp((*seq)[1], "A2") == 0);

	request = ti -> _request("opContext");
	request -> contexts() -> add("A*");
	request -> contexts() -> add("C*");
	request -> contexts() -> add("X");
	request -> contexts() -> add("Z");
	request -> ctx(ctx);
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> add_in_arg() <<= (const char*)"A";
	request -> invoke();
	TEST(request -> return_value() >>= seq);

	TEST(seq -> length() == 1 * 2);
	TEST(strcmp((*seq)[0], "A") == 0 &&
	     strcmp((*seq)[1], "A1") == 0);
    }
}
