// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf_2_3.h>
#include <TestMacro.h>

using namespace CORBA;

void
TestDIIException(TestIntf_ptr ti, bool remote)
{
    Request_var request;

    {
	request = ti -> _request("opVoidEx");
	request -> invoke();
	Exception* ex = request -> env() -> exception();
	TEST(UNKNOWN::_downcast(ex));

	request = ti -> _request("opVoidEx");
	request -> exceptions() -> add(TestIntf::_tc_ExVoid);
	request -> invoke();
	ex = request -> env() -> exception();
	UnknownUserException* uex;
	TEST(uex = UnknownUserException::_downcast(ex));
	const TestIntf::ExVoid* iex;
	TEST(uex -> exception() >>= iex);
    }

    {
	request = ti -> _request("opShortEx");
	request -> exceptions() -> add(TestIntf::_tc_ExShort);
	request -> add_in_arg() <<= (Short)10;
	Any& inOutAny = request -> add_inout_arg();
	inOutAny <<= (Short)20;
	Any& outAny = request -> add_out_arg();
	outAny.replace(_tc_short, 0);
	request -> set_return_type(_tc_short);
	request -> invoke();
	Exception* ex = request -> env() -> exception();
	UnknownUserException* uex;
	TEST(uex = UnknownUserException::_downcast(ex));
	const TestIntf::ExShort* iex;
	TEST(uex -> exception() >>= iex);
	TEST(iex -> value == 30);
    }

    {
	request = ti -> _request("opDoubleEx");
	request -> exceptions() -> add(TestIntf::_tc_ExDouble);
	request -> add_in_arg() <<= 10.0;
	Any& inOutAny = request -> add_inout_arg();
	inOutAny <<= 20.0;
	Any& outAny = request -> add_out_arg();
	outAny.replace(_tc_double, 0);
	request -> set_return_type(_tc_double);
	request -> invoke();
	Exception* ex = request -> env() -> exception();
	UnknownUserException* uex;
	TEST(uex = UnknownUserException::_downcast(ex));
	const TestIntf::ExDouble* iex;
	TEST(uex -> exception() >>= iex);
	TEST(iex -> value == 30);
    }

    {
	TestIntf::StringSequence in;
	in.length(3);
	in[0] = string_dup("!!!");
	in[1] = string_dup("@@@");
	in[2] = string_dup("###");
	TestIntf::StringSequence inOut;
	inOut.length(2);
	inOut[0] = string_dup("%");
	inOut[1] = string_dup("^^");
	request = ti -> _request("opStringSequenceEx");
	request -> exceptions() -> add(TestIntf::_tc_ExStringSequence);
	request -> add_in_arg() <<= in;
	Any& inOutAny = request -> add_inout_arg();
	inOutAny <<= inOut;
	Any& outAny = request -> add_out_arg();
	outAny.replace(TestIntf::_tc_StringSequence, 0);
	request -> set_return_type(TestIntf::_tc_StringSequence);
	request -> invoke();
	Exception* ex = request -> env() -> exception();
	UnknownUserException* uex;
	TEST(uex = UnknownUserException::_downcast(ex));
	const TestIntf::ExStringSequence* iex;
	TEST(uex -> exception() >>= iex);
	TEST(iex -> value.length() == 5);
	TEST(strcmp(iex -> value[0], "!!!") == 0);
	TEST(strcmp(iex -> value[1], "@@@") == 0);
	TEST(strcmp(iex -> value[2], "###") == 0);
	TEST(strcmp(iex -> value[3], "%") == 0);
	TEST(strcmp(iex -> value[4], "^^") == 0);
    }

    {
        try
        {
            request = ti -> _request("op_BAD_PARAM_Ex");
            request -> invoke();
            Exception* ex = request -> env() -> exception();
            BAD_PARAM* bp = BAD_PARAM::_downcast(ex);
            TEST(bp);
            bp -> _raise();
        }
        catch(const BAD_PARAM& ex)
        {
            TEST(ex.minor() == 2);
            TEST(ex.remote() == remote);
            TEST(ex.completed() == COMPLETED_NO);
        }
    }
}
