// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestDSI_impl.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;

TestIntf_2_3_DSI_impl::TestIntf_2_3_DSI_impl(TestIntf_2_3_ptr ti,
					     ORB_ptr orb)
    : ti_(TestIntf_2_3::_duplicate(ti)),
      orb_(ORB::_duplicate(orb))
{
}

RepositoryId
TestIntf_2_3_DSI_impl::_primary_interface(const PortableServer::ObjectId& oid,
					  PortableServer::POA_ptr poa)
{
    return string_dup("IDL:TestIntf_2_3:1.0");
}

Boolean
TestIntf_2_3_DSI_impl::_is_a(const char* name)
    throw(SystemException)
{
    if(strcmp(name, "IDL:TestIntf:1.0") == 0 ||
       strcmp(name, "IDL:TestIntf_2_3_NoLongDouble:1.0") == 0 ||
       strcmp(name, "IDL:TestIntf_2_3:1.0") == 0)
    {
	return true;
    }
    else
    {
#ifdef HAVE_VCPLUSPLUS_BUGS
	return ServantBase::_is_a(name);
#else
	return PortableServer::ServantBase::_is_a(name);
#endif
    }
}

void
TestIntf_2_3_DSI_impl::invoke(ServerRequest_ptr request) throw()
{
    String_var name = request -> operation();

    bool ex;
    if(strlen(name) > 2 &&
       strcmp((const char*)name + strlen(name) - 2, "Ex") == 0)
    {
	name[strlen(name) - 2] = '\0';
	ex = true;
    }
    else
	ex = false;

    // ----------------------------------------------------------------------
    // Methods from TestIntf
    // ----------------------------------------------------------------------

    if(strcmp(name, "version") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	ULong ret = ti_ -> version();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "concurrentRequestExecution") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Boolean ret = ti_ -> concurrentRequestExecution();

	Any any;
	any <<= Any::from_boolean(ret);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "sleepOneway") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ulong, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULong arg;
	*any >>= arg;

	ti_ -> sleepOneway(arg);

	return;
    }

    if(strcmp(name, "sleepTwoway") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ulong, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULong arg;
	*any >>= arg;

	ti_ -> sleepTwoway(arg);

	return;
    }

    if(strcmp(name, "opVoid") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	ti_ -> opVoid();

	if(ex)
	{
	    Any any;
	    any <<= new TestIntf::ExVoid();
	    request -> set_exception(any);
	}

	return;
    }

    if(strcmp(name, "opContext") == 0)
    {
	NVList_ptr list;
	Any* pany;
        Any any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_string, 0);
	request -> arguments(list);
	Context_ptr ctx = request -> ctx();

	pany = list -> item(0) -> value();
	const char* pattern;
	*pany >>= pattern;

	try
	{
	    TestIntf::StringSequence* ret =
		ti_ -> opContext(pattern, ctx);

	    any <<= ret;
	    request -> set_result(any);
	}
	catch(const BAD_CONTEXT& ex)
	{
	    any <<= ex;
	    request -> set_exception(any);
	}

	return;
    }

    if(strcmp(name, "_get_attrShort") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Short ret = ti_ -> attrShort();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrShort") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_short, 0);
	request -> arguments(list);

	Any* any = list -> item(0) -> value();
	Short arg;
	*any >>= arg;

	ti_ -> attrShort(arg);

	return;
    }

    if(strcmp(name, "opShort") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_short, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_short, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Short arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	Short arg1;
	*any >>= arg1;

	Short arg2;

	Short ret = ti_ -> opShort(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;
	
	any = list -> item(2) -> value();
	*any <<= arg2;
	
	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExShort(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrUShort") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	UShort ret = ti_ -> attrUShort();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrUShort") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ushort, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	UShort arg;
	*any >>= arg;

	ti_ -> attrUShort(arg);

	return;
    }

    if(strcmp(name, "opUShort") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ushort, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_ushort, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	UShort arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	UShort arg1;
	*any >>= arg1;

	UShort arg2;

	UShort ret = ti_ -> opUShort(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExUShort(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrLong") == 0)
    {
	NVList_ptr list;

	orb_ -> create_list(0, list);
	request -> arguments(list);

	Long ret = ti_ -> attrLong();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_long, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Long arg;
	*any >>= arg;

	ti_ -> attrLong(arg);

	return;
    }

    if(strcmp(name, "opLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_long, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_long, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Long arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	Long arg1;
	*any >>= arg1;

	Long arg2;

	Long ret = ti_ -> opLong(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExLong(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrULong") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	ULong ret = ti_ -> attrULong();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrULong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ulong, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULong arg;
	*any >>= arg;

	ti_ -> attrULong(arg);

	return;
    }

    if(strcmp(name, "opULong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ulong, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_ulong, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULong arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	ULong arg1;
	*any >>= arg1;

	ULong arg2;

	ULong ret = ti_ -> opULong(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExULong(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFloat") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Float ret = ti_ -> attrFloat();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFloat") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_float, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Float arg;
	*any >>= arg;

	ti_ -> attrFloat(arg);

	return;
    }

    if(strcmp(name, "opFloat") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_float, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_float, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Float arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	Float arg1;
	*any >>= arg1;

	Float arg2;

	Float ret = ti_ -> opFloat(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExFloat(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrDouble") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Double ret = ti_ -> attrDouble();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrDouble") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_double, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Double arg;
	*any >>= arg;

	ti_ -> attrDouble(arg);

	return;
    }

    if(strcmp(name, "opDouble") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_double, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_double, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Double arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	Double arg1;
	*any >>= arg1;

	Double arg2;

	Double ret = ti_ -> opDouble(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExDouble(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrBoolean") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Boolean ret = ti_ -> attrBoolean();

	Any any;
	any <<= Any::from_boolean(ret);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrBoolean") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_boolean, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Boolean arg;
	*any >>= Any::to_boolean(arg);

	ti_ -> attrBoolean(arg);

	return;
    }

    if(strcmp(name, "opBoolean") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_boolean, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_boolean, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Boolean arg0;
	*any >>= Any::to_boolean(arg0);

	any = list -> item(1) -> value();
	Boolean arg1;
	*any >>= Any::to_boolean(arg1);

	Boolean arg2;

	Boolean ret = ti_ -> opBoolean(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= Any::from_boolean(arg1);

	any = list -> item(2) -> value();
	*any <<= Any::from_boolean(arg2);

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExBoolean(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= Any::from_boolean(ret);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrChar") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Char ret = ti_ -> attrChar();

	Any any;
	any <<= Any::from_char(ret);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrChar") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_char, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Char arg;
	*any >>= Any::to_char(arg);

	ti_ -> attrChar(arg);

	return;
    }

    if(strcmp(name, "opChar") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_char, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_char, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Char arg0;
	*any >>= Any::to_char(arg0);

	any = list -> item(1) -> value();
	Char arg1;
	*any >>= Any::to_char(arg1);

	Char arg2;

	Char ret = ti_ -> opChar(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= Any::from_char(arg1);

	any = list -> item(2) -> value();
	*any <<= Any::from_char(arg2);

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExChar(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= Any::from_char(ret);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrOctet") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Octet ret = ti_ -> attrOctet();

	Any any;
	any <<= Any::from_octet(ret);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrOctet") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_octet, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Octet arg;
	*any >>= Any::to_octet(arg);

	ti_ -> attrOctet(arg);

	return;
    }

    if(strcmp(name, "opOctet") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_octet, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_octet, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	Octet arg0;
	*any >>= Any::to_octet(arg0);

	any = list -> item(1) -> value();
	Octet arg1;
	*any >>= Any::to_octet(arg1);

	Octet arg2;

	Octet ret = ti_ -> opOctet(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= Any::from_octet(arg1);

	any = list -> item(2) -> value();
	*any <<= Any::from_octet(arg2);

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExOctet(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= Any::from_octet(ret);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrString") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	String_var ret = ti_ -> attrString();

	Any any;
	any <<= ret.in();
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrString") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_string, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const char* arg;
	*any >>= arg;

	ti_ -> attrString(arg);

	return;
    }

    if(strcmp(name, "opString") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_string, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_string, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const char* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const char* tmpArg1;
	*any >>= tmpArg1;

	String_var arg1 = string_dup(tmpArg1);
	String_var arg2;

	String_var ret = ti_ -> opString(arg0, arg1.inout(), arg2.out());

	any = list -> item(1) -> value();
	*any <<= arg1.in();

	any = list -> item(2) -> value();
	*any <<= arg2.in();

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExString(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret.in();
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrAny") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any* ret = ti_ -> attrAny();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrAny") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_any, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const Any* arg;
	*any >>= arg;

	ti_ -> attrAny(*arg);

	return;
    }

    if(strcmp(name, "opAny") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_any, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_any, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const Any* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const Any* arg1;
	*any >>= arg1;
	
	Any* arg2;

	Any* ret = ti_ -> opAny(*arg0, *(Any*)arg1, arg2);

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExAny(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrTestEnum") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::TestEnum ret = ti_ -> attrTestEnum();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrTestEnum") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(TestIntf::_tc_TestEnum, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::TestEnum arg;
	*any >>= arg;

	ti_ -> attrTestEnum(arg);

	return;
    }

    if(strcmp(name, "opTestEnum") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(TestIntf::_tc_TestEnum, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_TestEnum, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::TestEnum arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	TestIntf::TestEnum arg1;
	*any >>= arg1;

	TestIntf::TestEnum arg2;

	TestIntf::TestEnum ret = ti_ -> opTestEnum(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExTestEnum(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrTestIntf") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf_var ret = ti_ -> attrTestIntf();

	Any any;
	any <<= ret.in();
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrTestIntf") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_TestIntf, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf_ptr arg;
	*any >>= arg;

	ti_ -> attrTestIntf(arg);

	return;
    }

    if(strcmp(name, "opTestIntf") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_TestIntf, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_TestIntf, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf_ptr arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	TestIntf_ptr tmpArg1;
	*any >>= tmpArg1;

	TestIntf_var arg1 = TestIntf::_duplicate(tmpArg1);
	TestIntf_var arg2;

	TestIntf_var ret = ti_ -> opTestIntf(arg0, arg1.inout(), arg2.out());

	any = list -> item(1) -> value();
	*any <<= arg1.in();

	any = list -> item(2) -> value();
	*any <<= arg2.in();

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExTestIntf(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret.in();
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixedStruct") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::FixedStruct ret = ti_ -> attrFixedStruct();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixedStruct") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedStruct, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedStruct* arg;
	*any >>= arg;

	ti_ -> attrFixedStruct(*arg);

	return;
    }

    if(strcmp(name, "opFixedStruct") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedStruct, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_FixedStruct, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedStruct* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::FixedStruct *parg1;
	*any >>= parg1;
        TestIntf::FixedStruct arg1(*parg1);

	TestIntf::FixedStruct arg2;

	TestIntf::FixedStruct ret =
	    ti_ -> opFixedStruct(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExFixedStruct(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrVariableStruct") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::VariableStruct* ret = ti_ -> attrVariableStruct();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrVariableStruct") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableStruct, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableStruct* arg;
	*any >>= arg;

	ti_ -> attrVariableStruct(*arg);

	return;
    }

    if(strcmp(name, "opVariableStruct") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableStruct, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_VariableStruct, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableStruct* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::VariableStruct* parg1;
	*any >>= parg1;
        TestIntf::VariableStruct arg1(*parg1);

	TestIntf::VariableStruct* arg2;

	TestIntf::VariableStruct* ret =
	    ti_ -> opVariableStruct(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExVariableStruct(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixedUnion") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::FixedUnion ret = ti_ -> attrFixedUnion();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixedUnion") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedUnion, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedUnion* arg;
	*any >>= arg;

	ti_ -> attrFixedUnion(*arg);

	return;
    }

    if(strcmp(name, "opFixedUnion") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedUnion, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_FixedUnion, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedUnion* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::FixedUnion* parg1;
	*any >>= parg1;
        TestIntf::FixedUnion arg1(*parg1);

	TestIntf::FixedUnion arg2;

	TestIntf::FixedUnion ret =
	    ti_ -> opFixedUnion(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExFixedUnion(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrVariableUnion") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::VariableUnion* ret = ti_ -> attrVariableUnion();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrVariableUnion") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableUnion, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableUnion* arg;
	*any >>= arg;

	ti_ -> attrVariableUnion(*arg);

	return;
    }

    if(strcmp(name, "opVariableUnion") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableUnion, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_VariableUnion, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableUnion* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::VariableUnion* parg1;
	*any >>= parg1;
        TestIntf::VariableUnion arg1(*parg1);

	TestIntf::VariableUnion* arg2;

	TestIntf::VariableUnion* ret =
	    ti_ -> opVariableUnion(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExVariableUnion(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrStringSequence") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::StringSequence* ret = ti_ -> attrStringSequence();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrStringSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_StringSequence, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::StringSequence* arg;
	*any >>= arg;

	ti_ -> attrStringSequence(*arg);

	return;
    }

    if(strcmp(name, "opStringSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_StringSequence, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_StringSequence, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::StringSequence* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::StringSequence* parg1;
	*any >>= parg1;
        TestIntf::StringSequence arg1(*parg1);

	TestIntf::StringSequence* arg2;

	TestIntf::StringSequence* ret =
	    ti_ -> opStringSequence(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExStringSequence(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixedArray") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::FixedArray_slice* ret = ti_ -> attrFixedArray();

	Any any;
	any <<= TestIntf::FixedArray_forany(ret, true);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixedArray") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArray, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::FixedArray_forany arg;
	*any >>= arg;

	ti_ -> attrFixedArray(arg);

	return;
    }

    if(strcmp(name, "opFixedArray") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArray, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_FixedArray, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::FixedArray_forany arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	TestIntf::FixedArray_forany arg1;
	*any >>= arg1;

	TestIntf::FixedArray arg2;

	TestIntf::FixedArray_slice* ret =
	    ti_ -> opFixedArray(arg0, arg1.inout(), arg2);
	
	any = list -> item(2) -> value();
	*any <<= TestIntf::FixedArray_forany(arg2, false);

	if(ex)
	{
	    Any exAny;
#ifdef HAVE_VCPLUSPLUS_BUGS
	    exAny <<= new TestIntf::ExFixedArray(
		(const TestIntf::FixedArray_slice*)ret);
#else
	    exAny <<= new TestIntf::ExFixedArray(ret);
#endif
	    TestIntf::FixedArray_free(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= TestIntf::FixedArray_forany(ret, true);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrVariableArray") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::VariableArray_slice* ret = ti_ -> attrVariableArray();

	Any any;
	any <<= TestIntf::VariableArray_forany(ret, true);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrVariableArray") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArray, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::VariableArray_forany arg;
	*any >>= arg;

	ti_ -> attrVariableArray(arg);

	return;
    }

    if(strcmp(name, "opVariableArray") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArray, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_VariableArray, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf::VariableArray_forany arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	TestIntf::VariableArray_forany arg1;
	*any >>= arg1;

	TestIntf::VariableArray_slice* arg2;

	TestIntf::VariableArray_slice* ret =
	    ti_ -> opVariableArray(arg0, arg1.inout(), arg2);
	
	any = list -> item(2) -> value();
	*any <<= TestIntf::VariableArray_forany(arg2, true);

	if(ex)
	{
	    Any exAny;
#ifdef HAVE_VCPLUSPLUS_BUGS
	    exAny <<= new TestIntf::ExVariableArray(
		(const TestIntf::VariableArray_slice*)ret);
#else
	    exAny <<= new TestIntf::ExVariableArray(ret);
#endif
	    TestIntf::VariableArray_free(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= TestIntf::VariableArray_forany(ret, true);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixedArraySequence") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::FixedArraySequence* ret =
	    ti_ -> attrFixedArraySequence();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixedArraySequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArraySequence, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedArraySequence* arg;
	*any >>= arg;

	ti_ -> attrFixedArraySequence(*arg);

	return;
    }

    if(strcmp(name, "opFixedArraySequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArraySequence, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_FixedArraySequence, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedArraySequence* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::FixedArraySequence* parg1;
	*any >>= parg1;
	TestIntf::FixedArraySequence arg1(*parg1);

	TestIntf::FixedArraySequence* arg2;

	TestIntf::FixedArraySequence* ret =
	    ti_ -> opFixedArraySequence(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExFixedArraySequence(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrVariableArraySequence") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::VariableArraySequence* ret =
	    ti_ -> attrVariableArraySequence();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrVariableArraySequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArraySequence, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableArraySequence* arg;
	*any >>= arg;

	ti_ -> attrVariableArraySequence(*arg);

	return;
    }

    if(strcmp(name, "opVariableArraySequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArraySequence, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_VariableArraySequence, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableArraySequence* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::VariableArraySequence* parg1;
	*any >>= parg1;
	TestIntf::VariableArraySequence arg1(*parg1);

	TestIntf::VariableArraySequence* arg2;

	TestIntf::VariableArraySequence* ret =
	    ti_ -> opVariableArraySequence(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExVariableArraySequence(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixedArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::FixedArrayBoundSequence* ret =
	    ti_ -> attrFixedArrayBoundSequence();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixedArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArrayBoundSequence, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedArrayBoundSequence* arg;
	*any >>= arg;

	ti_ -> attrFixedArrayBoundSequence(*arg);

	return;
    }

    if(strcmp(name, "opFixedArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_FixedArrayBoundSequence, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_FixedArrayBoundSequence, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::FixedArrayBoundSequence* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::FixedArrayBoundSequence* parg1;
	*any >>= parg1;
	TestIntf::FixedArrayBoundSequence arg1(*parg1);

	TestIntf::FixedArrayBoundSequence* arg2;

	TestIntf::FixedArrayBoundSequence* ret =
	    ti_ -> opFixedArrayBoundSequence(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExFixedArrayBoundSequence(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrVariableArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf::VariableArrayBoundSequence* ret =
	    ti_ -> attrVariableArrayBoundSequence();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrVariableArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArrayBoundSequence, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableArrayBoundSequence* arg;
	*any >>= arg;

	ti_ -> attrVariableArrayBoundSequence(*arg);

	return;
    }

    if(strcmp(name, "opVariableArrayBoundSequence") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(TestIntf::_tc_VariableArrayBoundSequence, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(TestIntf::_tc_VariableArrayBoundSequence, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const TestIntf::VariableArrayBoundSequence* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const TestIntf::VariableArrayBoundSequence* parg1;
	*any >>= parg1;
	TestIntf::VariableArrayBoundSequence arg1(*parg1);

	TestIntf::VariableArrayBoundSequence* arg2;

	TestIntf::VariableArrayBoundSequence* ret =
	    ti_ -> opVariableArrayBoundSequence(*arg0, arg1, arg2);

        *any <<= arg1;
	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf::ExVariableArrayBoundSequence(*ret);
	    delete ret;
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "op_UNKNOWN_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new UNKNOWN(1, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_PARAM_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_PARAM(2, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_NO_MEMORY_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new NO_MEMORY(3, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_IMP_LIMIT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new IMP_LIMIT(4, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_COMM_FAILURE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new COMM_FAILURE(5, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INV_OBJREF_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INV_OBJREF(6, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_NO_PERMISSION_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new NO_PERMISSION(7, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INTERNAL_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INTERNAL(8, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_MARSHAL_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new MARSHAL(9, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INITIALIZE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INITIALIZE(10, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_NO_IMPLEMENT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new NO_IMPLEMENT(11, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_TYPECODE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_TYPECODE(12, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_OPERATION_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_OPERATION(13, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_NO_RESOURCES_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new NO_RESOURCES(14, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_NO_RESPONSE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new NO_RESPONSE(15, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_PERSIST_STORE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new PERSIST_STORE(16, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_INV_ORDER_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_INV_ORDER(17, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TRANSIENT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TRANSIENT(18, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_FREE_MEM_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new FREE_MEM(19, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INV_IDENT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INV_IDENT(20, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INV_FLAG_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INV_FLAG(21, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INTF_REPOS_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INTF_REPOS(22, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_CONTEXT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_CONTEXT(23, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_OBJ_ADAPTER_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new OBJ_ADAPTER(24, COMPLETED_MAYBE);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_DATA_CONVERSION_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new DATA_CONVERSION(25, COMPLETED_YES);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_OBJECT_NOT_EXIST_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new OBJECT_NOT_EXIST(26, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TRANSACTION_REQUIRED_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TRANSACTION_REQUIRED(27, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TRANSACTION_ROLLEDBACK_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TRANSACTION_ROLLEDBACK(28, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INVALID_TRANSACTION_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INVALID_TRANSACTION(29, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_INV_POLICY_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new INV_POLICY(30, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_CODESET_INCOMPATIBLE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new CODESET_INCOMPATIBLE(31, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_REBIND_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new REBIND(32, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TIMEOUT_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TIMEOUT(33, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TRANSACTION_UNAVAILABLE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TRANSACTION_UNAVAILABLE(34, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_TRANSACTION_MODE_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new TRANSACTION_MODE(35, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    if(strcmp(name, "op_BAD_QOS_") == 0)
    {
	assert(ex);

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	Any any;
	any <<= new BAD_QOS(36, COMPLETED_NO);
	request -> set_exception(any);

	return;
    }

    // ----------------------------------------------------------------------
    // Methods from TestIntf_2_3_NoLongDouble
    // ----------------------------------------------------------------------

    if(strcmp(name, "_get_attrLongLong") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	LongLong ret = ti_ -> attrLongLong();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrLongLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_longlong, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	LongLong arg;
	*any >>= arg;

	ti_ -> attrLongLong(arg);

	return;
    }

    if(strcmp(name, "opLongLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_longlong, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_longlong, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	LongLong arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	LongLong arg1;
	*any >>= arg1;

	LongLong arg2;

	LongLong ret = ti_ -> opLongLong(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3_NoLongDouble::ExLongLong(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrULongLong") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	ULongLong ret = ti_ -> attrULongLong();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrULongLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_ulonglong, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULongLong arg;
	*any >>= arg;

	ti_ -> attrULongLong(arg);

	return;
    }

    if(strcmp(name, "opULongLong") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value()
	    -> replace(_tc_ulonglong, 0);
	list -> add(ARG_INOUT) -> value()
	    -> replace(_tc_ulonglong, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	ULongLong arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	ULongLong arg1;
	*any >>= arg1;

	ULongLong arg2;

	ULongLong ret = ti_ -> opULongLong(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3_NoLongDouble::ExULongLong(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrFixed") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	TestIntf_2_3_NoLongDouble::TestFixed ret = ti_ -> attrFixed();

	Any any;
	any <<= Any::from_fixed(ret, 24, 8);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrFixed") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	TypeCode_var tcFixed = orb_ -> create_fixed_tc(24, 8);
	list -> add(ARG_IN) -> value() -> replace(tcFixed, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf_2_3_NoLongDouble::TestFixed arg;
	*any >>= Any::to_fixed(arg, 24, 8);
	ti_ -> attrFixed(arg);

	return;
    }

    if(strcmp(name, "opFixed") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	TypeCode_var tcFixed = orb_ -> create_fixed_tc(24, 8);
	list -> add(ARG_IN) -> value() -> replace(tcFixed, 0);
	list -> add(ARG_INOUT) -> value() -> replace(tcFixed, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	TestIntf_2_3_NoLongDouble::TestFixed arg0;
	*any >>= Any::to_fixed(arg0, 24, 8);

	any = list -> item(1) -> value();
	TestIntf_2_3_NoLongDouble::TestFixed arg1;
	*any >>= Any::to_fixed(arg1, 24, 8);

	TestIntf_2_3_NoLongDouble::TestFixed arg2;

	TestIntf_2_3_NoLongDouble::TestFixed ret = ti_ -> opFixed(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= Any::from_fixed(arg1, 24, 8);

	any = list -> item(2) -> value();
	*any <<= Any::from_fixed(arg2, 24, 8);

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3_NoLongDouble::ExFixed(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= Any::from_fixed(ret, 24, 8);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrWChar") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	WChar ret = ti_ -> attrWChar();

	Any any;
	any <<= Any::from_wchar(ret);
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrWChar") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_wchar, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	WChar arg;
	*any >>= Any::to_wchar(arg);

	ti_ -> attrWChar(arg);

	return;
    }

    if(strcmp(name, "opWChar") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_wchar, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_wchar, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	WChar arg0;
	*any >>= Any::to_wchar(arg0);

	any = list -> item(1) -> value();
	WChar arg1;
	*any >>= Any::to_wchar(arg1);

	WChar arg2;

	WChar ret = ti_ -> opWChar(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= Any::from_wchar(arg1);

	any = list -> item(2) -> value();
	*any <<= Any::from_wchar(arg2);

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3_NoLongDouble::ExWChar(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= Any::from_wchar(ret);
	    request -> set_result(result);
	}

	return;
    }

    if(strcmp(name, "_get_attrWString") == 0)
    {
	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	WString_var ret = ti_ -> attrWString();

	Any any;
	any <<= ret.in();
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrWString") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_wstring, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const wchar_t* arg;
	*any >>= arg;

	ti_ -> attrWString(arg);

	return;
    }

    if(strcmp(name, "opWString") == 0)
    {
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_wstring, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_wstring, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	const wchar_t* arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	const wchar_t* tmpArg1;
	*any >>= tmpArg1;

	WString_var arg1 = wstring_dup(tmpArg1);
	WString_var arg2;

	WString_var ret = ti_ -> opWString(arg0, arg1.inout(),
						 arg2.out());

	any = list -> item(1) -> value();
	*any <<= arg1.in();

	any = list -> item(2) -> value();
	*any <<= arg2.in();

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3_NoLongDouble::ExWString(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret.in();
	    request -> set_result(result);
	}

	return;
    }

    // ----------------------------------------------------------------------
    // Methods from TestIntf_2_3
    // ----------------------------------------------------------------------

    if(strcmp(name, "_get_attrLongDouble") == 0)
    {
#if SIZEOF_LONG_DOUBLE < 12
	throw MARSHAL(OB::MinorLongDoubleNotSupported, COMPLETED_NO);
#endif

	NVList_ptr list;
	orb_ -> create_list(0, list);
	request -> arguments(list);

	LongDouble ret = ti_ -> attrLongDouble();

	Any any;
	any <<= ret;
	request -> set_result(any);

	return;
    }

    if(strcmp(name, "_set_attrLongDouble") == 0)
    {
#if SIZEOF_LONG_DOUBLE < 12
	throw MARSHAL(OB::MinorLongDoubleNotSupported, COMPLETED_NO);
#endif
	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_longdouble, 0);
	request -> arguments(list);

	any = list -> item(0) -> value();
	LongDouble arg;
	*any >>= arg;

	ti_ -> attrLongDouble(arg);

	return;
    }

    if(strcmp(name, "opLongDouble") == 0)
    {
#if SIZEOF_LONG_DOUBLE < 12
	throw MARSHAL(OB::MinorLongDoubleNotSupported, COMPLETED_NO);
#endif

	NVList_ptr list;
	Any* any;

	orb_ -> create_list(0, list);
	list -> add(ARG_IN) -> value() -> replace(_tc_longdouble, 0);
	list -> add(ARG_INOUT) -> value() -> replace(_tc_longdouble, 0);
	list -> add(ARG_OUT);
	request -> arguments(list);

	any = list -> item(0) -> value();
	LongDouble arg0;
	*any >>= arg0;

	any = list -> item(1) -> value();
	LongDouble arg1;
	*any >>= arg1;

	LongDouble arg2;

	LongDouble ret = ti_ -> opLongDouble(arg0, arg1, arg2);

	any = list -> item(1) -> value();
	*any <<= arg1;

	any = list -> item(2) -> value();
	*any <<= arg2;

	if(ex)
	{
	    Any exAny;
	    exAny <<= new TestIntf_2_3::ExLongDouble(ret);
	    request -> set_exception(exAny);
	}
	else
	{
	    Any result;
	    result <<= ret;
	    request -> set_result(result);
	}

	return;
    }

    //
    // Unknown operation
    //

    OB_CERR << "DSI implementation: unknown operation: " << name
		 << OB_ENDL;

    NVList_ptr list;
    orb_ -> create_list(0, list);
    request -> arguments(list);
    
    Any exAny;
    exAny <<= new BAD_OPERATION();
    request -> set_exception(exAny);
}
