// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf_2_3.h>
#include <TestMacro.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// Tests for TestIntf
// ----------------------------------------------------------------------

void
TestStub(TestIntf_ptr ti)
{
    ULong i, j, k, l;

    {
	ti -> opVoid();
    }

    {
	Short ret;
	ti -> attrShort(-32768);
	ret = ti -> attrShort();
	TEST(ret == -32768);

	ti -> attrShort(32767);
	ret = ti -> attrShort();
	TEST(ret == 32767);

	Short inOut, out;
	inOut = 20;
	ret = ti -> opShort(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	Long ret;
	ti -> attrLong(-2147483647 - 1);
	ret = ti -> attrLong();
	TEST(ret == -2147483647 - 1);

	ti -> attrLong(2147483647);
	ret = ti -> attrLong();
	TEST(ret == 2147483647);

	Long inOut, out;
	inOut = 20;
	ret = ti -> opLong(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	UShort ret;
	ti -> attrUShort(65535U);
	ret = ti -> attrUShort();
	TEST(ret == 65535U);

	UShort inOut, out;
	inOut = 20;
	ret = ti -> opUShort(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	ULong ret;
	ti -> attrULong(4294967295U);
	ret = ti -> attrULong();
	TEST(ret == 4294967295U);

	ULong inOut, out;
	inOut = 20;
	ret = ti -> opULong(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	Float ret;
	ti -> attrFloat(3.40282347E+38F);
	ret = ti -> attrFloat();
	TEST(ret == 3.40282347E+38F);

	ti -> attrFloat(1.17549435E-38F);
	ret = ti -> attrFloat();
	TEST(ret == 1.17549435E-38F);

	Float inOut, out;
	inOut = 20;
	ret = ti -> opFloat(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	Double ret;
	ti -> attrDouble(1.7976931348623157E+308);
	ret = ti -> attrDouble();
	TEST(ret == 1.7976931348623157E+308);

	ti -> attrDouble(2.2250738585072014E-308);
	ret = ti -> attrDouble();
#if !( defined(__alpha) && defined(__linux) )
 	TEST(ret == 2.2250738585072014E-308);
#endif

	Double inOut, out;
	inOut = 20;
	ret = ti -> opDouble(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	Boolean ret;
	ti -> attrBoolean(true);
	ret = ti -> attrBoolean();
	TEST(ret == true);

	ti -> attrBoolean(false);
	ret = ti -> attrBoolean();
	TEST(ret == false);

	Boolean inOut, out;
	inOut = true;
	ret = ti -> opBoolean(true, inOut, out);
	TEST(ret == true);
	TEST(inOut == true);
	TEST(out == true);

	inOut = true;
	ret = ti -> opBoolean(false, inOut, out);
	TEST(ret == false);
	TEST(inOut == false);
	TEST(out == false);

	inOut = false;
	ret = ti -> opBoolean(true, inOut, out);
	TEST(ret == false);
	TEST(inOut == false);
	TEST(out == false);
    }

    {
	Char ret;
	ti -> attrChar('a');
	ret = ti -> attrChar();
	TEST(ret == 'a');
	ti -> attrChar((Char)224);
	ret = ti -> attrChar();
	TEST(ret == (Char)224);

	Char inOut, out;
	inOut = 1;
	ret = ti -> opChar('a', inOut, out);
	TEST(ret == 'b');
	TEST(inOut == 'b');
	TEST(out == 'b');
    }

    {
	Octet ret;
	ti -> attrOctet(0xff);
	ret = ti -> attrOctet();
	TEST(ret == 0xff);

	ti -> attrOctet(0);
	ret = ti -> attrOctet();
	TEST(ret == 0);

	Octet inOut, out;
	inOut = 20;
	ret = ti -> opOctet(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	String_var ret;
	ti -> attrString("Hello");
	ret = ti -> attrString();
	TEST(strcmp(ret, "Hello") == 0);

	String_var inOut, out;
	inOut = string_dup("world!");
	ret = ti -> opString("Hello, ", inOut.inout(), out.out());
	TEST(strcmp(ret, "Hello, world!") == 0);
	TEST(strcmp(out, "Hello, world!") == 0);
    }

    {
	Any any;
	Any_var ret;

	any <<= (const char*)"abc";
	ti -> attrAny(any);
	ret = ti -> attrAny();
	const char* s;
	TEST(ret >>= s);
	TEST(strcmp(s, "abc") == 0);

	any <<= (Double)3.14;
	ti -> attrAny(any);
	ret = ti -> attrAny();
	Double d;
	TEST(ret >>= d);
	TEST(d == 3.14);

	TestIntf::VariableStruct vStruct;
	vStruct.s = string_dup("xyz");
	any <<= vStruct;
	Any inOut;
	Any_var out;
	ret = ti -> opAny(any, inOut, out);
	const TestIntf::VariableStruct* vStructRet;
	const TestIntf::VariableStruct* vStructInOut;
	const TestIntf::VariableStruct* vStructOut;
	TEST(ret >>= vStructRet);
	TEST(inOut >>= vStructInOut);
	TEST(out >>= vStructOut);
	TEST(strcmp(vStructRet -> s, "xyz") == 0);
	TEST(strcmp(vStructInOut -> s, "xyz") == 0);
	TEST(strcmp(vStructOut -> s, "xyz") == 0);

	any <<= ti;
	ti -> attrAny(any);
	Any_var any3 = ti -> attrAny();
	Object_var obj; // Used to be _ptr, but with the new
                        // CORBA 2.3 mapping, to_object() duplicates
	TEST(!(any3 >>= obj.inout()));
	TEST(any3 >>= Any::to_object(obj));
	TestIntf_ptr ti2;
	TEST(any3 >>= ti2);
	TEST(ti -> _hash(1000) == ti2 -> _hash(1000));
	TEST(ti -> _is_equivalent(ti2));
	TEST(ti -> _hash(1111) == obj -> _hash(1111));
	TEST(ti -> _is_equivalent(obj));
	TEST(obj -> _hash(1234) == ti2 -> _hash(1234));
	TEST(obj -> _is_equivalent(ti2));
    }

    {
	TestIntf::TestEnum ret;
	ti -> attrTestEnum(TestIntf::TestEnum2);
	ret = ti -> attrTestEnum();
	TEST(ret == TestIntf::TestEnum2);

	ti -> attrTestEnum(TestIntf::TestEnum3);
	ret = ti -> attrTestEnum();
	TEST(ret == TestIntf::TestEnum3);

	TestIntf::TestEnum inOut, out;
	inOut = TestIntf::TestEnum2;
	ret = ti -> opTestEnum(TestIntf::TestEnum3, inOut, out);
	TEST(ret == TestIntf::TestEnum3);
	TEST(inOut == TestIntf::TestEnum3);
	TEST(out == TestIntf::TestEnum3);
    }

    {
	TestIntf_var ret;
	ti -> attrTestIntf(ti);
	ret = ti -> attrTestIntf();
	TEST(ret -> _hash(999) == ti -> _hash(999));
	TEST(ret -> _is_equivalent(ti));

	TestIntf_var inOut, out;
	inOut = TestIntf::_duplicate(ti);
	ret = ti -> opTestIntf(ti, inOut.inout(), out.out());
	TEST(ret -> _hash(1001) == ti -> _hash(1001));
	TEST(ret -> _is_equivalent(ti));
	TEST(inOut -> _hash(5000) == ti -> _hash(5000));
	TEST(inOut -> _is_equivalent(ti));
	TEST(out -> _hash(2000) == ti -> _hash(2000));
	TEST(out -> _is_equivalent(ti));
    }

    {
	TestIntf::FixedStruct st;
	st.s = 100;
	st.l = -100;

	TestIntf::FixedStruct ret;
	ti -> attrFixedStruct(st);
	ret = ti -> attrFixedStruct();
	TEST(ret.s == st.s);
	TEST(ret.l == st.l);

	TestIntf::FixedStruct inOut, out;
	inOut.s = 1000;
	inOut.l = 10000;
	ret = ti -> opFixedStruct(st, inOut, out);
	TEST(ret.s == st.s);
	TEST(ret.l == st.l);
	TEST(inOut.s == st.s);
	TEST(inOut.l == st.l);
	TEST(out.s == st.s);
	TEST(out.l == st.l);
    }

    {
	TestIntf::VariableStruct st;
	st.s = string_dup("$$$");

	TestIntf::VariableStruct_var ret;
	ti -> attrVariableStruct(st);
	ret = ti -> attrVariableStruct();
	TEST(strcmp(ret -> s, st.s) == 0);

	TestIntf::VariableStruct inOut;
	TestIntf::VariableStruct_var out;
	inOut.s = string_dup("bla");
	ret = ti -> opVariableStruct(st, inOut, out);
	TEST(strcmp(ret -> s, st.s) == 0);
	TEST(strcmp(inOut.s, st.s) == 0);
	TEST(strcmp(out -> s, st.s) == 0);
    }

    {
	TestIntf::FixedUnion un;
	un.d(1.23);

	TestIntf::FixedUnion ret;
	ti -> attrFixedUnion(un);
	ret = ti -> attrFixedUnion();
	TEST(ret._d() == 2);
	TEST(ret.d() == 1.23);

	un.b(true);
	un._d(999);
	TestIntf::FixedUnion inOut, out;
	inOut.l(100);
	ret = ti -> opFixedUnion(un, inOut, out);
	TEST(ret._d() == 999);
	TEST(ret.b() == true);
	TEST(out._d() == 999);
	TEST(out.b() == true);
	TEST(inOut._d() == 999);
	TEST(inOut.b() == true);

	TestIntf::FixedStruct st;
	st.s = 10101;
	st.l = -10101;
	un.st(st);
	inOut.l(100);
	ret = ti -> opFixedUnion(un, inOut, out);
	TEST(ret._d() == 3);
	TEST(ret.st().s == 10101);
	TEST(ret.st().l == -10101);
	TEST(out._d() == 3);
	TEST(out.st().s == 10101);
	TEST(out.st().l == -10101);
	TEST(inOut._d() == 3);
	TEST(inOut.st().s == 10101);
	TEST(inOut.st().l == -10101);
    }

    {
	TestIntf::VariableUnion un;
	TestIntf::VariableStruct st;
	st.s = string_dup("$$$");
	un.st(st);

	TestIntf::VariableUnion_var ret;
	ti -> attrVariableUnion(un);
	ret = ti -> attrVariableUnion();
	TEST(strcmp(ret -> st().s, "$$$") == 0);

	un.ti(ti);
	TestIntf::VariableUnion inOut;
	TestIntf::VariableUnion_var out;
	inOut.st(st);
	ret = ti -> opVariableUnion(un, inOut, out);
	TEST(ret -> ti() -> _hash(1000) == ti -> _hash(1000));
	TEST(ret -> ti() -> _is_equivalent(ti));
	TEST(inOut.ti() -> _hash(5000) == ti -> _hash(5000));
	TEST(inOut.ti() -> _is_equivalent(ti));
	TEST(out -> ti() -> _hash(2000) == ti -> _hash(2000));
	TEST(out -> ti() -> _is_equivalent(ti));
    }

    {
	TestIntf::StringSequence seq;
	seq.length(3);
	seq[0] = string_dup("!!!");
	seq[1] = string_dup("@@@");
	seq[2] = string_dup("###");

	TestIntf::StringSequence_var ret;
	ti -> attrStringSequence(seq);
	ret = ti -> attrStringSequence();
	TEST(ret -> length() == 3);
	TEST(strcmp(ret[0], "!!!") == 0);
	TEST(strcmp(ret[1], "@@@") == 0);
	TEST(strcmp(ret[2], "###") == 0);

	TestIntf::StringSequence inOut;
	TestIntf::StringSequence_var out;
	inOut.length(2);
	inOut[0] = string_dup("%");
	inOut[1] = string_dup("^^");
	ret = ti -> opStringSequence(seq, inOut, out);
	TEST(ret -> length() == 5);
	TEST(strcmp(ret[0], "!!!") == 0);
	TEST(strcmp(ret[1], "@@@") == 0);
	TEST(strcmp(ret[2], "###") == 0);
	TEST(strcmp(ret[3], "%") == 0);
	TEST(strcmp(ret[4], "^^") == 0);
	TEST(inOut.length() == 5);
	TEST(strcmp(inOut[0], "!!!") == 0);
	TEST(strcmp(inOut[1], "@@@") == 0);
	TEST(strcmp(inOut[2], "###") == 0);
	TEST(strcmp(inOut[3], "%") == 0);
	TEST(strcmp(inOut[4], "^^") == 0);
	TEST(out -> length() == 5);
	TEST(strcmp(out[0], "!!!") == 0);
	TEST(strcmp(out[1], "@@@") == 0);
	TEST(strcmp(out[2], "###") == 0);
	TEST(strcmp(out[3], "%") == 0);
	TEST(strcmp(out[4], "^^") == 0);
    }

    {
	TestIntf::FixedArray ar =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray_var ret;
#ifdef HAVE_VCPLUSPLUS_BUGS
	ti -> attrFixedArray((const TestIntf::FixedArray_slice*)ar);
#else
	ti -> attrFixedArray(ar);
#endif
	ret = ti -> attrFixedArray();
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		for(k = 0 ; k < 4 ; k++)
		{
		    TEST(ar[i][j][k] == ret[i][j][k]);
		}

	TestIntf::FixedArray inOut, out;
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		for(k = 0 ; k < 4 ; k++)
		    inOut[i][j][k] = (short)(i + j + k);
#ifdef HAVE_VCPLUSPLUS_BUGS
	ret = ti -> opFixedArray((const TestIntf::FixedArray_slice*)ar,
				 inOut, out);
#else
	ret = ti -> opFixedArray(ar, inOut, out);
#endif
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		for(k = 0 ; k < 4 ; k++)
		{
		    TEST(ar[i][j][k] == ret[i][j][k]);
		    TEST(ar[i][j][k] == inOut[i][j][k]);
		    TEST(ar[i][j][k] == out[i][j][k]);
		}
    }

    {
	TestIntf::VariableArray ar =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};
	
	TestIntf::VariableArray_var ret;
#ifdef HAVE_VCPLUSPLUS_BUGS
	ti -> attrVariableArray((const TestIntf::VariableArray_slice*)ar);
#else
	ti -> attrVariableArray(ar);
#endif
	ret = ti -> attrVariableArray();
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
	    {
		TEST(strcmp(ar[i][j], ret[i][j]) == 0);
	    }

	TestIntf::VariableArray inOut;
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		inOut[i][j] = string_dup("abc");
	TestIntf::VariableArray_var out;
#ifdef HAVE_VCPLUSPLUS_BUGS
	ret = ti -> opVariableArray((const TestIntf::VariableArray_slice*)
				    ar, inOut, out.out());
#else
	ret = ti -> opVariableArray(ar, inOut, out.out());
#endif
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
	    {
		TEST(strcmp(ar[i][j], ret[i][j]) == 0);
		TEST(strcmp(ar[i][j], inOut[i][j]) == 0);
		TEST(strcmp(ar[i][j], out[i][j]) == 0);
	    }
    }

    {
	TestIntf::FixedArray ar0 =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray ar1 =
	{
	    {
		{ 2, 3, 4, 1 },
		{ 10, 11, 11, -10 },
		{ -0, 939, 123, 888 }
	    },
	    {
		{ 17, 37, 47, 27 },
		{ 710, -710, 711, -711 },
		{ -0, -99, 13, 8338 }
	    }
	};
	
	TestIntf::FixedArray ar2 =
	{
	    {
		{ 1, 2, -3, -234 },
		{ 10, -11, 11, -10 },
		{ -999, 30, 1888, 123 }
	    },
	    {
		{ 27, 37, 117, 47 },
		{ 710, -7150, 711, -711 },
		{ -0, 13, 929, 88 }
	    }
	};
	
	TestIntf::FixedArraySequence seq;
	seq.length(3);
	TestIntf::FixedArray_copy(seq[0], ar0);
	TestIntf::FixedArray_copy(seq[1], ar1);
	TestIntf::FixedArray_copy(seq[2], ar2);

	TestIntf::FixedArraySequence_var ret;

	ti -> attrFixedArraySequence(seq);
	ret = ti -> attrFixedArraySequence();

	for(l = 0 ; l < 3 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(seq[l][i][j][k] == ret[l][i][j][k]);
		    }

	TestIntf::FixedArraySequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
			inOut[l][i][j][k] = (short)(i + j + k + l);

	TestIntf::FixedArraySequence_var out;

	ret = ti -> opFixedArraySequence(seq, inOut, out);

	TEST(ret -> length() == 7);
	TEST(out -> length() == 7);
	TEST(inOut.length() == 7);

	for(l = 0 ; l < 3 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(seq[l][i][j][k] == ret[l][i][j][k]);
			TEST(seq[l][i][j][k] == inOut[l][i][j][k]);
			TEST(seq[l][i][j][k] == out[l][i][j][k]);
		    }

	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(ret[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
			TEST(inOut[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
			TEST(out[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
		    }
    }

    {
	TestIntf::VariableArray ar0 =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};

	TestIntf::VariableArray ar1 =
	{ 
	    {
		string_dup("a~a"),
		string_dup("b~b"),
		string_dup("c~c")
	    },
	    {
		string_dup("A~A"),
		string_dup("B~B"),
		string_dup("C~C")
	    }
	};

	TestIntf::VariableArraySequence seq;
	seq.length(2);
	TestIntf::VariableArray_copy(ar0, seq[0]);
	TestIntf::VariableArray_copy(ar1, seq[1]);

	TestIntf::VariableArraySequence_var ret;

	ti -> attrVariableArraySequence(seq);
	ret = ti -> attrVariableArraySequence();

	for(l = 0 ; l < 2 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(seq[l][i][j], ret[l][i][j]) == 0);
		}

	TestIntf::VariableArraySequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    inOut[l][i][j] = string_dup("***");

	TestIntf::VariableArraySequence_var out;
	
	ret = ti -> opVariableArraySequence(seq, inOut, out);

	TEST(ret -> length() == 6);
	TEST(out -> length() == 6);
	TEST(inOut.length() == 6);

	for(l = 0 ; l < 2 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(seq[l][i][j], ret[l][i][j]) == 0);
		    TEST(strcmp(seq[l][i][j], inOut[l][i][j]) == 0);
		    TEST(strcmp(seq[l][i][j], out[l][i][j]) == 0);
		}

	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(ret[2 + l][i][j], "***") == 0);
		    TEST(strcmp(inOut[2 + l][i][j], "***") == 0);
		    TEST(strcmp(out[2 + l][i][j], "***") == 0);
		}
    }

    {
	TestIntf::FixedArray ar0 =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray ar1 =
	{
	    {
		{ 2, 3, 4, 1 },
		{ 10, 11, 11, -10 },
		{ -0, 939, 123, 888 }
	    },
	    {
		{ 17, 37, 47, 27 },
		{ 710, -710, 711, -711 },
		{ -0, -99, 13, 8338 }
	    }
	};
	
	TestIntf::FixedArray ar2 =
	{
	    {
		{ 1, 2, -3, -234 },
		{ 10, -11, 11, -10 },
		{ -999, 30, 1888, 123 }
	    },
	    {
		{ 27, 37, 117, 47 },
		{ 710, -7150, 711, -711 },
		{ -0, 13, 929, 88 }
	    }
	};
	
	TestIntf::FixedArrayBoundSequence seq;
	seq.length(3);
	TestIntf::FixedArray_copy(seq[0], ar0);
	TestIntf::FixedArray_copy(seq[1], ar1);
	TestIntf::FixedArray_copy(seq[2], ar2);

	TestIntf::FixedArrayBoundSequence_var ret;

	ti -> attrFixedArrayBoundSequence(seq);
	ret = ti -> attrFixedArrayBoundSequence();

	for(l = 0 ; l < 3 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(seq[l][i][j][k] == ret[l][i][j][k]);
		    }

	TestIntf::FixedArrayBoundSequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
			inOut[l][i][j][k] = (short)(i + j + k + l);

	TestIntf::FixedArrayBoundSequence_var out;

	ret = ti -> opFixedArrayBoundSequence(seq, inOut, out);

	TEST(ret -> length() == 7);
	TEST(out -> length() == 7);
	TEST(inOut.length() == 7);

	for(l = 0 ; l < 3 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(seq[l][i][j][k] == ret[l][i][j][k]);
			TEST(seq[l][i][j][k] == inOut[l][i][j][k]);
			TEST(seq[l][i][j][k] == out[l][i][j][k]);
		    }

	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(ret[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
			TEST(inOut[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
			TEST(out[3 + l][i][j][k] ==
			     (Long)(i + j + k + l));
		    }
    }

    {
	TestIntf::VariableArray ar0 =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};

	TestIntf::VariableArray ar1 =
	{ 
	    {
		string_dup("a~a"),
		string_dup("b~b"),
		string_dup("c~c")
	    },
	    {
		string_dup("A~A"),
		string_dup("B~B"),
		string_dup("C~C")
	    }
	};

	TestIntf::VariableArrayBoundSequence seq;
	seq.length(2);
	TestIntf::VariableArray_copy(seq[0], ar0);
	TestIntf::VariableArray_copy(seq[1], ar0);

	TestIntf::VariableArrayBoundSequence_var ret;

	ti -> attrVariableArrayBoundSequence(seq);
	ret = ti -> attrVariableArrayBoundSequence();

	for(l = 0 ; l < 2 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(seq[l][i][j], ret[l][i][j]) == 0);
		}

	TestIntf::VariableArrayBoundSequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    inOut[l][i][j] = string_dup("***");

	TestIntf::VariableArrayBoundSequence_var out;
	
	ret = ti -> opVariableArrayBoundSequence(seq, inOut, out);

	TEST(ret -> length() == 6);
	TEST(out -> length() == 6);
	TEST(inOut.length() == 6);

	for(l = 0 ; l < 2 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(seq[l][i][j], ret[l][i][j]) == 0);
		    TEST(strcmp(seq[l][i][j], inOut[l][i][j]) == 0);
		    TEST(strcmp(seq[l][i][j], out[l][i][j]) == 0);
		}

	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(ret[2 + l][i][j], "***") == 0);
		    TEST(strcmp(inOut[2 + l][i][j], "***") == 0);
		    TEST(strcmp(out[2 + l][i][j], "***") == 0);
		}
    }
}

// ----------------------------------------------------------------------
// Tests for TestIntf_2_3_NoLongDouble
// ----------------------------------------------------------------------

void
TestStub_2_3_NoLongDouble(TestIntf_2_3_NoLongDouble_ptr ti)
{
    {
	LongLong ret;
#ifndef HAVE_VCPLUSPLUS_BUGS
	ti -> attrLongLong(-9223372036854775807LL - 1);
#else
	ti -> attrLongLong(-9223372036854775807L - 1);
#endif
	ret = ti -> attrLongLong();
#ifndef HAVE_VCPLUSPLUS_BUGS
	TEST(ret == -9223372036854775807LL - 1);
#else
	TEST(ret == -9223372036854775807L - 1);
#endif

#ifndef HAVE_VCPLUSPLUS_BUGS
	ti -> attrLongLong(9223372036854775807LL);
#else
	ti -> attrLongLong(9223372036854775807L);
#endif
	ret = ti -> attrLongLong();
#ifndef HAVE_VCPLUSPLUS_BUGS
	TEST(ret == 9223372036854775807LL);
#else
	TEST(ret == 9223372036854775807L);
#endif

	LongLong inOut, out;
	inOut = 20;
	ret = ti -> opLongLong(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	ULongLong ret;
#ifndef HAVE_VCPLUSPLUS_BUGS
	ti -> attrULongLong(9223372036854775807ULL);
#else
	ti -> attrULongLong(9223372036854775807UL);
#endif
	ret = ti -> attrULongLong();
#ifndef HAVE_VCPLUSPLUS_BUGS
	TEST(ret == 9223372036854775807ULL);
#else
	TEST(ret == 9223372036854775807UL);
#endif

	ULongLong inOut, out;
	inOut = 20;
	ret = ti -> opULongLong(10, inOut, out);
	TEST(ret == 30);
	TEST(inOut == 30);
	TEST(out == 30);
    }

    {
	TestIntf_2_3_NoLongDouble::TestFixed b;
	TestIntf_2_3_NoLongDouble::TestFixed ret;

	b = TestIntf_2_3_NoLongDouble::TestFixed("0");
	ti -> attrFixed(b);
	ret = ti -> attrFixed();
	TEST(ret == b);

	b = TestIntf_2_3_NoLongDouble::TestFixed("1234567890.12345670");
	ti -> attrFixed(b);
	ret = ti -> attrFixed();
	TEST(ret == b);

	b = TestIntf_2_3_NoLongDouble::TestFixed("-9876543210.87654320");
	ti -> attrFixed(b);
	ret = ti -> attrFixed();
	TEST(ret == b);

	TestIntf_2_3_NoLongDouble::TestFixed inOut, out;
	inOut = TestIntf_2_3_NoLongDouble::TestFixed("20.00000000");
	ret =
	    ti -> opFixed(TestIntf_2_3_NoLongDouble::TestFixed("10.00000000"),
			  inOut, out);
	b = TestIntf_2_3_NoLongDouble::TestFixed("30.00000000");
	TEST(ret == b);
	TEST(inOut == b);
	TEST(out == b);
    }

    {
	WChar ret;
	ti -> attrWChar(L'a');
	ret = ti -> attrWChar();
	TEST(ret == L'a');
	ti -> attrWChar(224);
	ret = ti -> attrWChar();
	TEST(ret == 224);
	ti -> attrWChar(0x20ac);
	ret = ti -> attrWChar();
	TEST(ret == 0x20ac);

	WChar inOut, out;
	inOut = 1;
	ret = ti -> opWChar(L'a', inOut, out);
	TEST(ret == L'b');
	TEST(inOut == L'b');
	TEST(out == L'b');
    }

    {
	WString_var ret;
	WString_var ws;

	ti -> attrWString(L"");
	ret = ti -> attrWString();
	ws = wstring_dup(L"");
	TEST(OB::wcscmp(ret, ws) == 0);

	ti -> attrWString(L"Hello");
	ret = ti -> attrWString();
	ws = wstring_dup(L"Hello");
	TEST(OB::wcscmp(ret, ws) == 0);

	WString_var inOut, out;
	inOut = wstring_dup(L"world!");
	ret = ti -> opWString(L"Hello, ", inOut.inout(), out.out());
	ws = wstring_dup(L"Hello, world!");
	TEST(OB::wcscmp(ret, ws) == 0);
	TEST(OB::wcscmp(out, ws) == 0);
    }
}

// ----------------------------------------------------------------------
// Tests for TestIntf_2_3
// ----------------------------------------------------------------------

void
TestStub_2_3(TestIntf_2_3_ptr ti)
{
#ifndef WIN32
#if SIZEOF_LONG_DOUBLE >= 12
    {
	try
	{
	    LongDouble ret;
	    ti -> attrLongDouble(1.7976931348623157E+308);
	    ret = ti -> attrLongDouble();
	    TEST(ret == 1.7976931348623157E+308);

	    ti -> attrLongDouble(2.2250738585072014E-308);
	    ret = ti -> attrLongDouble();
#if !( defined(__alpha) && defined(__linux) )
	    TEST(ret == 2.2250738585072014E-308);
#endif

	    LongDouble inOut, out;
	    inOut = 20;
	    ret = ti -> opLongDouble(10, inOut, out);
	    TEST(ret == 30);
	    TEST(inOut == 30);
	    TEST(out == 30);
	}
	catch(const MARSHAL& ex)
	{
	    //
	    // Ignore exceptions caused by unsupported data types
	    //
	    if(ex.minor() != OB::MinorLongDoubleNotSupported)
		throw;
	}
    }
#endif
#endif
}
