// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestIntf.h>
#include <TestMacro.h>

using namespace CORBA;

void
TestStubContext(TestIntf_ptr ti, ORB_ptr orb)
{
    Context_var ctx;
    Context_var ctx2;

    {
	orb -> get_default_context(ctx.out());
	
	TestIntf::StringSequence_var seq;
	
	try
	{
	    seq = ti -> opContext("*", ctx);
	    TEST(false);
	}
	catch(const BAD_CONTEXT&)
	{
	    // Expected
	}
    }

    {
	Any any;
	ULong i;
	TestIntf::StringSequence_var seq;

	orb -> get_default_context(ctx.out());

	any <<= (const char*)"A1";
	ctx -> set_one_value("A", any);

	any <<= (const char*)"A2";
	ctx -> set_one_value("AAA", any);

	any <<= (const char*)"B1";
	ctx -> set_one_value("B", any);

	any <<= (const char*)"B2";
	ctx -> set_one_value("BBB", any);

	any <<= (const char*)"X1";
	ctx -> set_one_value("X", any);

	any <<= (const char*)"X2";
	ctx -> set_one_value("XXX", any);

	any <<= (const char*)"Y1";
	ctx -> set_one_value("Y", any);

	any <<= (const char*)"Y2";
	ctx -> set_one_value("YYY", any);

	seq = ti -> opContext("*", ctx);
	TEST(seq -> length() == 3 * 2);
	for(i = 0 ; i < seq -> length() ; i += 2)
        {
	    if(strcmp(seq[i], "A") == 0)
	        TEST(strcmp(seq[i + 1], "A1") == 0);

	    if(strcmp(seq[i], "AAA") == 0)
	        TEST(strcmp(seq[i + 1], "A2") == 0);

	    if(strcmp(seq[i], "X") == 0)
	        TEST(strcmp(seq[i + 1], "X1") == 0);
	}

	seq = ti -> opContext("A*", ctx);
	TEST(seq -> length() == 2 * 2);
	for(i = 0 ; i < seq -> length() ; i += 2)
        {
	    if(strcmp(seq[i], "A") == 0)
	        TEST(strcmp(seq[i + 1], "A1") == 0);

	    if(strcmp(seq[i], "AAA") == 0)
	        TEST(strcmp(seq[i + 1], "A2") == 0);
	}

	seq = ti -> opContext("AA*", ctx);
	TEST(seq -> length() == 1 * 2);
	TEST(strcmp(seq[0], "AAA") == 0 &&
	     strcmp(seq[1], "A2") == 0);

	seq = ti -> opContext("A", ctx);
	TEST(seq -> length() == 1 * 2);
	TEST(strcmp(seq[0], "A") == 0 &&
	     strcmp(seq[1], "A1") == 0);

	ctx -> create_child("child", ctx2.out());

	any <<= (const char*)"C1";
	ctx2 -> set_one_value("C", any);

	any <<= (const char*)"C2";
	ctx2 -> set_one_value("CCC", any);

	any <<= (const char*)"X1-1";
	ctx2 -> set_one_value("X", any);

	seq = ti -> opContext("*", ctx2);
	TEST(seq -> length() == 5 * 2);
	for(i = 0 ; i < seq -> length() ; i += 2)
        {
	    if(strcmp(seq[i], "A") == 0)
	        TEST(strcmp(seq[i + 1], "A1") == 0);

	    if(strcmp(seq[i], "AAA") == 0)
	        TEST(strcmp(seq[i + 1], "A2") == 0);

	    if(strcmp(seq[i], "C") == 0)
	        TEST(strcmp(seq[i + 1], "C1") == 0);

	    if(strcmp(seq[i], "CCC") == 0)
	        TEST(strcmp(seq[i + 1], "C2") == 0);

	    if(strcmp(seq[i], "X") == 0)
	        TEST(strcmp(seq[i + 1], "X1-1") == 0);
	}
    }
}
