// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OBPolicies.h>

#include <TestIntf.h>
#include <TestMacro.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;

void
TestStubTimeout(TestIntf_ptr tiOriginal, ORB_ptr orb)
{
    //
    // Create policy list containing new timeout policy
    //
    PolicyList pl(1);
    pl.length(1);
    Any any;
    any <<= (ULong)2000;
    pl[0] = orb -> create_policy(OB::TIMEOUT_POLICY_ID, any);
    
    //
    // _set_policy_overrides returns new proxy with the policies applied
    //
    Object_var obj = tiOriginal -> _set_policy_overrides(pl, ADD_OVERRIDE);

    //
    // _narrow to correct type
    //
    TestIntf_var ti = TestIntf::_narrow(obj);
    assert(!is_nil(ti));
    
    try
    {
	ti -> sleepTwoway(3);
	TEST(false);
    }
    catch(const NO_RESPONSE&)
    {
    }

#ifdef WIN32
    Sleep(2000);
#else
    sleep(2);
#endif

    //
    // The following test doesn't work, because there is a race
    // between the sleepOneway and the sleepTwoway if the server is
    // running in thread pool or thread-per-request mode. (This isn't
    // such an exiting test anyway, so perhaps we should permanently
    // remove it in a future version.)
    //
/*
    try
    {
	ti -> sleepOneway(3);
    }
    catch(const NO_RESPONSE&)
    {
	TEST(false);
    }

    try
    {
	ti -> sleepTwoway(0);
	TEST(false);
    }
    catch(const NO_RESPONSE&)
    {
    }

#ifdef WIN32
    Sleep(2000);
#else
    sleep(2);
#endif
*/

    try
    {
	ti -> sleepTwoway(0);
    }
    catch(const NO_RESPONSE&)
    {
	TEST(false);
    }
}
