// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Test_impl.h>

#include <limits.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;

TestIntf_2_3_impl::TestIntf_2_3_impl(ORB_ptr orb)
    : orb_(ORB::_duplicate(orb))
{
}

// ----------------------------------------------------------------------
// Methods from TestIntf
// ----------------------------------------------------------------------

void
TestIntf_2_3_impl::deactivate()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    orb_ -> shutdown(false);
}

ULong
TestIntf_2_3_impl::version()
    throw(SystemException)
{
    return OB_INTEGER_VERSION;
}

Boolean
TestIntf_2_3_impl::concurrentRequestExecution()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    PortableServer::POA_var poa = _default_POA();
    OBPortableServer::POA_var obpoa = OBPortableServer::POA::_narrow(poa);
    OB::DispatchStrategy_var strategy = obpoa -> the_dispatch_strategy();

    return strategy -> id() != OB::SAME_THREAD;
}

void
TestIntf_2_3_impl::sleepOneway(ULong sec)
    throw(SystemException)
{
    OB_SYNCHRONIZED

#ifdef WIN32
    Sleep(sec * 1000);
#else
    sleep(sec);
#endif
}

void
TestIntf_2_3_impl::sleepTwoway(ULong sec)
    throw(SystemException)
{
    OB_SYNCHRONIZED

#ifdef WIN32
    Sleep(sec * 1000);
#else
    sleep(sec);
#endif
}

void
TestIntf_2_3_impl::opVoid()
    throw(SystemException)
{
    OB_SYNCHRONIZED

}

TestIntf::StringSequence*
TestIntf_2_3_impl::opContext(const char* pattern, Context_ptr ctx)
    throw(SystemException)
{
#if defined(__GNUC__) && defined(__OPTIMIZE__) // COMPILERFIX
    volatile int dummy = 1;
    dummy++;
#endif

    OB_SYNCHRONIZED

    NVList_var values;
    ctx -> get_values("", 0, pattern, values.out());

    ULong len = values -> count();
    TestIntf::StringSequence* result =
        new TestIntf::StringSequence(len * 2);
    result -> length(len * 2);
    for(ULong i = 0 ; i < len ; i++)
    {
	const char* s;
	*(values -> item(i) -> value()) >>= s;
	(*result)[i * 2] = values -> item(i) -> name();
	(*result)[i * 2 + 1] = string_dup(s);
    }

    return result;
}

void
TestIntf_2_3_impl::opVoidEx()
    throw(TestIntf::ExVoid,
	  SystemException)
{
    OB_SYNCHRONIZED

    throw TestIntf::ExVoid();
}

Short
TestIntf_2_3_impl::attrShort()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aShort;
}

void
TestIntf_2_3_impl::attrShort(Short val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aShort = val;
}

Short
TestIntf_2_3_impl::opShort(Short a0,
			   Short& a1,
			   Short& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aShort = a0 + a1;
    a1 = a2 = aShort;
    return aShort;
}

Short
TestIntf_2_3_impl::opShortEx(Short a0,
			     Short& a1,
			     Short&)
    throw(TestIntf::ExShort,
	  SystemException)
{
    OB_SYNCHRONIZED

#if defined(__GNUC__) && defined(__OPTIMIZE__) // COMPILERFIX
    volatile int i = 1;
    i++;
#endif

    aShort = a0 + a1;
    throw TestIntf::ExShort(aShort);
    return 0; // Some compilers need this
}

Long
TestIntf_2_3_impl::attrLong()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aLong;
}

void
TestIntf_2_3_impl::attrLong(Long val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aLong = val;
}

Long
TestIntf_2_3_impl::opLong(Long a0,
			  Long& a1,
			  Long& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aLong = a0 + a1;
    a1 = a2 = aLong;
    return aLong;
}

Long
TestIntf_2_3_impl::opLongEx(Long a0,
			    Long& a1,
			    Long&)
    throw(TestIntf::ExLong,
	  SystemException)
{
    OB_SYNCHRONIZED

    aLong = a0 + a1;
    throw TestIntf::ExLong(aLong);
    return 0; // Some compilers need this
}

UShort
TestIntf_2_3_impl::attrUShort()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aUShort;
}

void
TestIntf_2_3_impl::attrUShort(UShort val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aUShort = val;
}

UShort
TestIntf_2_3_impl::opUShort(UShort a0,
			    UShort& a1,
			    UShort& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aUShort = a0 + a1;
    a1 = a2 = aUShort;
    return aUShort;
}

UShort
TestIntf_2_3_impl::opUShortEx(UShort a0,
			    UShort& a1,
			    UShort&)
    throw(TestIntf::ExUShort,
	  SystemException)
{
    OB_SYNCHRONIZED

    aUShort = a0 + a1;
    throw TestIntf::ExUShort(aUShort);
    return 0; // Some compilers need this
}

ULong
TestIntf_2_3_impl::attrULong()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aULong;
}

void
TestIntf_2_3_impl::attrULong(ULong val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aULong = val;
}

ULong
TestIntf_2_3_impl::opULong(ULong a0,
			   ULong& a1,
			   ULong& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aULong = a0 + a1;
    a1 = a2 = aULong;
    return aULong;
}

ULong
TestIntf_2_3_impl::opULongEx(ULong a0,
			   ULong& a1,
			   ULong&)
    throw(TestIntf::ExULong,
	  SystemException)
{
    OB_SYNCHRONIZED

    aULong = a0 + a1;
    throw TestIntf::ExULong(aULong);
    return 0; // Some compilers need this
}

Float
TestIntf_2_3_impl::attrFloat()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aFloat;
}

void
TestIntf_2_3_impl::attrFloat(Float val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFloat = val;
}

Float
TestIntf_2_3_impl::opFloat(Float a0,
			   Float& a1,
			   Float& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFloat = a0 + a1;
    a1 = a2 = aFloat;
    return aFloat;
}

Float
TestIntf_2_3_impl::opFloatEx(Float a0,
			   Float& a1,
			   Float&)
    throw(TestIntf::ExFloat,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFloat = a0 + a1;
    throw TestIntf::ExFloat(aFloat);
    return 0; // Some compilers need this
}

Double
TestIntf_2_3_impl::attrDouble()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aDouble;
}

void
TestIntf_2_3_impl::attrDouble(Double val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aDouble = val;
}

Double
TestIntf_2_3_impl::opDouble(Double a0,
			    Double& a1,
			    Double& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aDouble = a0 + a1;
    a1 = a2 = aDouble;
    return aDouble;
}

Double
TestIntf_2_3_impl::opDoubleEx(Double a0,
			    Double& a1,
			    Double&)
    throw(TestIntf::ExDouble,
	  SystemException)
{
    OB_SYNCHRONIZED

    aDouble = a0 + a1;
    throw TestIntf::ExDouble(aDouble);
    return 0; // Some compilers need this
}

Boolean
TestIntf_2_3_impl::attrBoolean()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aBoolean;
}

void
TestIntf_2_3_impl::attrBoolean(Boolean val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aBoolean = val;
}

Boolean
TestIntf_2_3_impl::opBoolean(Boolean a0,
			     Boolean& a1,
			     Boolean& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aBoolean = a0 && a1;
    a1 = a2 = aBoolean;
    return aBoolean;
}

Boolean
TestIntf_2_3_impl::opBooleanEx(Boolean a0,
			     Boolean& a1,
			     Boolean&)
    throw(TestIntf::ExBoolean,
	  SystemException)
{
    OB_SYNCHRONIZED

    aBoolean = a0 && a1;
    throw TestIntf::ExBoolean(aBoolean);
    return 0; // Some compilers need this
}

Char
TestIntf_2_3_impl::attrChar()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aChar;
}

void
TestIntf_2_3_impl::attrChar(Char val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aChar = val;
}

Char
TestIntf_2_3_impl::opChar(Char a0,
			  Char& a1,
			  Char& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aChar = a0 + a1;
    a1 = a2 = aChar;
    return aChar;
}

Char
TestIntf_2_3_impl::opCharEx(Char a0,
			  Char& a1,
			  Char&)
    throw(TestIntf::ExChar,
	  SystemException)
{
    OB_SYNCHRONIZED

    aChar = a0 + a1;
    throw TestIntf::ExChar(aChar);
    return 0; // Some compilers need this
}

Octet
TestIntf_2_3_impl::attrOctet()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aOctet;
}

void
TestIntf_2_3_impl::attrOctet(Octet val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aOctet = val;
}

Octet
TestIntf_2_3_impl::opOctet(Octet a0,
			   Octet& a1,
			   Octet& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aOctet = a0 + a1;
    a1 = a2 = aOctet;
    return aOctet;
}

Octet
TestIntf_2_3_impl::opOctetEx(Octet a0,
			   Octet& a1,
			   Octet&)
    throw(TestIntf::ExOctet,
	  SystemException)
{
    OB_SYNCHRONIZED

    aOctet = a0 + a1;
    throw TestIntf::ExOctet(aOctet);
    return 0; // Some compilers need this
}

char*
TestIntf_2_3_impl::attrString()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return string_dup(aString);
}

void
TestIntf_2_3_impl::attrString(const char* val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aString = val;
}

char*
TestIntf_2_3_impl::opString(const char* a0,
			    char*& a1,
			    String_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aString = string_alloc(strlen(a0) + strlen(a1));
    strcpy(aString.inout(), a0);
    strcat(aString.inout(), a1);
    string_free(a1);
    a1 = string_dup(aString);
    a2 = string_dup(aString);
    return string_dup(aString);
}

char*
TestIntf_2_3_impl::opStringEx(const char* a0,
			    char*& a1,
			    String_out)
    throw(TestIntf::ExString,
	  SystemException)
{
    OB_SYNCHRONIZED

    aString = string_alloc(strlen(a0) + strlen(a1));
    strcpy(aString.inout(), a0);
    strcat(aString.inout(), a1);
    throw TestIntf::ExString(aString);
    return 0; // Some compilers need this
}

Any*
TestIntf_2_3_impl::attrAny()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new Any(aAny);
}

void
TestIntf_2_3_impl::attrAny(const Any& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aAny = val;
}

Any*
TestIntf_2_3_impl::opAny(const Any& a0,
			 Any& a1,
			 Any_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aAny = a0;
    a1 = aAny;
    a2 = new Any(aAny);
    return new Any(aAny);
}

Any*
TestIntf_2_3_impl::opAnyEx(const Any& a0,
			 Any&,
			 Any_out)
    throw(TestIntf::ExAny,
	  SystemException)
{
    OB_SYNCHRONIZED

    aAny = a0;
    throw TestIntf::ExAny(aAny);
    return 0; // Some compilers need this
}

TestIntf::TestEnum
TestIntf_2_3_impl::attrTestEnum()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aTestEnum;
}

void
TestIntf_2_3_impl::attrTestEnum(TestIntf::TestEnum val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aTestEnum = val;
}

TestIntf::TestEnum
TestIntf_2_3_impl::opTestEnum(TestIntf::TestEnum a0,
			      TestIntf::TestEnum& a1,
			      TestIntf::TestEnum& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aTestEnum = a0;
    a1 = a2 = aTestEnum;
    return aTestEnum;
}

TestIntf::TestEnum
TestIntf_2_3_impl::opTestEnumEx(TestIntf::TestEnum a0,
			      TestIntf::TestEnum&,
			      TestIntf::TestEnum&)
    throw(TestIntf::ExTestEnum,
	  SystemException)
{
    OB_SYNCHRONIZED

    aTestEnum = a0;
    throw TestIntf::ExTestEnum(aTestEnum);
    return TestIntf::TestEnum1; // Some compilers need this
}

TestIntf_ptr
TestIntf_2_3_impl::attrTestIntf()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return TestIntf::_duplicate(aTestIntf);
}

void
TestIntf_2_3_impl::attrTestIntf(TestIntf_ptr val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aTestIntf = TestIntf::_duplicate(val);
}

TestIntf_ptr
TestIntf_2_3_impl::opTestIntf(TestIntf_ptr a0,
			      TestIntf_ptr& a1,
			      TestIntf_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aTestIntf = TestIntf::_duplicate(a0);
    release(a1);
    a1 = TestIntf::_duplicate(aTestIntf);
    a2 = TestIntf::_duplicate(aTestIntf);
    return TestIntf::_duplicate(aTestIntf);
}

TestIntf_ptr
TestIntf_2_3_impl::opTestIntfEx(TestIntf_ptr a0,
				TestIntf_ptr&,
				TestIntf_out)
    throw(TestIntf::ExTestIntf,
	  SystemException)
{
    OB_SYNCHRONIZED

    aTestIntf = TestIntf::_duplicate(a0);
    throw TestIntf::ExTestIntf(aTestIntf);
    return TestIntf::_nil(); // Some compilers need this
}

TestIntf::FixedStruct
TestIntf_2_3_impl::attrFixedStruct()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aFixedStruct;
}

void
TestIntf_2_3_impl::attrFixedStruct(const TestIntf::FixedStruct& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedStruct = val;
}

TestIntf::FixedStruct
TestIntf_2_3_impl::opFixedStruct(const TestIntf::FixedStruct& a0,
				 TestIntf::FixedStruct& a1,
				 TestIntf::FixedStruct& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedStruct = a0;
    a1 = a2 = aFixedStruct;
    return aFixedStruct;
}

TestIntf::FixedStruct
TestIntf_2_3_impl::opFixedStructEx(const TestIntf::FixedStruct& a0,
				 TestIntf::FixedStruct&,
				 TestIntf::FixedStruct&)
    throw(TestIntf::ExFixedStruct,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFixedStruct = a0;
    throw TestIntf::ExFixedStruct(aFixedStruct);
    return TestIntf::FixedStruct(); // Some compilers need this
}

TestIntf::VariableStruct*
TestIntf_2_3_impl::attrVariableStruct()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::VariableStruct(aVariableStruct);
}

void
TestIntf_2_3_impl::attrVariableStruct(const TestIntf::VariableStruct&
				       val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableStruct = val;
}

TestIntf::VariableStruct*
TestIntf_2_3_impl::opVariableStruct(const TestIntf::VariableStruct& a0,
				    TestIntf::VariableStruct& a1,
				    TestIntf::VariableStruct_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableStruct = a0;
    a1 = aVariableStruct;
    a2 = new TestIntf::VariableStruct(aVariableStruct);
    return new TestIntf::VariableStruct(aVariableStruct);
}

TestIntf::VariableStruct*
TestIntf_2_3_impl::opVariableStructEx(const TestIntf::VariableStruct&
				    a0,
				    TestIntf::VariableStruct&,
				    TestIntf::VariableStruct_out)
    throw(TestIntf::ExVariableStruct,
	  SystemException)
{
    OB_SYNCHRONIZED

    aVariableStruct = a0;
    throw TestIntf::ExVariableStruct(aVariableStruct);
    return new TestIntf::VariableStruct; // Some compilers need this
}

TestIntf::FixedUnion
TestIntf_2_3_impl::attrFixedUnion()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aFixedUnion;
}

void
TestIntf_2_3_impl::attrFixedUnion(const TestIntf::FixedUnion& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedUnion = val;
}

TestIntf::FixedUnion
TestIntf_2_3_impl::opFixedUnion(const TestIntf::FixedUnion& a0,
				TestIntf::FixedUnion& a1,
				TestIntf::FixedUnion& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedUnion = a0;
    a1 = a2 = aFixedUnion;
    return aFixedUnion;
}

TestIntf::FixedUnion
TestIntf_2_3_impl::opFixedUnionEx(const TestIntf::FixedUnion& a0,
				TestIntf::FixedUnion&,
				TestIntf::FixedUnion&)
    throw(TestIntf::ExFixedUnion,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFixedUnion = a0;
    throw TestIntf::ExFixedUnion(aFixedUnion);
    return TestIntf::FixedUnion(); // Some compilers need this
}

TestIntf::VariableUnion*
TestIntf_2_3_impl::attrVariableUnion()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::VariableUnion(aVariableUnion);
}

void
TestIntf_2_3_impl::attrVariableUnion(const TestIntf::VariableUnion& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableUnion = val;
}

TestIntf::VariableUnion*
TestIntf_2_3_impl::opVariableUnion(const TestIntf::VariableUnion& a0,
				   TestIntf::VariableUnion& a1,
				   TestIntf::VariableUnion_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableUnion = a0;
    a1 = aVariableUnion;
    a2 = new TestIntf::VariableUnion(aVariableUnion);
    return new TestIntf::VariableUnion(aVariableUnion);
}

TestIntf::VariableUnion*
TestIntf_2_3_impl::opVariableUnionEx(const TestIntf::VariableUnion& a0,
				   TestIntf::VariableUnion&,
				   TestIntf::VariableUnion_out)
    throw(TestIntf::ExVariableUnion,
	  SystemException)
{
    OB_SYNCHRONIZED

    aVariableUnion = a0;
    throw TestIntf::ExVariableUnion(aVariableUnion);
    return new TestIntf::VariableUnion; // Some compilers need this
}

TestIntf::StringSequence*
TestIntf_2_3_impl::attrStringSequence()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::StringSequence(aStringSequence);
}

void
TestIntf_2_3_impl::attrStringSequence(const TestIntf::StringSequence& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aStringSequence = val;
}

TestIntf::StringSequence*
TestIntf_2_3_impl::opStringSequence(const TestIntf::StringSequence& a0,
				    TestIntf::StringSequence& a1,
				    TestIntf::StringSequence_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aStringSequence = a0;
    aStringSequence.length(a0.length() + a1.length());
    for(ULong i = 0 ; i < a1.length() ; i++)
	aStringSequence[a0.length() + i] = a1[i];

    a1 = aStringSequence;
    a2 = new TestIntf::StringSequence(aStringSequence);
    return new TestIntf::StringSequence(aStringSequence);
}

TestIntf::StringSequence*
TestIntf_2_3_impl::
opStringSequenceEx(const TestIntf::StringSequence& a0,
		   TestIntf::StringSequence& a1,
		   TestIntf::StringSequence_out)
    throw(TestIntf::ExStringSequence,
	  SystemException)
{
    OB_SYNCHRONIZED

    aStringSequence = a0;
    aStringSequence.length(a0.length() + a1.length());
    for(ULong i = 0 ; i < a1.length() ; i++)
	aStringSequence[a0.length() + i] = a1[i];

    throw TestIntf::ExStringSequence(aStringSequence);
    return new TestIntf::StringSequence; // Some compilers need this
}

TestIntf::FixedArray_slice*
TestIntf_2_3_impl::attrFixedArray()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return TestIntf::FixedArray_dup(aFixedArray);
}

void
TestIntf_2_3_impl::attrFixedArray(const TestIntf::FixedArray val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::FixedArray_copy(aFixedArray, val);
}

TestIntf::FixedArray_slice*
TestIntf_2_3_impl::opFixedArray(const TestIntf::FixedArray a0,
				TestIntf::FixedArray a1,
				TestIntf::FixedArray a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::FixedArray_copy(aFixedArray, a0);
    TestIntf::FixedArray_copy(a1, aFixedArray);
    TestIntf::FixedArray_copy(a2, aFixedArray);
    return TestIntf::FixedArray_dup(aFixedArray);
}

TestIntf::FixedArray_slice*
TestIntf_2_3_impl::opFixedArrayEx(const TestIntf::FixedArray a0,
				TestIntf::FixedArray,
				TestIntf::FixedArray)
    throw(TestIntf::ExFixedArray,
	  SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::FixedArray_copy(aFixedArray, a0);
#ifdef HAVE_VCPLUSPLUS_BUGS
    throw TestIntf::ExFixedArray(
	(const TestIntf::FixedArray_slice*)aFixedArray);
#else
    throw TestIntf::ExFixedArray(aFixedArray);
#endif
    return TestIntf::FixedArray_alloc(); // Some compilers need this
}

TestIntf::VariableArray_slice*
TestIntf_2_3_impl::attrVariableArray()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return TestIntf::VariableArray_dup(aVariableArray);
}

void
TestIntf_2_3_impl::attrVariableArray(const TestIntf::VariableArray val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::VariableArray_copy(aVariableArray, val);
}

TestIntf::VariableArray_slice*
TestIntf_2_3_impl::opVariableArray(const TestIntf::VariableArray a0,
				   TestIntf::VariableArray a1,
				   TestIntf::VariableArray_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::VariableArray_copy(aVariableArray, a0);
    TestIntf::VariableArray_copy(a1, aVariableArray);
    a2 = TestIntf::VariableArray_dup(aVariableArray);
    return TestIntf::VariableArray_dup(aVariableArray);
}

TestIntf::VariableArray_slice*
TestIntf_2_3_impl::opVariableArrayEx(const TestIntf::VariableArray a0,
				     TestIntf::VariableArray,
				     TestIntf::VariableArray_out)
    throw(TestIntf::ExVariableArray,
	  SystemException)
{
    OB_SYNCHRONIZED

    TestIntf::VariableArray_copy(aVariableArray, a0);
#ifdef HAVE_VCPLUSPLUS_BUGS
    throw TestIntf::ExVariableArray(
	(const TestIntf::VariableArray_slice*)aVariableArray);
#else
    throw TestIntf::ExVariableArray(aVariableArray);
#endif
    return TestIntf::VariableArray_alloc(); // Some compilers need this
}

TestIntf::FixedArraySequence*
TestIntf_2_3_impl::attrFixedArraySequence()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::FixedArraySequence(aFixedArraySequence);
}

void
TestIntf_2_3_impl::
attrFixedArraySequence(const TestIntf::FixedArraySequence& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedArraySequence = val;
}

TestIntf::FixedArraySequence*
TestIntf_2_3_impl::
opFixedArraySequence(const TestIntf::FixedArraySequence& a0,
		     TestIntf::FixedArraySequence& a1,
		     TestIntf::FixedArraySequence_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedArraySequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArraySequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArraySequence[n++], a1[i]);

    a1 = aFixedArraySequence;
    a2 = new TestIntf::FixedArraySequence(aFixedArraySequence);
    return new TestIntf::FixedArraySequence(aFixedArraySequence);
}

TestIntf::FixedArraySequence*
TestIntf_2_3_impl::
opFixedArraySequenceEx(const TestIntf::FixedArraySequence& a0,
		       TestIntf::FixedArraySequence& a1,
		       TestIntf::FixedArraySequence_out)
    throw(TestIntf::ExFixedArraySequence,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFixedArraySequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArraySequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArraySequence[n++], a1[i]);

    throw TestIntf::ExFixedArraySequence(aFixedArraySequence);
    // Some compilers need this
    return new TestIntf::FixedArraySequence(aFixedArraySequence);
}

TestIntf::VariableArraySequence*
TestIntf_2_3_impl::attrVariableArraySequence()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::VariableArraySequence(aVariableArraySequence);
}

void
TestIntf_2_3_impl::
attrVariableArraySequence(const TestIntf::VariableArraySequence& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableArraySequence = val;
}

TestIntf::VariableArraySequence*
TestIntf_2_3_impl::
opVariableArraySequence(const TestIntf::VariableArraySequence& a0,
			TestIntf::VariableArraySequence& a1,
			TestIntf::VariableArraySequence_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableArraySequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArraySequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArraySequence[n++], a1[i]);

    a1 = aVariableArraySequence;
    a2 = new TestIntf::VariableArraySequence(aVariableArraySequence);
    return new TestIntf::VariableArraySequence(aVariableArraySequence);
}

TestIntf::VariableArraySequence*
TestIntf_2_3_impl::
opVariableArraySequenceEx(const TestIntf::VariableArraySequence& a0,
			  TestIntf::VariableArraySequence& a1,
			  TestIntf::VariableArraySequence_out)
    throw(TestIntf::ExVariableArraySequence,
	  SystemException)
{
    OB_SYNCHRONIZED

    aVariableArraySequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArraySequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArraySequence[n++], a1[i]);

    throw TestIntf::ExVariableArraySequence(aVariableArraySequence);
    // Some compilers need this
    return new TestIntf::VariableArraySequence(aVariableArraySequence);
}

TestIntf::FixedArrayBoundSequence*
TestIntf_2_3_impl::attrFixedArrayBoundSequence()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return new TestIntf::FixedArrayBoundSequence(aFixedArrayBoundSequence);
}

void
TestIntf_2_3_impl::attrFixedArrayBoundSequence(
    const TestIntf::FixedArrayBoundSequence& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedArrayBoundSequence = val;
}

TestIntf::FixedArrayBoundSequence*
TestIntf_2_3_impl::opFixedArrayBoundSequence(
    const TestIntf::FixedArrayBoundSequence& a0,
    TestIntf::FixedArrayBoundSequence& a1,
    TestIntf::FixedArrayBoundSequence_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixedArrayBoundSequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArrayBoundSequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArrayBoundSequence[n++], a1[i]);

    a1 = aFixedArrayBoundSequence;
    a2 = new TestIntf::FixedArrayBoundSequence(aFixedArrayBoundSequence);
    return new TestIntf::FixedArrayBoundSequence(aFixedArrayBoundSequence);
}

TestIntf::FixedArrayBoundSequence*
TestIntf_2_3_impl::opFixedArrayBoundSequenceEx(
    const TestIntf::FixedArrayBoundSequence& a0,
    TestIntf::FixedArrayBoundSequence& a1,
    TestIntf::FixedArrayBoundSequence_out a2)
    throw(TestIntf::ExFixedArrayBoundSequence,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFixedArrayBoundSequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArrayBoundSequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::FixedArray_copy(aFixedArrayBoundSequence[n++], a1[i]);

    a1 = aFixedArrayBoundSequence;
    a2 = new TestIntf::FixedArrayBoundSequence(aFixedArrayBoundSequence);

    throw TestIntf::ExFixedArrayBoundSequence(aFixedArrayBoundSequence);
    // Some compilers need this
    return new TestIntf::FixedArrayBoundSequence(aFixedArrayBoundSequence);
}

TestIntf::VariableArrayBoundSequence*
TestIntf_2_3_impl::attrVariableArrayBoundSequence()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return
	new TestIntf::VariableArrayBoundSequence(aVariableArrayBoundSequence);
}

void
TestIntf_2_3_impl::
attrVariableArrayBoundSequence(
    const TestIntf::VariableArrayBoundSequence& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableArrayBoundSequence = val;
}

TestIntf::VariableArrayBoundSequence*
TestIntf_2_3_impl::opVariableArrayBoundSequence(
    const TestIntf::VariableArrayBoundSequence& a0,
    TestIntf::VariableArrayBoundSequence& a1,
    TestIntf::VariableArrayBoundSequence_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aVariableArrayBoundSequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArrayBoundSequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArrayBoundSequence[n++], a1[i]);

    a1 = aVariableArrayBoundSequence;
    a2 = new TestIntf::VariableArrayBoundSequence(aVariableArrayBoundSequence);
    return
	new TestIntf::VariableArrayBoundSequence(aVariableArrayBoundSequence);
}

TestIntf::VariableArrayBoundSequence*
TestIntf_2_3_impl::opVariableArrayBoundSequenceEx(
    const TestIntf::VariableArrayBoundSequence& a0,
    TestIntf::VariableArrayBoundSequence& a1,
    TestIntf::VariableArrayBoundSequence_out a2)
    throw(TestIntf::ExVariableArrayBoundSequence,
	  SystemException)
{
    OB_SYNCHRONIZED

    aVariableArrayBoundSequence.length(a0.length() + a1.length());
    
    ULong i;
    ULong n = 0;

    for(i = 0 ; i < a0.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArrayBoundSequence[n++], a0[i]);
	
    for(i = 0 ; i < a1.length() ; i++)
	TestIntf::VariableArray_copy(aVariableArrayBoundSequence[n++], a1[i]);

    a1 = aVariableArrayBoundSequence;
    a2 = new TestIntf::VariableArrayBoundSequence(aVariableArrayBoundSequence);
    throw TestIntf::ExVariableArrayBoundSequence(aVariableArrayBoundSequence);
    // Some compilers need this
    return
	new TestIntf::VariableArrayBoundSequence(aVariableArrayBoundSequence);
}

void
TestIntf_2_3_impl::op_UNKNOWN_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw UNKNOWN(1, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_BAD_PARAM_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_PARAM(2, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_NO_MEMORY_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw NO_MEMORY(3, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_IMP_LIMIT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw IMP_LIMIT(4, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_COMM_FAILURE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw COMM_FAILURE(5, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_INV_OBJREF_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INV_OBJREF(6, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_NO_PERMISSION_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw NO_PERMISSION(7, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_INTERNAL_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INTERNAL(8, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_MARSHAL_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw MARSHAL(9, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_INITIALIZE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INITIALIZE(10, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_NO_IMPLEMENT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw NO_IMPLEMENT(11, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_BAD_TYPECODE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_TYPECODE(12, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_BAD_OPERATION_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_OPERATION(13, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_NO_RESOURCES_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw NO_RESOURCES(14, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_NO_RESPONSE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw NO_RESPONSE(15, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_PERSIST_STORE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw PERSIST_STORE(16, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_BAD_INV_ORDER_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_INV_ORDER(17, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TRANSIENT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TRANSIENT(18, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_FREE_MEM_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw FREE_MEM(19, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_INV_IDENT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INV_IDENT(20, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_INV_FLAG_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INV_FLAG(21, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_INTF_REPOS_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INTF_REPOS(22, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_BAD_CONTEXT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_CONTEXT(23, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_OBJ_ADAPTER_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw OBJ_ADAPTER(24, COMPLETED_MAYBE);
}

void
TestIntf_2_3_impl::op_DATA_CONVERSION_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw DATA_CONVERSION(25, COMPLETED_YES);
}

void
TestIntf_2_3_impl::op_OBJECT_NOT_EXIST_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw OBJECT_NOT_EXIST(26, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TRANSACTION_REQUIRED_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TRANSACTION_REQUIRED(27, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TRANSACTION_ROLLEDBACK_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TRANSACTION_ROLLEDBACK(28, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_INVALID_TRANSACTION_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INVALID_TRANSACTION(29, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_INV_POLICY_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw INV_POLICY(30, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_CODESET_INCOMPATIBLE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw CODESET_INCOMPATIBLE(31, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_REBIND_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw REBIND(32, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TIMEOUT_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TIMEOUT(33, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TRANSACTION_UNAVAILABLE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TRANSACTION_UNAVAILABLE(34, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_TRANSACTION_MODE_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw TRANSACTION_MODE(35, COMPLETED_NO);
}

void
TestIntf_2_3_impl::op_BAD_QOS_Ex()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    throw BAD_QOS(36, COMPLETED_NO);
}


// ----------------------------------------------------------------------
// Methods from TestIntf_2_3_NoLongDouble
// ----------------------------------------------------------------------

LongLong
TestIntf_2_3_impl::attrLongLong()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aLongLong;
}

void
TestIntf_2_3_impl::attrLongLong(LongLong val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aLongLong = val;
}

LongLong
TestIntf_2_3_impl::opLongLong(LongLong a0,
			      LongLong& a1,
			      LongLong& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aLongLong = a0 + a1;
    a1 = a2 = aLongLong;
    return aLongLong;
}

LongLong
TestIntf_2_3_impl::opLongLongEx(LongLong a0,
				LongLong& a1,
				LongLong&)
    throw(TestIntf_2_3_NoLongDouble::ExLongLong,
	  SystemException)
{
    OB_SYNCHRONIZED

    aLongLong = a0 + a1;
    throw TestIntf_2_3_NoLongDouble::ExLongLong(aLongLong);
    return 0; // Some compilers need this
}

ULongLong
TestIntf_2_3_impl::attrULongLong()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aULongLong;
}

void
TestIntf_2_3_impl::attrULongLong(ULongLong val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aULongLong = val;
}

ULongLong
TestIntf_2_3_impl::opULongLong(ULongLong a0,
			       ULongLong& a1,
			       ULongLong& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aULongLong = a0 + a1;
    a1 = a2 = aULongLong;
    return aULongLong;
}

ULongLong
TestIntf_2_3_impl::opULongLongEx(ULongLong a0,
				 ULongLong& a1,
				 ULongLong&)
    throw(TestIntf_2_3_NoLongDouble::ExULongLong,
	  SystemException)
{
    OB_SYNCHRONIZED

    aULongLong = a0 + a1;
    throw TestIntf_2_3_NoLongDouble::ExULongLong(aULongLong);
    return 0; // Some compilers need this
}

TestIntf_2_3_NoLongDouble::TestFixed
TestIntf_2_3_impl::attrFixed()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aFixed;
}

void
TestIntf_2_3_impl::attrFixed(const TestIntf_2_3_NoLongDouble::TestFixed& val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixed = val;
}

TestIntf_2_3_NoLongDouble::TestFixed
TestIntf_2_3_impl::opFixed(const TestIntf_2_3_NoLongDouble::TestFixed& a0,
			   TestIntf_2_3_NoLongDouble::TestFixed& a1,
			   TestIntf_2_3_NoLongDouble::TestFixed_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aFixed = a0 + a1;
    a1 = a2 = aFixed;
    return aFixed;
}

TestIntf_2_3_NoLongDouble::TestFixed
TestIntf_2_3_impl::opFixedEx(const TestIntf_2_3_NoLongDouble::TestFixed& a0,
			     TestIntf_2_3_NoLongDouble::TestFixed& a1,
			     TestIntf_2_3_NoLongDouble::TestFixed_out)
    throw(TestIntf_2_3_NoLongDouble::ExFixed,
	  SystemException)
{
    OB_SYNCHRONIZED

    aFixed = a0 + a1;
    throw TestIntf_2_3_NoLongDouble::ExFixed(aDouble);
    return (Long)0; // Some compilers need this
}

WChar
TestIntf_2_3_impl::attrWChar()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aWChar;
}

void
TestIntf_2_3_impl::attrWChar(WChar val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aWChar = val;
}

WChar
TestIntf_2_3_impl::opWChar(WChar a0,
			   WChar& a1,
			   WChar& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aWChar = a0 + a1;
    a1 = a2 = aWChar;
    return aWChar;
}

WChar
TestIntf_2_3_impl::opWCharEx(WChar a0,
			     WChar& a1,
			     WChar&)
    throw(TestIntf_2_3_NoLongDouble::ExWChar,
	  SystemException)
{
    OB_SYNCHRONIZED

    aWChar = a0 + a1;
    throw TestIntf_2_3_NoLongDouble::ExWChar(aWChar);
    return 0; // Some compilers need this
}

wchar_t*
TestIntf_2_3_impl::attrWString()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return wstring_dup(aWString);
}

void
TestIntf_2_3_impl::attrWString(const wchar_t* val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aWString = val;
}

wchar_t*
TestIntf_2_3_impl::opWString(const wchar_t* a0,
			     wchar_t*& a1,
			     WString_out a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aWString = wstring_alloc(OB::wcslen(a0) + OB::wcslen(a1));
    OB::wcscpy(aWString.inout(), a0);
    OB::wcscat(aWString.inout(), a1);
    wstring_free(a1);
    a1 = wstring_dup(aWString);
    a2 = wstring_dup(aWString);
    return wstring_dup(aWString);
}

wchar_t*
TestIntf_2_3_impl::opWStringEx(const wchar_t* a0,
			     wchar_t*& a1,
			     WString_out)
    throw(TestIntf_2_3_NoLongDouble::ExWString,
	  SystemException)
{
    OB_SYNCHRONIZED

    aWString = wstring_alloc(OB::wcslen(a0) + OB::wcslen(a1));
    OB::wcscpy(aWString.inout(), a0);
    OB::wcscat(aWString.inout(), a1);
    throw TestIntf_2_3_NoLongDouble::ExWString(aWString);
    return 0; // Some compilers need this
}

// ----------------------------------------------------------------------
// Methods from TestIntf_2_3
// ----------------------------------------------------------------------

LongDouble
TestIntf_2_3_impl::attrLongDouble()
    throw(SystemException)
{
    OB_SYNCHRONIZED

    return aLongDouble;
}

void
TestIntf_2_3_impl::attrLongDouble(LongDouble val)
    throw(SystemException)
{
    OB_SYNCHRONIZED

    aLongDouble = val;
}

LongDouble
TestIntf_2_3_impl::opLongDouble(LongDouble a0,
				LongDouble& a1,
				LongDouble& a2)
    throw(SystemException)
{
    OB_SYNCHRONIZED

#if SIZEOF_LONG_DOUBLE >= 12
    aLongDouble = a0 + a1;
    a1 = a2 = aLongDouble;
#endif
    return aLongDouble;
}

LongDouble
TestIntf_2_3_impl::opLongDoubleEx(LongDouble a0,
				  LongDouble& a1,
				  LongDouble&)
    throw(TestIntf_2_3::ExLongDouble,
	  SystemException)
{
    OB_SYNCHRONIZED

#if SIZEOF_LONG_DOUBLE >= 12
    aLongDouble = a0 + a1;
    throw TestIntf_2_3::ExLongDouble(aLongDouble);
    return 0; // Some compilers need this
#else
    return aLongDouble; // Some compilers need this
#endif
}
