// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OBPolicies.h>

#include <TestMacro.h>
#include <Multi.h>

#include <stdlib.h>

using namespace CORBA;

int
run(ORB_ptr orb, int argc, char* argv[])
{
    try
    {
/*
	//
	// Set the RetryPolicy on the ORB
	//
	Any a;
	a <<= OB::RETRY_ALWAYS;
	PolicyList policies;
	policies.length(1);
	policies[0] = orb -> create_policy(OB::RETRY_POLICY_ID, a);
	Object_var pmObj = orb ->
	    resolve_initial_references("ORBPolicyManager");
	PolicyManager_var pm = PolicyManager::_narrow(pmObj);
	pm -> add_policy_overrides(policies);	
*/

	//
	// Get "multi" object
	//
	Object_var obj = orb -> string_to_object("relfile:/Multi.ref");
	if(is_nil(obj))
	{
	    OB_CERR << argv[0] << ": cannot read IOR from Multi.ref"
		    << OB_ENDL;
	    return EXIT_FAILURE;
	}
	
	Multi_var multi = Multi::_narrow(obj);
	assert(!is_nil(multi));

	//
	// 5 servers are running. When the 6th is deactivated, I
	// should get an exception.
	//
	ULong oldpid = 0;
  	for(ULong i = 0 ; i < 5 ; i++)
  	{
	    OB_COUT << "Getting process ID of server #" << i + 1 << "... "
		    << OB_FLUSH;

	    while(true)
	    {
		ULong pid = multi -> getpid();
		if(oldpid != pid)
		{
		    oldpid = pid;
		    break;
		}
	    }

	    OB_COUT << "OK!" << OB_ENDL;
	    
	    OB_COUT << "Deactivating server... " << OB_FLUSH;
	    multi -> deactivate();
	    OB_COUT << "OK!" << OB_ENDL;
  	}

	OB_COUT << "Checking whether I now get an exception... " << OB_FLUSH;
	try
	{
	    while(true)
	    {
		ULong pid = multi -> getpid();
		TEST(pid == oldpid);
	    }
	}
	catch(const TRANSIENT& ex)
	{
	    OB_COUT << "OK!" << OB_ENDL;
	    OB_COUT << "Must be COMPLETED_NO... " << OB_FLUSH;
	    TEST(ex.completed() == COMPLETED_NO);
	    OB_COUT << "OK!" << OB_ENDL;
	}
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}


int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
        OB_CERR << ex << OB_ENDL;
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
