// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <Test.h>

#include <stdlib.h>

using namespace CORBA;

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
        
        Object_var obj = orb -> string_to_object("relfile:/Test.ref");
        if(CORBA::is_nil(obj))
        {
            OB_CERR << argv[0] << ": cannot read IOR from Test.ref"
			 << OB_ENDL;
            return EXIT_FAILURE;
        }
        TestServer_var server = TestServer::_narrow(obj);
        TEST(!is_nil(server));

        TestInfoSeq_var info = server -> get_info();

        for(ULong i = 0 ; i < info -> length() ; i++)
        {
            try
            {
                info[i].obj -> aMethod();
                TEST(strlen(info[i].except_id) == 0);
            }
            catch(const SystemException& ex)
            {
                if(strcmp(info[i].except_id, ex._rep_id()) != 0)
                {
                    OB_CERR << "TestClient: Invocation on object #" << i
				 << " raised an unexpected exception"
				 << OB_ENDL;
                    OB_CERR << "Expected " << info[i].except_id
				 << " but caught "
				 << ex._rep_id() << OB_ENDL;
                    TEST(false);
                }
            }
        }

	server -> deactivate();
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
