// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <TestUtil.h>

#include <TestLocationForwardActivator_skel.h>
#include <TestLocationForwardActivator_impl.h>
#include <Test_impl.h>
#include <TestLocationForward_impl.h>

#include <stdlib.h>

using namespace CORBA;
using namespace PortableServer;

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);

        POA_var root = TestUtil_GetRootPOA(orb);

        POA_var poa;
        PolicyList policies;

        POAManager_var manager = root -> the_POAManager();
        
        manager -> activate();

        //
        // Create POA
        //
        policies.length(4);
        policies[0] =
            root -> create_id_assignment_policy(PortableServer::USER_ID);
        policies[1] =
            root -> create_id_uniqueness_policy(PortableServer::UNIQUE_ID);
        policies[2] =
            root -> create_servant_retention_policy(PortableServer::RETAIN);
        policies[3] = root ->
         create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        poa = root -> create_POA("poa", manager, policies);

        TestLocationForwardActivator_impl* activatorImpl = 
	    new TestLocationForwardActivator_impl();
	PortableServer::ServantBase_var activatorServant = activatorImpl;
        TestLocationForwardActivator_var activator = activatorImpl -> _this();
        poa -> set_servant_manager(activator);

        ObjectId_var oid = string_to_ObjectId("test");
        Object_var reference =
            poa -> create_reference_with_id(oid, "IDL:Test:1.0");

        //
        // Read all object references from file
        //
        Object_var obj = orb -> string_to_object("relfile:///Test.ref");
        if(CORBA::is_nil(obj))
        {
            OB_CERR << argv[0] << ": cannot read IOR from Test.ref"
                 << OB_ENDL;
            return EXIT_FAILURE;
        }
        TestLocationForwardServer_var server =
            TestLocationForwardServer::_narrow(obj);
        TEST(!is_nil(server));
        Object_var servant = server -> get_servant();

        activator -> setForwardRequest(servant);

        TestLocationForward_impl testImpl(orb, root);
        activatorImpl -> setActivatedServant(&testImpl);

        manager -> activate();

        TestLocationForwardActivator_var remoteActivator =
            server -> get_activator();
        TEST(!is_nil(remoteActivator));
        remoteActivator -> setForwardRequest(reference);

        //
        // Run some calls
        //
        TestLocationForward_var local =
            TestLocationForward::_narrow(reference);

        //
        // First should be local
        //
        local -> aMethod();
        local -> deactivate_servant();

        //
        // Second, should be remote
        //
        local -> aMethod();
        local -> deactivate_servant();

        //
        // Third should be local again
        //
        local -> aMethod();
        local -> deactivate_servant();

        //
        // Clean up
        //
        poa -> destroy(true, true);

	server -> deactivate();
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
