// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <Test.h>

#include <stdlib.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;
    try
    {
	//
	// Create ORB
	//
	orb = CORBA::ORB_init(argc, argv);
	CORBA::Object_var obj;

	//
	// Get test servers object references
	//
	OB_IFSTREAM in("Test.ref");
	if(in.fail())
	{
	    OB_CERR << argv[0] << ": cannot read Test.ref" << OB_ENDL;
	}

	char s[1000];
	in >> s;
	obj = orb -> string_to_object(s);
	if(CORBA::is_nil(obj))
	{
	    OB_CERR << argv[0] << ": cannot read IOR from Test.ref" << OB_ENDL;
	    return EXIT_FAILURE;
	}
	TestServer_var server1 = TestServer::_narrow(obj);
	assert(!CORBA::is_nil(server1));

	in >> s;
	obj = orb -> string_to_object(s);
	if(CORBA::is_nil(obj))
	{
	    OB_CERR << argv[0] << ": cannot read IOR from Test2.ref"
		    << OB_ENDL;
	    return EXIT_FAILURE;
	}
	TestServer_var server2 = TestServer::_narrow(obj);
	assert(!CORBA::is_nil(server2));

        in.close();

	//
	// Get object references from servers
	//
	TestInfoSeq_var info1 = server1 -> get_info();
	TestInfoSeq_var info2 = server2 -> get_info();

	//
	// Call a few methods on each object
	//
	info1[0].obj -> aMethod();
	info2[0].obj -> aMethod();
	info1[0].obj -> aMethod();
	info2[0].obj -> aMethod();

	//
	// Deactivate servers (and thus orbs)
	//
	server1 -> deactivate();
	server2 -> deactivate();
    }
    catch(const CORBA::Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
