// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <POAManagerProxy.h>

#include <Test.h>

#include <stdlib.h>

using namespace CORBA;

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
        
        Object_var obj = orb -> string_to_object("relfile:/Test.ref");
        if(CORBA::is_nil(obj))
        {
            OB_CERR << argv[0] << ": cannot read IOR from Test.ref"
			 << OB_ENDL;
            return EXIT_FAILURE;
        }
        TestServer_var server = TestServer::_narrow(obj);
        TEST(!is_nil(server));

        obj = orb -> string_to_object("relfile:/POAManagerProxy.ref");
        if(CORBA::is_nil(obj))
        {
            OB_CERR << argv[0]
			 << ": cannot read IOR from POAManagerProxy.ref"
			 << OB_ENDL;
            return EXIT_FAILURE;
        }
        POAManagerProxy_var manager = POAManagerProxy::_narrow(obj);
        TEST(!is_nil(server));

        try
        {
            manager -> activate();
        }
        catch(const POAManagerProxy::AdapterInactive&)
        {
            TEST(false);
        }

        TestInfoSeq_var info = server -> get_info();

        void TestPOAManagerCommon(POAManagerProxy_ptr, const TestInfoSeq&);
        TestPOAManagerCommon(manager, info);

	server -> deactivate();
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
