// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <POAManagerProxy.h>

#include <Test.h>

#include <stdlib.h>

using namespace CORBA;

#ifdef HAVE_JTC
class TestHoldingState : public JTCThread, public JTCMonitor
{
    Test_var test_;

public:

    enum State
    {
        NONE,
        CALL_STARTED,
        CALL_FAILURE,
        CALL_SUCCESS
    };

private:

    State state_;

    void setState(State val)
    {
        JTCSynchronized sync(*this);
        state_ = val;
    }

public:

    TestHoldingState(Test_ptr test)
        : test_(Test::_duplicate(test)),
          state_(NONE)
    {
    }

    virtual void
    run()
    {
        setState(CALL_STARTED);
        try
        {
            test_ -> aMethod();
        }
        catch(const CORBA::TRANSIENT&)
        {
            setState(CALL_FAILURE);
            return;
        }
	catch(const CORBA::SystemException& ex)
	{
	    OB_CERR << "Unexpected: " << ex << OB_ENDL;
	    abort();
	}
        setState(CALL_SUCCESS);
    }

    State callState()
    {
        JTCSynchronized sync(*this);
        return state_;
    }
};

typedef JTCHandleT<TestHoldingState> TestHoldingStateHandle;
#endif

void
TestPOAManagerCommon(POAManagerProxy_ptr manager, const TestInfoSeq& info)
{
    for(ULong i = 0 ; i < info.length() ; i++)
    {
        manager -> activate();

        info[i].obj -> aMethod();

        try
        {
            manager -> discard_requests(false);
        }
        catch(const POAManagerProxy::AdapterInactive&)
        {
            TEST(false);
        }

        try
        {
            info[i].obj -> aMethod();
        }
        catch(const TRANSIENT&)
        {
	    // Expected
        }
        
        manager -> activate();
        info[i].obj -> aMethod();

        //
        // If JTC was used the run a test for hold_requests
        //
#ifdef HAVE_JTC
        manager -> hold_requests(false);

        TestHoldingStateHandle t = new TestHoldingState(info[i].obj);
        t -> start();

        //
        // Wait for the call to start
        //
	try
	{
	    JTCThread::sleep(500);
	}
	catch(const JTCInterruptedException&)
	{
	    // Ignore
	}

        TEST(t -> callState() == TestHoldingState::CALL_STARTED);

        manager -> activate();

        //
        // Wait for the call to complete
        //
	try
	{
	    JTCThread::sleep(500);
	}
	catch(const JTCInterruptedException&)
	{
	    // Ignore
	}

        TEST(t -> callState() == TestHoldingState::CALL_SUCCESS);
#endif 
    }
}
