// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <stdio.h>

#include <OB/CORBA.h>
#include <TestMacro.h>

#include <Constants.h>

using namespace CORBA;

void
TestConst()
{
    {
        const Measurement constEnum = FURLONGS;
	TEST(constEnum == ConstEnum);

	const Long constLong = -1234;
	TEST(constLong == ConstLong);
	
	const LongLong constLongLong = constLong * constLong;
	TEST(constLongLong == ConstLongLong);
	
	const LongLong constLongLongMin = OB_LONGLONG(-9223372036854775808);
	TEST(constLongLongMin == ConstLongLongMin);

	const LongLong constLongLongMax = OB_LONGLONG(9223372036854775807);
	TEST(constLongLongMax == ConstLongLongMax);
	
	const ULong constULong = (5432 + constLong ) * 100 % 124;
	TEST(constULong == ConstULong);
	
	const ULongLong constULongLong = constULong + (constULong << 20);
	TEST(constULongLong == ConstULongLong);
	
	const ULongLong constULongLongMax = OB_ULONGLONG(18446744073709551615);
	TEST(constULongLongMax == ConstULongLongMax);
	
	const Double constDouble = 50.23 - 1532.718 * 0.029;
	TEST(constDouble == ConstDouble);
	
	const Float constFloat = (Float)(constDouble * 1.11);
//
// Visual C++ fails on this test. If I put a print statement for constFloat
// and ConstFloat, the test succeeds. So I assume this is a Visual C++ bug.
//
#ifndef HAVE_VCPLUSPLUS_BUGS
	TEST(constFloat == ConstFloat);
#endif
	
	const Short constShort = -23 % 10;
	TEST(constShort == ConstShort);
	
	const Boolean constBoolean = true;
	TEST(constBoolean == ConstBoolean);
	
	const Octet constOctet = 254;
	TEST(constOctet == ConstOctet);
	
        const Fixed constFixed = Fixed("275.189") * Fixed("1.163") +
                                 Fixed("48.0093");
        TEST(constFixed == ConstFixed);

	const char* constString = "This is ConstString";
	TEST(strcmp(constString, ConstString) == 0);

	const char* constEmptyString = "";
	TEST(strcmp(constEmptyString, ConstEmptyString) == 0);

	const wchar_t* constWString = L"This is ConstWString";
	TEST(OB::wcscmp(constWString, ConstWString) == 0);

	const wchar_t* constEmptyWString = L"";
	TEST(OB::wcscmp(constEmptyWString, ConstEmptyWString) == 0);
    }

    {
	const Long constLong = 12345678;
	TEST(constLong == TestConstModule::ConstLong);
	
	const Double constDouble = ConstDouble / 2.0;
	TEST(constDouble == TestConstModule::ConstDouble);
	
	const Boolean constBoolean = false;
	TEST(constBoolean == TestConstModule::ConstBoolean);
	
	const char* constString = "This is ConstString in a module";
	TEST(strcmp(constString, TestConstModule::ConstString) == 0);

	const wchar_t* constWString = L"This is ConstWString in a module";
	TEST(OB::wcscmp(constWString, TestConstModule::ConstWString) == 0);

        const Measurement constEnum = METERS;
	TEST(constEnum == TestConstModule::ConstEnum);
    }

    {
	const Long constLong = (0xf | 0xf000) & 0xfffe;
	TEST(constLong == TestConstInterface::ConstLong);
	
	const ULong constULong = 0xFFFFFFFFU;
	TEST(constULong == TestConstInterface::ConstULong);
	
	const Char constChar0 = (Char)0;
	TEST(constChar0 == TestConstInterface::ConstChar0);
	
	const Char constChar1 = (Char)'c';
	TEST(constChar1 == TestConstInterface::ConstChar1);
	
	const Char constChar2 = (Char)'\n';
	TEST(constChar2 == TestConstInterface::ConstChar2);
	
	const Char constChar3 = (Char)'\377';
	TEST(constChar3 == TestConstInterface::ConstChar3);
	
	const Char constChar4 = (Char)'\x88';
	TEST(constChar4 == TestConstInterface::ConstChar4);
	
	const Char constChar5 = (Char)'\'';
	TEST(constChar5 == TestConstInterface::ConstChar5);
	
	const WChar constWChar = L'Z';
	TEST(constWChar == TestConstInterface::ConstWChar);

	const char* constString = "\n\t\v\b\r\f\a\\\?\'\"\xff\377\7";
	TEST(strcmp(constString, TestConstInterface::ConstString) == 0);

	const wchar_t* constWString = L"\n\t\v\b\r\f\a\\\?\'\"\xff\377\7";
	TEST(OB::wcscmp(constWString, TestConstInterface::ConstWString) == 0);

        const Measurement constEnum = FEET;
	TEST(constEnum == TestConstInterface::ConstEnum);
    }
}
