// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CosProperty.h>

//
// Tests for initial PropertySetDef
//
void
TestInitial(CosPropertyService::PropertySetDefFactory_ptr factory)
{
    CosPropertyService::PropertySetDef_var set;
    CORBA::Any any;

    //
    // Set up initial PropertySetDef
    //
    any <<= "initial_property_value";
    CosPropertyService::PropertyDefs initialProperties;
    CosPropertyService::PropertyDef p;
    p.property_name = CORBA::string_dup("initial_property");
    p.property_value = any;
    p.property_mode = CosPropertyService::normal;
    initialProperties.length(1);
    initialProperties[0] = p;

    set = factory -> create_initial_propertysetdef(initialProperties);

    {
	OB_COUT << "  Testing get_number_of_properties()... "
		     << OB_FLUSH;

	assert(set -> get_number_of_properties() == 1);

	OB_COUT << "OK!" << OB_ENDL;
    }

    {
	OB_COUT << "  Testing is_property_defined()... " << OB_FLUSH;

	try
	{
	    assert(set -> is_property_defined("initial_property"));
	}
	catch(const CosPropertyService::InvalidPropertyName&)
	{
	    assert(false);
	}

	try
	{
	    assert(!set -> is_property_defined("illegal_property"));
	}
	catch(const CosPropertyService::InvalidPropertyName&)
	{
	    assert(false);
	}

	OB_COUT << "OK!" << OB_ENDL;
    }

    {
	OB_COUT << "  Testing get_property_value()... " << OB_FLUSH;

	CORBA::Any_var any;

	try
	{
	    any = set -> get_property_value("initial_property");
	}
	catch(const CosPropertyService::InvalidPropertyName&)
	{
	    assert(false);
	}
	catch(const CosPropertyService::PropertyNotFound&)
	{
	    assert(false);
	}

	const char* value;
	any >>= value;
	assert(!strcmp(value, "initial_property_value"));

	OB_COUT << "OK!" << OB_ENDL;
    }
}
